##--------------------------------------------------------------------------------------------------------
## PROJECT : The effect of spatial resolution on SDM efficiency
## SCRIPT : Define transect design
##
## Author : Charlotte Lambert
## Last update : Oct 2021
## R version 4.0.4 (2021-02-15) -- "Lost Library Book"
##--------------------------------------------------------------------------------------------------------

library(rgdal)
library(rgeos)
options(stringsAsFactors = FALSE)
library(dplyr)
library(tidyverse)
library(raster)
library(maps)
library(maptools)
library(ggplot2)
library(ggthemes)
library(ggspatial)
library(ggpubr)
library(RColorBrewer)
library(reshape2)
library(lubridate)
library(viridis)
library(ncdf4)
library(ggspatial)
library(dssd)
library(sf)

#### ----------------------- ####
#### ---- SOURCES FILES ---- ####
#### ----------------------- ####
sourcedir <- "F:/Project_Spatial_Resolution/1_Source/GIS/"
setwd(sourcedir)

### 1 - Open land --------------------------------------------------------------
land_ENA <- readOGR(dsn = "~/1_Source/GIS/Land_ENA_EPSG2154", layer = "LAND")
land_WIO <- readOGR(dsn = "~/1_source/GIS/Land_WIO_EPSG2737", layer = "LAND")


### 2 - Open WE files ----------------------------------------------------------
## bounding box
bbx_ENA_wgs <- st_read(dsn = "~/1_Source/GIS", layer = "BoundingBox_ENA")
bbx_ENA <- bbx_ENA_wgs %>% st_transform(crs = st_crs(2154))

## clip land
land_ENA <- land_ENA %>% st_as_sf() %>% st_transform(crs = st_crs(2154)) %>% st_intersection(bbx_ENA)

## strata
strata_ENA_wgs <- st_read(dsn = "~/1_Source/GIS", layer = "Strate_ENA", type = 3)
strata_ENA <- strata_ENA_wgs %>% st_transform(crs = st_crs(2154)) #%>% st_difference(land_ENA) # retire le difference car crer des sous-multipolygone qui font buguer ensuite make.region


### 3 - Open bounding box IO ---------------------------------------------------
## bounding box
bbx_WIO_wgs <- st_read(dsn = "~/1_Source/GIS", layer = "BoundingBox_WIO")
bbx_WIO <- bbx_WIO_wgs  %>% st_transform(crs = st_crs(2737))

## clip land
land_WIO <- land_WIO %>% st_as_sf()  %>% st_intersection(st_as_sf(bbx_WIO_wgs)) %>% st_transform(crs = st_crs(2737))

## strata
strata_WIO_wgs <- st_read(dsn = "~/1_Source/GIS", layer = "Strate_WIO", type = 3)
strata_WIO <- strata_WIO_wgs %>% st_transform(crs = st_crs(2737)) #%>% st_difference(land_WIO)




#### ------------------------- ####
#### ---- GENERATE DESIGN ---- ####
#### ------------------------- ####

### 1 - Western Europe ---------------------------------------------------------
region_ENA <- make.region(region.name = "Eastern North Atlantic", 
                      strata.name = strata_ENA$Nom,
                      shape = strata_ENA)
plot(region_ENA)

cover_ENA <- make.coverage(region_ENA, n.grid.points = 1000)

design_ENA <- make.design(region = region_ENA, 
                      transect.type = "line",
                      design = "systematic",
                      samplers = 300,
                      # line.length = 100000,
                      edge.protocol = "minus",
                      design.angle = c(135,45,0,90,90,90,90,135,90,90,90,90,145,90),
                      truncation = 700,
                      coverage.grid = cover_ENA)

transects_ENA <- generate.transects(design_ENA)
plot(region_ENA, transects_ENA)
transects_ENA

plot(run.coverage(design_ENA, reps = 100))


### 2 - Mozambique Channel -------------------------------------------------------
region_WIO <- make.region(region.name = "Western Indian Ocean", 
                         strata.name = strata_WIO$Nom,
                         shape = strata_WIO)
plot(region_WIO)

cover_WIO <- make.coverage(region_WIO, n.grid.points = 2000)

design_WIO <- make.design(region = region_WIO, 
                         transect.type = "line",
                         design = "systematic",
                         samplers = 300,
                         # line.length = 100000,
                         edge.protocol = "minus",
                         design.angle = c(90,45,135,135,135,45,135,135,135,45,90,90,90,0),
                         truncation = 700,
                         coverage.grid = cover_WIO)

transects_WIO <- generate.transects(design_WIO)
plot(region_WIO, transects_WIO)

transects_WIO

plot(run.coverage(design_WIO, reps = 100))


### 3 - Clip on land -----------------------------------------------------------
## strata
strata_ENA_clipped <- strata_ENA %>% st_difference(land_ENA)
strata_WIO_clipped <- strata_WIO %>% st_difference(land_WIO)

# transects
transects_ENA_def <- transects_ENA@samplers %>% st_difference(land_ENA)
transects_WIO_def <- transects_WIO@samplers %>% st_difference(land_WIO)



### 4 - Save -------------------------------------------------------------------

write.transects(transects_ENA_def, dsn = paste(sourcedir, "Transects_ENA_EPSG2154.shp", sep = "/"))
write.transects(transects_WIO_def, dsn = paste(sourcedir, "Transects_WIO_EPSG2737.shp", sep = "/"))

write_sf(strata_ENA_clipped, dsn = paste(sourcedir, "Strata_ENA_clipped_EPSG2154.shp", sep = "/"))
write_sf(strata_WIO_clipped, dsn = paste(sourcedir, "Strata_WIO_clipped_EPSG2737.shp", sep = "/"))

write_sf(land_ENA, dsn = paste(sourcedir, "Land_ENA_EPSG2154.shp", sep = "/"))
write_sf(land_WIO, dsn = paste(sourcedir, "Land_WIO_EPSG2737.shp", sep = "/"))

