##--------------------------------------------------------------------------------------------------------
## PROJECT : The effect of spatial resolution on SDM efficiency
## SCRIPT : Segment transects
##
## Author : Charlotte Lambert
## Last update : Nov 2021
## R version 4.0.4 (2021-02-15) -- "Lost Library Book"
##--------------------------------------------------------------------------------------------------------

library(rgdal)
library(rgeos)
options(stringsAsFactors = FALSE)
library(dplyr)
library(tidyverse)
library(raster)
library(maps)
library(maptools)
library(ggplot2)
library(ggthemes)
library(ggspatial)
library(ggpubr)
library(RColorBrewer)
library(reshape2)
library(lubridate)
library(viridis)
library(ncdf4)
library(sf)
library(virtualspecies)


#### ----------------------- ####
#### ---- SOURCES FILES ---- ####
#### ----------------------- ####
sourcedir <- "~/1_source/GIS/"

### 1 - Open WE files ----------------------------------------------------------
bbx_ENA <- st_read(dsn = "~/1_source/GIS", layer = "BoundingBox_ENA")
land_ENA <- st_read(dsn = "~/1_source/GIS", layer = "Land_ENA_EPSG2154")
strata_ENA <- st_read(dsn = "~/1_source/GIS", layer = "Strata_ENA_clipped_EPSG2154", type = 3)
transects_ENA <- st_read(dsn = "~/1_source/GIS", layer = "Transects_ENA_EPSG2154", type = 3) 


### 2 - Open IO files ----------------------------------------------------------
bbx_WIO <- st_read(dsn = "~/1_source/GIS", layer = "BoundingBox_WIO")
land_WIO <- st_read(dsn = "~/1_source/GIS", layer = "Land_WIO_EPSG2737") 
strata_WIO <- st_read(dsn = "~/1_source/GIS", layer = "Strata_WIO_clipped_EPSG2737", type = 3)
transects_WIO <- st_read(dsn = "~/1_source/GIS", layer = "Transects_WIO_EPSG2737", type = 3) 


#### ------------------------ ####
#### ---- Western Europe ---- ####
#### ------------------------ ####

### 1 - Calculate transect length
transects_ENA$length <- st_length(transects_ENA)


### 2 - Segmentation to 10km
## functions
CreateSegment <- function(coords, from, to) {
  distance <- 0
  coordsOut <- c()
  biggerThanFrom <- F
  for (i in 1:(nrow(coords) - 1)) {
    d <- sqrt((coords[i, 1] - coords[i + 1, 1])^2 + (coords[i, 2] - coords[i + 1, 2])^2)
    distance <- distance + d
    if (!biggerThanFrom && (distance > from)) {
      w <- 1 - (distance - from)/d
      x <- coords[i, 1] + w * (coords[i + 1, 1] - coords[i, 1])
      y <- coords[i, 2] + w * (coords[i + 1, 2] - coords[i, 2])
      coordsOut <- rbind(coordsOut, c(x, y))
      biggerThanFrom <- T
    }
    if (biggerThanFrom) {
      if (distance > to) {
        w <- 1 - (distance - to)/d
        x <- coords[i, 1] + w * (coords[i + 1, 1] - coords[i, 1])
        y <- coords[i, 2] + w * (coords[i + 1, 2] - coords[i, 2])
        coordsOut <- rbind(coordsOut, c(x, y))
        break
      }
      coordsOut <- rbind(coordsOut, c(coords[i + 1, 1], coords[i + 1, 2]))
    }
  }
  return(coordsOut)
}

CreateSegments <- function(coords, length = 0, n.parts = 0) {
  stopifnot((length > 0 || n.parts > 0))
  # calculate total length line
  total_length <- 0
  for (i in 1:(nrow(coords) - 1)) {
    d <- sqrt((coords[i, 1] - coords[i + 1, 1])^2 + (coords[i, 2] - coords[i + 1, 2])^2)
    total_length <- total_length + d
  }
  
  # calculate stationing of segments
  if (length > 0) {
    stationing <- c(seq(from = 0, to = total_length, by = length), total_length)
  } else {
    stationing <- c(seq(from = 0, to = total_length, length.out = n.parts), 
                    total_length)
  }
  
  # calculate segments and store the in list
  newlines <- list()
  for (i in 1:(length(stationing) - 1)) {
    newlines[[i]] <- CreateSegment(coords, stationing[i], stationing[i + 1])
  }
  return(newlines)
}

MergeLast <- function(lst) {
  l <- length(lst)
  lst[[l - 1]] <- rbind(lst[[l - 1]], lst[[l]])
  lst <- lst[1:(l - 1)]
  return(lst)
}

SegmentSpatialLines <- function(sl, length = 0, n.parts = 0, merge.last = FALSE) {
  stopifnot((length > 0 || n.parts > 0))
  id <- 0
  newlines <- list()
  sl <- as(sl, "SpatialLines")
  for (lines in sl@lines) {
    for (line in lines@Lines) {
      crds <- line@coords
      # create segments
      segments <- CreateSegments(coords = crds, length, n.parts)
      if (merge.last && length(segments) > 1) {
        # in case there is only one segment, merging would result into error
        segments <- MergeLast(segments)
      }
      # transform segments to lineslist for SpatialLines object
      for (segment in segments) {
        newlines <- c(newlines, Lines(list(Line(unlist(segment))), ID = as.character(id)))
        id <- id + 1
      }
    }
  }
  return(SpatialLines(newlines))
}

Merge <- function(segments) {
  
  n <- as.numeric(nrow(segments))
  
  if (n > 2 && as.numeric(st_length(segments$geometry[n])) < 3000){
    
    corr <- segments[(n-1):n,] %>% summarize(geometry = st_union(geometry)) 
    
    segments_corr <- rbind(segments[1:(n-2),], corr)
    
    return(segments_corr)
    
  }
  
  else if (n == 2 && as.numeric(st_length(segments$geometry[n])) < 3000){
    
    corr <- segments[(n-1):n,] %>% summarize(geometry = st_union(geometry)) 
    
    segments_corr <- corr
    
    return(segments_corr)
    
  }
  
  else {
    
    return(segments)
  }
  
} 


## segment
segments_ENA <- NULL

idTransects <- unique(transects_ENA$transect)

for(i in 1:length(idTransects)){
  
  trans <- transects_ENA %>% subset(transect == idTransects[i]) %>%
              as_Spatial()
  seg <- SegmentSpatialLines(trans, length = 10000, merge.last = FALSE)
  seg <- st_as_sf(seg)
  st_crs(seg) <- 2154
  
  seg <- Merge(seg)
  
  seg_df <- st_as_sf(
    data.frame(
      id = 1:nrow(seg),
      Shape = seg,
      transect = trans$transect,
      strata = trans$strata,
      TransLengthKm = as.numeric(round(trans$length,0)/1000),
      SegLengthKm = as.numeric(round(st_length(seg),0)/1000)
    )) 
  seg_df <- seg_df %>% 
    mutate(IdSeg = qdapTools::id(seg_df, prefix= paste(trans$transect, "_Seg_", sep="")))
  
  segments_ENA <- rbind(segments_ENA, seg_df)
}



### 3 - Retrieve centroid coordinates for segments
## projected coordinates
segments_ENA <- segments_ENA %>%   
  mutate(x = do.call(rbind, st_geometry(st_centroid(segments_ENA)))[,1],  ## or unlist(lapply( st_geometry(st_centroid(tableIdSeg)) , "[[", 1))
         y = do.call(rbind, st_geometry(st_centroid(segments_ENA)))[,2]) 

## wgs coordinates
segments_ENA <- segments_ENA %>% st_transform(crs = st_crs(4326))
segments_ENA <- segments_ENA %>%   
  mutate(Longitude = do.call(rbind, st_geometry(st_centroid(segments_ENA)))[,1],  ## or unlist(lapply( st_geometry(st_centroid(tableIdSeg)) , "[[", 1))
         Latitude = do.call(rbind, st_geometry(st_centroid(segments_ENA)))[,2]) 

plot(st_geometry(segments_ENA), axes = T)


### 4 - Export file
segments_ENA$SegID <- as.integer(rownames(segments_ENA)) 
segments_ENA <- arrange(segments_ENA, SegID)
segments_ENA <- segments_ENA %>% st_transform(crs = st_crs(2154))
write_sf(segments_ENA, dsn = paste(sourcedir, "Segments_ENA_EPSG2154.shp", sep = "/"))



#### ---------------------- ####
#### ---- Indian Ocean ---- ####
#### ---------------------- ####

### 1 - Calculate transect length
transects_WIO$length <- st_length(transects_WIO)


### 2 - Segmentation to 10km
## segment
segments_WIO <- NULL

idTransects <- unique(transects_WIO$transect)

for(i in 1:length(idTransects)){
  
  trans <- transects_WIO %>% subset(transect == idTransects[i]) %>%
    as_Spatial()
  seg <- SegmentSpatialLines(trans, length = 10000, merge.last = FALSE)
  seg <- st_as_sf(seg)
  st_crs(seg) <- 2737
  
  seg <- Merge(seg)
  
  seg_df <- st_as_sf(
    data.frame(
      id = 1:nrow(seg),
      Shape = seg,
      transect = trans$transect,
      strata = trans$strata,
      TransLengthKm = as.numeric(round(trans$length,0)/1000),
      SegLengthKm = as.numeric(round(st_length(seg),0)/1000)
    )) 
  seg_df <- seg_df %>% 
    mutate(IdSeg = qdapTools::id(seg_df, prefix= paste(trans$transect, "_Seg_", sep="")))
  
  segments_WIO <- rbind(segments_WIO, seg_df)
}



### 3 - Retrieve centroid coordinates for segments
## projected coordinates
segments_WIO <- segments_WIO %>%   
  mutate(x = do.call(rbind, st_geometry(st_centroid(segments_WIO)))[,1],  ## or unlist(lapply( st_geometry(st_centroid(tableIdSeg)) , "[[", 1))
         y = do.call(rbind, st_geometry(st_centroid(segments_WIO)))[,2]) 

## wgs coordinates
segments_WIO <- segments_WIO %>% st_transform(crs = st_crs(4326))
segments_WIO <- segments_WIO %>%   
  mutate(Longitude = do.call(rbind, st_geometry(st_centroid(segments_WIO)))[,1],  ## or unlist(lapply( st_geometry(st_centroid(tableIdSeg)) , "[[", 1))
         Latitude = do.call(rbind, st_geometry(st_centroid(segments_WIO)))[,2]) 

plot(st_geometry(segments_WIO), axes = T)


### 4 - Export file
segments_WIO$SegID <- as.integer(rownames(segments_WIO)) 
segments_WIO <- arrange(segments_WIO, SegID)
segments_WIO <- segments_WIO %>% st_transform(crs = st_crs(2737))
write_sf(segments_WIO, dsn = paste(sourcedir, "Segments_WIO_EPSG2737.shp", sep = "/"))




