##--------------------------------------------------------------------------------------------------------
## PROJECT : The effect of spatial resolution on SDM efficiency
## SCRIPT : Exploration and formatting of environmental variables
##
## Author : Charlotte Lambert
## Last update : Oct 2021
## R version 4.0.4 (2021-02-15) -- "Lost Library Book"
##--------------------------------------------------------------------------------------------------------

library(sp)
library(sf)
library(dplyr)
library(tidyverse)
library(raster)
library(maps)
library(maptools)
library(ggplot2)
library(ggthemes)
library(ggspatial)
library(ggpubr)
library(RColorBreENAr)
library(reshape2)
library(lubridate)
library(viridis)
library(ncdf4)


#### ----------------------- ####
#### ---- SOURCES FILES ---- ####
#### ----------------------- ####
sourcedir <- "F:/Project_Spatial_Resolution/1_source/Variables"
setwd(sourcedir)

### 1 - Open ENA files ----------------------------------------------------------
bbx_ENA <- st_read(dsn = "~/1_source/GIS", layer = "BoundingBox_ENA")
land_ENA <- st_read(dsn = "~/1_source/GIS", layer = "Land_ENA_EPSG2154") 
strata_ENA <- st_read(dsn = "~/1_source/GIS", layer = "Strata_ENA_clipped_EPSG2154", type = 3)

### 2 - Open bounding box WIO ---------------------------------------------------
bbx_WIO <- st_read(dsn = "~/1_source/GIS", layer = "BoundingBox_WIO")
land_WIO <- st_read(dsn = "~/1_source/GIS", layer = "Land_WIO_EPSG2737") 
strata_WIO <- st_read(dsn = "~/1_source/GIS", layer = "Strata_WIO_clipped_EPSG2737", type = 3) 

### 4 - Open bathy
bathy_WIO <- raster("~/GEBCO/gebco_2020_geotiff/gebco_2020_n0.0_s-90.0_w0.0_e90.0.tif")
bathy_ENA1 <- raster("~/GEBCO/gebco_2020_geotiff/gebco_2020_n90.0_s0.0_w-90.0_e0.0.tif")
bathy_ENA2 <- raster("~/GEBCO/gebco_2020_geotiff/gebco_2020_n90.0_s0.0_w0.0_e90.0.tif")
bathy <- raster::merge(bathy_WIO, bathy_ENA1, bathy_ENA2)
# bathy <- crop(bathy, extent(-4.8,0.5,44.3,47.1))

### 5 - Generate terrain layers
slope <- terrain(bathy, opt = "slope", neighbors = 8, unit = "degrees")


#### ----------------------------------------- ####
#### ---- PROCESS SURFACE COPERNICUS DATA ---- ####
#### ----------------------------------------- ####

### 1 - Open the downloaded ncdf -----------------------------------------------
var_phy <- nc_open(paste(sourcedir, "/global-reanalysis-phy-001-030-daily_1635320728282.nc", sep = ""))
var_bio <- nc_open(paste(sourcedir, "/cmems_mod_glo_bgc_my_0.083deg-lmtl_PT1D-i_1635320620597.nc", sep = ""))

print(var_phy)
names(var_phy$var)
#  "mlotst" (MLD thickness) "siconc"  "thetao"(temp)  "usi"     "sithick" "bottomT" "vsi"     "vo" "uo" (currents) "so" (sal)  "zos"  (SSH)
print(var_bio)
names(var_bio$var)
#  "mnkc_umeso"   "zooc"         "mnkc_mumeso"  "mnkc_epi"     "mnkc_hmlmeso" "mnkc_mlmeso"  "zeu"          "mnkc_lmeso"   "npp"  

lon_var_phy <- ncvar_get(var_phy,"longitude") 
lat_var_phy <- ncvar_get(var_phy,"latitude") 
time_var_phy <- ncvar_get(var_phy,"time") 
depth_var_phy <- ncvar_get(var_phy,"depth") # dim(depth) = 1 # if dim(depth) != 1, add the depth dimension in the rest of the script

lon_var_bio <- ncvar_get(var_bio,"longitude") 
lat_var_bio <- ncvar_get(var_bio,"latitude") 
time_var_bio <- ncvar_get(var_bio,"time") 
depth_var_bio <- ncvar_get(var_bio,"depth") # dim(depth) = 1 # if dim(depth) != 1, add the depth dimension in the rest of the script


### 2 - Convert ncdf and save it into raster on disk ---------------------------
ncdfToRaster <- function(data, variable, xmin, xmax, ymin, ymax, lon, lat, crs, datadir,
                         nameRaster, gradient = FALSE, currentSpeed = FALSE, eke = FALSE) {
  
  # convert the time field into understandable field
  getNcTime <- function(nc) {
    require(units)
    require(ncdf4)
    options(warn=1) #show warnings by default
    if (is.character(nc)) nc <- nc
    ncdims <- names(nc$dim) #get netcdf dimensions
    timevar <- ncdims[which(ncdims %in% c("time", "Time", "datetime", "Datetime", "date", "Date"))] #find (first) time variable
    if (length(timevar) > 1) {
      warning(paste("Found more than one time var. Using the first:", timevar[1]))
      timevar <- timevar[1]
    }
    if (length(timevar)!=1) stop("ERROR! Could not identify the correct time variable")
    times <- ncvar_get(nc, timevar) #get time data
    timeatt <- ncatt_get(nc, timevar) #get attributes
    timeunit <- timeatt$units
    units(times) <- as_units(timeunit)
    as.POSIXct(times)
  }
  
  # save raster stack
  writeENVI <- function(x, f, ...) {
    writeRaster(x, f, overwrite=TRUE, ...)
    cat("band names = {", paste(names(x),collapse=","), "}", "\n", file=extension(f, "hdr"), append=TRUE)
  }
  
  if (gradient == FALSE & currentSpeed == FALSE & eke == FALSE) {
    
    array <- ncvar_get(data, variable) # store the data in a 3-dimensional array
    ras <- raster(t(array[,dim(array)[2]:1]), xmn = xmin, xmx = xmax ,ymn = ymin, ymx = ymax, crs = crs)
    names(ras) <- variable
    writeENVI(ras, paste(datadir, "/", nameRaster , "-daily-", substr(min(getNcTime(data)), start = 1, stop = 10), ".envi", sep =""))
    
  } else if (gradient == TRUE) {
    
    writeLines("Be sure you have chosen the right variable for which you want to calculate the gradient e.g. temperature, chlorophyll, salinity")
    
    array <- ncvar_get(data,variable) 
    ras <- raster(t(array[,dim(array)[2]:1]), xmn = xmin, xmx = xmax ,ymn = ymin, ymx = ymax, crs = crs)
    grad_raster <- grec::detectFronts(ras) 
    names(grad_raster) <- paste("grad", variable, sep = "_")
    writeENVI(grad_raster, paste(datadir, "/", nameRaster, "-daily-", substr(min(getNcTime(data)), start = 1, stop = 10), ".envi", sep =""))
    
  } else if (currentSpeed == TRUE) {
    
    if (length(variable) == 1) {
      
      stop("Be sure you have chosen the two current components (U and V) as variables to calculate the current speed of the currents or the eddy kinetic energy.")
      
    } else {
      
      U.array <- ncvar_get(data, variable[1]) # store the data in a 3-dimensional array
      
      V.array <- ncvar_get(data, variable[2]) # store the data in a 3-dimensional array
      
      speed.array <- sqrt(U.array^2+V.array^2)
      
      ras <- raster(t(speed.array[,dim(speed.array)[2]:1]), xmn = xmin, xmx = xmax ,ymn = ymin, ymx = ymax, crs = "+proj=longlat +datum=WGS84")
      names(ras) <- "CurrentSpeed"
      
      writeENVI(ras, paste(datadir, "/", nameRaster, "-daily-", substr(min(getNcTime(data)), start = 1, stop = 10), ".envi", sep =""))
    }
    
  } else if (eke == TRUE) {
    
    if (length(variable) ==1) {
      
      stop("Be sure you have chosen the two current components (U and V) as variables to calculate the current speed of the currents or the eddy kinetic energy.")
      
    } else {
      
      U.array <- ncvar_get(data, variable[1]) # store the data in a 3-dimensional array
      
      V.array <- ncvar_get(data, variable[2]) # store the data in a 3-dimensional array
      
      eke.array <- 0.5*(U.array^2+V.array^2)
      
      ras <- raster(t(eke.array[,dim(eke.array)[2]:1]), xmn = xmin, xmx = xmax ,ymn = ymin, ymx = ymax, crs = "+proj=longlat +datum=WGS84")
      names(ras) <- "EKE"
      
      writeENVI(ras, paste(datadir, "/", nameRaster, "-daily-", substr(min(getNcTime(data)), start = 1, stop = 10), ".envi", sep =""))
      
    }
    
  }
}

ncdfToRaster(var_bio, "npp", xmin = -24, xmax = 63, ymin = -50, ymax = 70, datadir = paste(sourcedir, "/native_0083"),
             lon = lon_var_bio, lat = lat_var_bio, crs = "+proj=longlat +datum=WGS84", nameRaster = "NPP")
ncdfToRaster(var_bio, "npp", xmin = -24, xmax = 63, ymin = -50, ymax = 70, datadir = paste(sourcedir, "/native_0083"),
             lon = lon_var_bio, lat = lat_var_bio, crs = "+proj=longlat +datum=WGS84", nameRaster = "gradNPP", gradient = T)
ncdfToRaster(var_bio, "zeu", xmin = -24, xmax = 63, ymin = -50, ymax = 70, datadir = paste(sourcedir, "/native_0083"),
             lon = lon_var_bio, lat = lat_var_bio, crs = "+proj=longlat +datum=WGS84", nameRaster = "ZEu")


ncdfToRaster(var_phy,"thetao", xmin = -24, xmax = 63, ymin = -50, ymax = 70, datadir = paste(sourcedir, "/native_0083"),
             lon = lon_var_phy, lat = lat_var_phy, crs = "+proj=longlat +datum=WGS84", nameRaster = "Temp")
ncdfToRaster(var_phy,"thetao", xmin = -24, xmax = 63, ymin = -50, ymax = 70,datadir = paste(sourcedir, "/native_0083"), 
             lon = lon_var_phy, lat = lat_var_phy, crs = "+proj=longlat +datum=WGS84", nameRaster = "gradTemp", gradient = TRUE)
ncdfToRaster(var_phy,variable = c("uo", "vo"), xmin = -24, xmax = 63, ymin = -50, ymax = 70, datadir = paste(sourcedir, "/native_0083"),
             lon = lon_var_phy, lat = lat_var_phy, crs = "+proj=longlat +datum=WGS84", nameRaster = "CurrentSpeed", currentSpeed = TRUE)
ncdfToRaster(var_phy,variable = c("uo", "vo"), xmin = -24, xmax = 63, ymin = -50, ymax = 70, datadir = paste(sourcedir, "/native_0083"),
             lon = lon_var_phy, lat = lat_var_phy, crs = "+proj=longlat +datum=WGS84", nameRaster = "EKE", eke = TRUE)
ncdfToRaster(var_phy,"so", xmin = -24, xmax = 63, ymin = -50, ymax = 70, datadir = paste(sourcedir, "/native_0083"),
             lon = lon_var_phy, lat = lat_var_phy, crs = "+proj=longlat +datum=WGS84", nameRaster = "Salinity")
ncdfToRaster(var_phy,"so", xmin = -24, xmax = 63, ymin = -50, ymax = 70, datadir = paste(sourcedir, "/native_0083"),
             lon = lon_var_phy, lat = lat_var_phy, crs = "+proj=longlat +datum=WGS84", nameRaster = "gradSal", gradient = TRUE)
ncdfToRaster(var_phy,"zos", xmin = -24, xmax = 63, ymin = -50, ymax = 70, datadir = paste(sourcedir, "/native_0083"),
             lon = lon_var_phy, lat = lat_var_phy, crs = "+proj=longlat +datum=WGS84", nameRaster = "SSH")
ncdfToRaster(var_phy,"zos", xmin = -24, xmax = 63, ymin = -50, ymax = 70, datadir = paste(sourcedir, "/native_0083"),
             lon = lon_var_phy, lat = lat_var_phy, crs = "+proj=longlat +datum=WGS84", nameRaster = "gradSSH", gradient = T)
ncdfToRaster(var_phy,"mlotst", xmin = -24, xmax = 63, ymin = -50, ymax = 70, datadir = paste(sourcedir, "/native_0083"),
             lon = lon_var_phy, lat = lat_var_phy, crs = "+proj=longlat +datum=WGS84", nameRaster = "MLD")
ncdfToRaster(var_phy,"mlotst", xmin = -24, xmax = 63, ymin = -50, ymax = 70, datadir = paste(sourcedir, "/native_0083"),
             lon = lon_var_phy, lat = lat_var_phy, crs = "+proj=longlat +datum=WGS84", nameRaster = "gradMLD", gradient = T)


save.image(paste(sourcedir, "Covariates.RData", sep = "/"))



### 3 - Re-open rasters --------------------------------------------------------
var_name <- c("Temp", "MLD", "ZEu", "EKE", "NPP", "SSH", "Salinity", "CurrentSpeed",
              "gradTemp", "gradSSH", "gradSal", "gradNPP", "gradMLD")

for(i in seq_along(var_name)){
  toto <- raster(paste(sourcedir, "/native_0083/", var_name[i], "-daily-2019-02-01.envi", sep =""))
  assign(var_name[i], toto)
}



### 4 - Generate stacks --------------------------------------------------------
writeENVI <- function(x, f, ...) {
  writeRaster(x, f, overwrite=TRUE, ...)
  cat("band names = {", paste(names(x),collapse=","), "}", "\n", file=extension(f, "hdr"), append=TRUE)
}

## resample bathymetry on the same grid than other variables
bathy2 <- resample(bathy, Temp)
bathy2 <- mask(bathy2, land, inverse = T)
bathy2[bathy2 >0] <- NA
slope2 <- resample(slope, Temp)


## stack variables, crop to study areas
stack <- stack(bathy2, slope2, Temp, SSH, NPP, MLD, ZEu, CurrentSpeed, EKE, gradTemp, gradSSH, gradMLD, gradNPP)
names(stack) <- c("Bathy", "Slope", "Temp", "SSH", "NPP", "MLD", "ZEu", "CurrentSpeed", "EKE", "gradTemp", "gradSSH", "gradMLD", "gradNPP")

stack_WIO <- crop(stack, bbx_WIO)
stack_ENA <- crop(stack, bbx_ENA)

writeENVI(stack_WIO, paste(sourcedir, "EnvironmentalVariables_WIO_0083.envi", sep = "/"))
writeENVI(stack_ENA, paste(sourcedir, "EnvironmentalVariables_ENA_0083.envi", sep = "/"))


## quantify variability of oceanographic conditions on the two study areas
# load Covariates_stack.RData
'%!in%' <- function(x,y)!('%in%'(x,y))
variability <- full_join(data.frame(sd = cellStats(stack_WIO, stat = "sd"),
                                    mean = cellStats(stack_WIO, stat = "mean"), 
                                    Region = "WIO" ) %>%
                           dplyr::as_tibble(rownames = "Variable"), 
                         data.frame(sd = cellStats(stack_ENA, stat = "sd"),
                                    mean = cellStats(stack_ENA, stat = "mean"), 
                                    Region = "ENA" ) %>%
                           dplyr::as_tibble(rownames = "Variable")) %>% 
  subset(Variable %!in% c("gradSSH", "gradTemp", "gradMLD", "gradNPP"))

ggplot(variability) + geom_pointrange(aes(y = mean, ymin = mean - sd, ymax = mean + sd, x = Region)) +
  facet_wrap("Variable", scales = "free") + theme_bw() + 
  theme(axis.title = element_blank(), 
        strip.background = element_blank(), 
        strip.text = element_text(face = "bold", hjust = 0, size = 12))
ggsave(paste0(sourcedir, "/Variables_variability.png"), dpi = 350, height = 8, width = 8)

ggplot(variability) + geom_col(aes(y = sd, x = Region), position = "dodge") +
  facet_wrap("Variable", scales = "free") + theme_bw()
ggsave(paste0(sourcedir, "/Variables_variability2.png"), dpi = 350, height = 8, width = 8)



### 5 - Aggregate to lower resolutions -----------------------------------------
## IO
stack_WIO_017 <- aggregate(stack_WIO, fact = 1.5, fun = mean)
stack_WIO_025 <- aggregate(stack_WIO, fact = 3, fun = mean)
stack_WIO_050 <- aggregate(stack_WIO, fact = 6, fun = mean)
stack_WIO_100 <- aggregate(stack_WIO, fact = 10, fun = mean)

writeENVI(stack_WIO_017, paste(sourcedir, "EnvironmentalVariables_WIO_017.envi", sep = "/"))
writeENVI(stack_WIO_025, paste(sourcedir, "EnvironmentalVariables_WIO_025.envi", sep = "/"))
writeENVI(stack_WIO_050, paste(sourcedir, "EnvironmentalVariables_WIO_050.envi", sep = "/"))
writeENVI(stack_WIO_100, paste(sourcedir, "EnvironmentalVariables_WIO_100.envi", sep = "/"))


## ENA
stack_ENA_017 <- aggregate(stack_ENA, fact = 1.5, fun = mean)
stack_ENA_025 <- aggregate(stack_ENA, fact = 3, fun = mean)
stack_ENA_050 <- aggregate(stack_ENA, fact = 6, fun = mean)
stack_ENA_100 <- aggregate(stack_ENA, fact = 10, fun = mean)

writeENVI(stack_ENA_017, paste(sourcedir, "EnvironmentalVariables_ENA_017.envi", sep = "/"))
writeENVI(stack_ENA_025, paste(sourcedir, "EnvironmentalVariables_ENA_025.envi", sep = "/"))
writeENVI(stack_ENA_050, paste(sourcedir, "EnvironmentalVariables_ENA_050.envi", sep = "/"))
writeENVI(stack_ENA_100, paste(sourcedir, "EnvironmentalVariables_ENA_100.envi", sep = "/"))



save.image(paste(sourcedir, "Covariates_stacks.RData", sep = "/"))
