##--------------------------------------------------------------------------------------------------------
## PROJECT : The effect of spatial resolution on SDM efficiency
## SCRIPT : Simulate species distribution
##
## Author : Charlotte Lambert
## Last update : Jan 2022
## R version 4.0.4 (2021-02-15) -- "Lost Library Book"
##--------------------------------------------------------------------------------------------------------

library(rgdal)
library(rgeos)
options(stringsAsFactors = FALSE)
library(dplyr)
library(tidyverse)
library(raster)
library(maps)
library(maptools)
library(ggplot2)
library(ggthemes)
library(ggspatial)
library(ggpubr)
library(RColorBrewer)
library(reshape2)
library(lubridate)
library(viridis)
library(ncdf4)
library(sf)
library(virtualspecies)


#### ----------------------- ####
#### ---- SOURCES FILES ---- ####
#### ----------------------- ####
sourcedir <- "~/1_Source/Variables/"
outdir <- "~/2_Species_simulations/"
setwd(outdir)

### 1 - Open WE files ----------------------------------------------------------
bbx_ENA <- st_read(dsn = "~/1_source/GIS", layer = "BoundingBox_ENA")
land_ENA <- st_read(dsn = "~/1_source/GIS", layer = "Land_ENA_EPSG2154") %>%
               st_transform(crs = st_crs(4326)) 
strata_ENA <- st_read(dsn = "~/1_source/GIS", layer = "Strata_ENA_clipped_EPSG2154", type = 3) %>%
   st_transform(crs = st_crs(4326))


### 2 - Open bounding box IO ---------------------------------------------------
bbx_WIO <- st_read(dsn = "~/1_source/GIS", layer = "BoundingBox_WIO")
land_WIO <- st_read(dsn = "~/1_source/GIS", layer = "Land_WIO_EPSG2737") %>%
   st_transform(crs = st_crs(4326)) 
strata_WIO <- st_read(dsn = "~/1_source/GIS", layer = "Strata_WIO_clipped_EPSG2737", type = 3) %>%
   st_transform(crs = st_crs(4326))


### 4 - Open environmental stacks ----------------------------------------------
stack_ENA_0083 <- stack(paste(sourcedir, "EnvironmentalVariables_ENA_0083.envi", sep = ""))
stack_ENA_017 <- stack(paste(sourcedir, "EnvironmentalVariables_ENA_017.envi", sep = ""))
stack_ENA_025 <- stack(paste(sourcedir, "EnvironmentalVariables_ENA_025.envi", sep = ""))
stack_ENA_050 <- stack(paste(sourcedir, "EnvironmentalVariables_ENA_050.envi", sep = ""))
stack_ENA_100 <- stack(paste(sourcedir, "EnvironmentalVariables_ENA_100.envi", sep = ""))

stack_WIO_0083 <- stack(paste(sourcedir, "EnvironmentalVariables_WIO_0083.envi", sep = ""))
stack_WIO_017 <- stack(paste(sourcedir, "EnvironmentalVariables_WIO_017.envi", sep = ""))
stack_WIO_025 <- stack(paste(sourcedir, "EnvironmentalVariables_WIO_025.envi", sep = ""))
stack_WIO_050 <- stack(paste(sourcedir, "EnvironmentalVariables_WIO_050.envi", sep = ""))
stack_WIO_100 <- stack(paste(sourcedir, "EnvironmentalVariables_WIO_100.envi", sep = ""))


#### ------------------------------- ####
#### ---- CORRELATION VARIABLES ---- ####
#### ------------------------------- ####

### 1 - Western Europe
GGally::ggcorr(data = as.data.frame(stack_ENA_0083[[1:9]]), geom = "text")
ggsave(paste(outdir, "Correlation_variables_ENA.png", sep = ""), device = "png", dpi = 350, width = 7, height = 5)

### 2 - Indian Ocean
GGally::ggcorr(data = as.data.frame(stack_WIO_0083[[1:9]]), geom = "text")
ggsave(paste(outdir, "Correlation_variables_WIO.png", sep = ""), device = "png", dpi = 350, width = 7, height = 5)



#### -------------------------- ####
#### ---- GENERATE SPECIES ---- ####
#### -------------------------- ####

### 1 - Define species ENA ------------------------------------------------------
## Sp 1
sp1_ENA_0083 <- generateSpFromFun(raster.stack = stack_ENA_0083[[c("Temp", "SSH", "ZEu", "NPP")]],
                        parameters = formatFunctions(Temp = c(fun = 'dcauchy', location = 14, scale = 10),
                                                     SSH = c(fun = "dlogis", location = -0.6, scale = 0.15),
                                                     ZEu = c(fun = "logisticFun", beta = 37, alpha = 6),
                                                     NPP = c(fun = "dlogis", location = 1500, scale = 2000)),
                        formula = "0.4*Temp + 0.3*ZEu + 0.15*SSH + 0.15*NPP",
                        plot = TRUE)
plotResponse(sp1_ENA_0083)


## Sp 2
sp2_ENA_025 <- generateSpFromFun(raster.stack = stack_ENA_025[[c("Temp", "Slope", "Bathy", "NPP")]],
                             parameters = formatFunctions(Temp = c(fun = 'dcauchy', location = 12, scale = 10),
                                                          Slope = c(fun = "logisticFun", beta = 10, alpha = -3),
                                                          Bathy = c(fun = "dcauchy", location = -600, scale = 1000),
                                                          NPP = c(fun = "dlogis", location = 1500, scale = 2000)),
                             formula = "0.4*Temp + 0.1*Bathy + 0.2*NPP + 0.3*Slope",
                             plot = TRUE)
plotResponse(sp2_ENA_025)


### 3 - Define species - WIO ----------------------------------------------------
## sp 1
sp1_WIO_025 <- generateSpFromFun(raster.stack = stack_WIO_025[[c("Temp", "SSH", "MLD", "NPP")]],
                                parameters = formatFunctions(Temp = c(fun = 'dcauchy', location = 25, scale = 10),
                                                             SSH = c(fun = "dlogis", location = 0.4, scale = 0.25),
                                                             MLD = c(fun = "logisticFun", beta = 30, alpha = 4),
                                                             NPP = c(fun = "dlogis", location = 4000, scale = 2500)),
                                formula = "0.4*SSH + 0.25*MLD + 0.2*Temp + 0.15*NPP",
                                plot = F)

## sp 2
sp2_WIO_0083 <- generateSpFromFun(raster.stack = stack_WIO_0083[[c("Temp", "CurrentSpeed", "ZEu", "Bathy")]],
                                 parameters = formatFunctions(Temp = c(fun = 'logisticFun', beta = 26, alpha = -1),
                                                              CurrentSpeed = c(fun = "dcauchy", location = 1.4, scale = 1.5),
                                                              ZEu = c(fun = "dlogis", location = 80, scale = 30),
                                                              Bathy = c(fun = "logisticFun", beta = -1000, alpha = 500)),
                                 formula = "0.5*Temp + 0.15*CurrentSpeed + 0.2*ZEu + 0.15*Bathy",
                                 plot = TRUE) #; plotResponse(sp2_WIO_0083)


### 4 - Plot curves ------------------------------------------------------------
## Plot curves
png(paste(outdir, "ENAsp0083_curves.png", sep = ""), height = 10, width = 16, units = "cm", res = 400)
plotResponse(sp1_ENA_0083)
dev.off()

png(paste(outdir, "ENAsp025_curves.png", sep = ""), height = 10, width = 16, units = "cm", res = 400)
plotResponse(sp2_ENA_025)
dev.off()

png(paste(outdir, "WIOsp025_curves.png", sep = ""), height = 10, width = 16, units = "cm", res = 400)
plotResponse(sp1_WIO_025)
dev.off()

png(paste(outdir, "WIOsp0083_curves.png", sep = ""), height = 10, width = 16, units = "cm", res = 400)
plotResponse(sp2_WIO_0083)
dev.off()



save.image("~/2_Species_simulations/Simulations.RData")




#### ---------------------------- ####
#### ---- CONVERT TO SPECIES ---- ####
#### ----- PRESENCE/ABSENCE ----- ####
#### ---------------------------- ####

### 1 - Select environment suitability maps
ENAsp0083 <- raster::mask(sp1_ENA_0083$suitab.raster, as_Spatial(strata_ENA))
ENAsp025 <- raster::mask(sp2_ENA_025$suitab.raster, as_Spatial(strata_ENA))

WIOsp0083 <- raster::mask(sp2_WIO_0083$suitab.raster, as_Spatial(strata_WIO))
WIOsp025 <- raster::mask(sp1_WIO_025$suitab.raster, as_Spatial(strata_WIO))

par(mfrow = c(2,2))
plot(ENAsp0083, legend = F, 
     col = viridis(20), addfun = c(plot(as_Spatial(land_ENA), col = "grey", add = T),
                                   plot(as_Spatial(strata_ENA), col = NA, add = T, lwd = 0.2)))
plot(ENAsp025, legend = F, 
     col = viridis(20), addfun = c(plot(as_Spatial(land_ENA), col = "grey", add = T),
                                   plot(as_Spatial(strata_ENA), col = NA, add = T, lwd = 0.2)))
plot(WIOsp0083, legend = F, 
     col = viridis(20), addfun = c(plot(as_Spatial(land_WIO), col = "grey", add = T),
                                   plot(as_Spatial(strata_WIO), col = NA, add = T, lwd = 0.2)))
plot(WIOsp025, legend = F, 
     col = viridis(20), addfun = c(plot(as_Spatial(land_WIO), col = "grey", add = T),
                                   plot(as_Spatial(strata_WIO), col = NA, add = T, lwd = 0.2)))



### 2 - Convert suitability to presence-absence maps
ENAsp0083_PA <- convertToPA(ENAsp0083, species.prevalence = 0.1)

ENAsp025_PA <- convertToPA(ENAsp025, species.prevalence = 0.1)

WIOsp0083_PA <- convertToPA(WIOsp0083, species.prevalence = 0.1)

WIOsp025_PA <- convertToPA(WIOsp025, species.prevalence = 0.1)


### 3 - Export Proba occurence and presence-absence maps
writeRaster(ENAsp0083_PA$suitab.raster, 
            filename = paste(outdir, "Species0083_ENA_Suitability.grd", sep = ""), format = "raster")
writeRaster(ENAsp025_PA$suitab.raster, 
            filename = paste(outdir, "Species025_ENA_Suitability.grd", sep = ""), format = "raster")
writeRaster(WIOsp0083_PA$suitab.raster, 
            filename = paste(outdir, "Species0083_WIO_Suitability.grd", sep = ""), format = "raster")
writeRaster(WIOsp025_PA$suitab.raster, 
            filename = paste(outdir, "Species025_WIO_Suitability.grd", sep = ""), format = "raster")

writeRaster(ENAsp0083_PA$probability.of.occurrence, 
            filename = paste(outdir, "Species0083_ENA_ProbOcc.grd", sep = ""), format = "raster")
writeRaster(ENAsp025_PA$probability.of.occurrence, 
            filename = paste(outdir, "Species025_ENA_ProbOcc.grd", sep = ""), format = "raster")
writeRaster(WIOsp0083_PA$probability.of.occurrence, 
            filename = paste(outdir, "Species0083_WIO_ProbOcc.grd", sep = ""), format = "raster")
writeRaster(WIOsp025_PA$probability.of.occurrence, 
            filename = paste(outdir, "Species025_WIO_ProbOcc.grd", sep = ""), format = "raster")

writeRaster(ENAsp0083_PA$pa.raster, 
            filename = paste(outdir, "Species0083_ENA_PresAbs.grd", sep = ""), format = "raster")
writeRaster(ENAsp025_PA$pa.raster, 
            filename = paste(outdir, "Species025_ENA_PresAbs.grd", sep = ""), format = "raster")
writeRaster(WIOsp0083_PA$pa.raster, 
            filename = paste(outdir, "Species0083_WIO_PresAbs.grd", sep = ""), format = "raster")
writeRaster(WIOsp025_PA$pa.raster, 
            filename = paste(outdir, "Species025_WIO_PresAbs.grd", sep = ""), format = "raster")


### 4 - Project
# occ
ENAsp0083_occ <- projectRaster(from = ENAsp0083_PA$probability.of.occurrence, 
                          to = raster(extent(st_transform(strata_ENA, crs = "epsg:2154")), 
                                      crs = crs(st_transform(strata_ENA, crs = "epsg:2154")), 
                                      res = c(12969.728, 12969.728)))
ENAsp025_occ <- projectRaster(from = ENAsp025_PA$probability.of.occurrence, 
                          to = raster(extent(st_transform(strata_ENA, crs = "epsg:2154")), 
                                      crs = crs(st_transform(strata_ENA, crs = "epsg:2154")), 
                                      res = c(22268.039, 22268.039)))

WIOsp0083_occ <- projectRaster(from = WIOsp0083_PA$probability.of.occurrence, 
                          to = raster(extent(st_transform(strata_WIO, crs = "epsg:2737")), 
                                      crs = crs(st_transform(strata_WIO, crs = "epsg:2737")), 
                                      res = c(8562.396, 8562.396)))
WIOsp025_occ <- projectRaster(from = WIOsp025_PA$probability.of.occurrence, 
                          to = raster(extent(st_transform(strata_WIO, crs = "epsg:2737")), 
                                      crs = crs(st_transform(strata_WIO, crs = "epsg:2737")), 
                                      res = c(27176.464, 27176.464)))

writeRaster(ENAsp0083_occ, 
            filename = paste(outdir, "Species0083_ENA_ProbOcc_EPSG2154.grd", sep = ""), format = "raster")
writeRaster(ENAsp025_occ, 
            filename = paste(outdir, "Species025_ENA_ProbOcc_EPSG2154.grd", sep = ""), format = "raster")
writeRaster(WIOsp0083_occ, 
            filename = paste(outdir, "Species0083_WIO_ProbOcc_EPSG2737.grd", sep = ""), format = "raster")
writeRaster(WIOsp025_occ, 
            filename = paste(outdir, "Species025_WIO_ProbOcc_EPSG2737.grd", sep = ""), format = "raster")

par(mfrow = c(2,2))
plot(ENAsp0083_occ, legend = F)
plot(ENAsp025_occ, legend = F)
plot(WIOsp025_occ, legend = F)
plot(WIOsp0083_occ, legend = F)

# suit
ENAsp0083_suit <- projectRaster(from = ENAsp0083_PA$suitab.raster, 
                           to = raster(extent(st_transform(strata_ENA, crs = "epsg:2154")), 
                                       crs = crs(st_transform(strata_ENA, crs = "epsg:2154")), 
                                       res = c(12969.728, 12969.728)))
ENAsp025_suit <- projectRaster(from = ENAsp025_PA$suitab.raster, 
                           to = raster(extent(st_transform(strata_ENA, crs = "epsg:2154")), 
                                       crs = crs(st_transform(strata_ENA, crs = "epsg:2154")), 
                                       res = c(22268.039, 22268.039)))

WIOsp0083_suit <- projectRaster(from = WIOsp0083_PA$suitab.raster, 
                           to = raster(extent(st_transform(strata_WIO, crs = "epsg:2737")), 
                                       crs = crs(st_transform(strata_WIO, crs = "epsg:2737")), 
                                       res = c(8562.396, 8562.396)))
WIOsp025_suit <- projectRaster(from = WIOsp025_PA$suitab.raster, 
                           to = raster(extent(st_transform(strata_WIO, crs = "epsg:2737")), 
                                       crs = crs(st_transform(strata_WIO, crs = "epsg:2737")), 
                                       res = c(27176.464, 27176.464)))

writeRaster(ENAsp0083_suit, 
            filename = paste(outdir, "Species0083_ENA_Suitability_EPSG2154.grd", sep = ""), format = "raster")
writeRaster(ENAsp025_suit, 
            filename = paste(outdir, "Species025_ENA_Suitability_EPSG2154.grd", sep = ""), format = "raster")
writeRaster(WIOsp0083_suit, 
            filename = paste(outdir, "Species0083_WIO_Suitability_EPSG2737.grd", sep = ""), format = "raster")
writeRaster(WIOsp025_suit, 
            filename = paste(outdir, "Species025_WIO_Suitability_EPSG2737.grd", sep = ""), format = "raster")



save.image("~/2_Species_simulations/Simulations.RData")



