##--------------------------------------------------------------------------------------------------------
## PROJECT : The effect of spatial resolution on SDM efficiency
## SCRIPT : Generate survey
##
## Author : Charlotte Lambert
## Last update : Jan 2022
## R version 4.0.4 (2021-02-15) -- "Lost Library Book"
##--------------------------------------------------------------------------------------------------------


library(dplyr)
library(tidyverse)
library(terra)
library(maps)
library(maptools)
library(ggplot2)
library(ggthemes)
library(ggspatial)
library(ggpubr)
library(RColorBrewer)
library(reshape2)
library(lubridate)
library(viridis)
library(ncdf4)
library(sf)
library(spatstat)
library(spatstat.utils)
library(multidplyr)


#### ----------------------- ####
#### ---- SOURCES FILES ---- ####
#### ----------------------- ####
sourcedir <- "~/1_source/Variables/"
spdir <- "~/2_Species_simulations/"
outdir <- "~/3_Survey_simulation/"


### 1 - Open ENA files ----------------------------------------------------------
bbx_ENA <- st_read(dsn = "~/1_source/GIS", layer = "BoundingBox_ENA")
land_ENA <- st_read(dsn = "~/1_source/GIS", layer = "Land_ENA_EPSG2154")
strata_ENA <- st_read(dsn = "~/1_source/GIS", layer = "Strata_ENA_clipped_EPSG2154", type = 3)
transects_ENA <- st_read(dsn = "~/1_source/GIS", layer = "Transects_ENA_EPSG2154", type = 3) 
segments_ENA <- st_read(dsn = "~/1_source/GIS", layer = "Segments_ENA_EPSG2154", type = 3) %>%
  rename(transect = transct, TransectLengthKm = TrnsLnK, SegLengthKm = SgLngtK, Longitude = Longitd, Latitude = Latitud) 


### 2 - Open species maps ------------------------------------------------------
ENAsp025_suit <- terra::rast(paste(spdir, "Species025_ENA_Suitability_EPSG2154.grd", sep = ""))
ENAsp025_occ <- terra::rast(paste(spdir, "Species025_ENA_ProbOcc_EPSG2154.grd", sep = ""))
ENAsp025_occ[ENAsp025_occ < 0] <- 0

source("~/3_Survey_simulation/Functions_simul_survey.R")



#### ------------------------------ ####
#### ---- GENERATE FIELD POINT ---- ####
#### ------------------------------ ####

### 1 - Simule point process and remove points outside strata ------------------
simultpp <- simul_spat(occ_map = ENAsp025_occ,
                       N = 100000, # taille estimee de la pop
                       # mean_group_size = 4,
                       projection_espg = 2154, # lambert93
                       n_sim = 100
)
pts_sf <- st_as_sf(simultpp, coords = c("x", "y"), crs = st_crs(2154)) %>% # proj lambert93
  dplyr::mutate(x = sf::st_coordinates(.)[,1],
                y = sf::st_coordinates(.)[,2],
                obs_id = 1:nrow(simultpp)
                ) %>% rename(Simulation = name)
pts_sf <- st_intersection(pts_sf, strata_ENA)


### 2 - Rasterize for reference map --------------------------------------------
simu <- 1:100
ENAsp025_ab <- pbapply::pblapply(seq_along(simu), function(i){
  ras <- terra::rasterize(x = pts_sf %>% subset(Simulation == paste0("Simulation ", simu[i])) %>% vect(), 
                          y = ENAsp025_occ,
                          fun = sum)
  ras <- raster::raster(ras)
  names(ras) <- paste0("ENA_Sp025_Simulation.", simu[i])
  return(ras)
}) 
ENAsp025_ab <- raster::stack(sapply(ENAsp025_ab, "[[", 1))

plot(ENAsp025_ab, col = viridis(20, option = "H"))

writeENVI <- function(x, f, ...) {
  writeRaster(x, f, overwrite=TRUE, ...)
  cat("band names = {", paste(names(x),collapse=","), "}", "\n", file=extension(f, "hdr"), append=TRUE)
}
writeENVI(x = ENAsp025_ab, f = paste0(outdir, "ENA_sp025_Simulated_obs_rasterized.envi"))



### 3 - Detection process ------------------------------------------------------
seg_buffer <- st_buffer(segments_ENA, dist = 200, endCapStyle = "FLAT")
pts_in <- st_join(pts_sf, seg_buffer, join = st_within) %>% drop_na(SegID)

plot(st_geometry(strata_ENA))
points(simultpp[which(simultpp$Simulation == "Simulation 100"), c("x", "y")], pch = 19, cex = 0.5)
plot(st_geometry(pts_in[which(pts_in$Simulation == "Simulation 100"),]), pch = 19, cex = 0.5, col = "red", add = T)

theme_set(theme_bw(base_size = 12))
g <- ggplot() +
  geom_sf(data = transects_ENA, color = "black") +
  geom_point(data = pts_sf[which(pts_sf$Simulation == "Simulation 1"),],
             aes(x = x, y = y), alpha = 0.3, shape = 20
  ) +
  geom_point(data = pts_in[which(pts_in$Simulation == "Simulation 1"),],
             aes(x = x.x, y = y.y), shape = 21, fill = "midnightblue"
  ) +
  annotation_scale(location = "tl", width_hint = 0.5) +
  annotation_north_arrow(location = "tr",
                         which_north = "true",
                         pad_x = unit(0.2, "cm"),
                         pad_y = unit(0.1, "cm"),
                         style = north_arrow_fancy_orienteering
  ) +
  labs(title = "Detection process",
       caption = paste("Sightings = ", nrow(pts_in[which(pts_in$Simulation == "Simulation 1"),1]), sep = " ")
  ) +
  theme(legend.position = "bottom",
        legend.key.width = unit(0.5, "cm"),
        legend.text = element_text(size = 6),
        panel.grid = element_line(colour = "transparent"), #element_blank(),
        plot.title = element_text(lineheight = 0.8, face = "bold"),
        axis.text = element_text(size = 10),
        strip.background = element_rect(fill = "white"),
        axis.title.x = element_blank(),
        axis.title.y = element_blank(),
        panel.background = element_rect(fill = "azure"),
        panel.border = element_rect(fill = NA)
  )
g


### 4 - Generate survey files --------------------------------------------------
dd <- pts_in %>%
  st_drop_geometry() %>%
  mutate(Id_obs = 1:n(),
         n = 1)

countdata <- as.data.frame(dd %>% group_by(SegID, Simulation) %>% summarize(n_detected = sum(n)))
simu <- unique(dd$Simulation)

segdata <- pbapply::pblapply(seq_along(simu), function(i){
  left_join(segments_ENA,
            countdata %>% subset(Simulation == simu[i]),
            by = "SegID") %>%
    mutate(Simulation = simu[i])
  }) %>% data.table::rbindlist() %>%
  mutate(n_detected = ifelse(is.na(n_detected), 0, n_detected)  ) %>%
  dplyr::select(x, y, Longitude, Latitude, strata, transect,
                TransectLengthKm, IdSeg, SegID, SegLengthKm, Simulation, n_detected) %>%
  mutate(ER = n_detected/SegLengthKm)


### 5 - Export files -----------------------------------------------------------

st_write(pts_sf,
         dsn = "~/3_Survey_simulation/ENA_sp025_Simulated_Survey_EPSG2154.gpkg",
         layer = "Point_process_complete")
st_write(pts_in,
         dsn = "~/3_Survey_simulation/ENA_sp025_Simulated_Survey_EPSG2154.gpkg",
         layer = "Point_process_detected", append = TRUE)
st_layers("~/3_Survey_simulation/ENA_sp025_Simulated_Survey_EPSG2154.gpkg")

write.table(segdata, "~/3_Survey_simulation/ENA_sp025_Simulated_Survey_EPSG2154_effort_with_obs.csv", 
            dec = ".", sep = ";")
write.table(pts_in, "~/3_Survey_simulation/ENA_sp025_Simulated_Survey_EPSG2154_obs_with_effort.csv", 
            dec = ".", sep = ";")
write.table(pts_sf, "~/3_Survey_simulation/ENA_sp025_Simulated_Survey_EPSG2154_presence.csv", 
            dec = ".", sep = ";")



### 6 - Summarize and exploration plot -----------------------------------------
smry <- segdata %>% 
  group_by(strata) %>% 
  summarize(n_detected = sum(n_detected),
            effort = sum(SegLengthKm)) %>%
  rename(Nom = strata) %>% 
  mutate(ER = n_detected/effort)

strata_ENA2 <- strata_ENA %>% left_join(smry, by = "Nom")
plot(strata_ENA2[c("n_detected", "effort", "ER")], graticule = T)


save.image("~/3_Survey_simulation/Survey_ENA_sp025.RData")



### 7 - Plot survey ------------------------------------------------------------
g1 <- ggplot() + 
  layer_spatial(raster::raster(ENAsp025_suit)) + scale_fill_viridis(na.value = NA, option = "H", limits = c(0,1)) + 
  layer_spatial(land_ENA, fill = "black", color = NA) + 
  layer_spatial(strata_ENA, fill = NA, color = "black") + 
  # layer_spatial(transects_ENA, color = "black") +
  ggtitle("Environmental suitability map") +
  layer_spatial(bbx_ENA, color = "black", size = 1, fill = NA)+
  annotation_scale(location = "bl", width_hint = 0.5) +
  annotation_north_arrow(location = "tr",
                         which_north = "true",
                         pad_x = unit(0.2, "cm"),
                         pad_y = unit(0.1, "cm"),
                         style = north_arrow_fancy_orienteering
  ) + theme_bw() +
  theme(legend.position = "bottom",
        legend.key.width = unit(0.5, "cm"),
        legend.text = element_text(size = 8),
        plot.title = element_text(lineheight = 0.8, face = "bold", size = 14),
        axis.text = element_text(size = 12),
        strip.background = element_rect(fill = "white"),
        axis.title = element_blank(),
        panel.background = element_rect(fill = "white"),
        panel.border = element_rect(fill = NA),
        panel.grid = element_line(colour = "grey80"), #element_blank(),
        legend.title = element_blank()
  )

g2 <- ggplot() + 
  layer_spatial(raster::raster(ENAsp025_occ)) + scale_fill_viridis(na.value = NA, option = "H", limits = c(0,1)) + 
  layer_spatial(land_ENA, fill = "black", color = NA) + 
  layer_spatial(strata_ENA, fill = NA, color = "black") + 
  # layer_spatial(transects_ENA, color = "black") +
  ggtitle("Species occurrence map") +
  layer_spatial(bbx_ENA, color = "black", size = 1, fill = NA)+
  annotation_scale(location = "bl", width_hint = 0.5) +
  annotation_north_arrow(location = "tr",
                         which_north = "true",
                         pad_x = unit(0.2, "cm"),
                         pad_y = unit(0.1, "cm"),
                         style = north_arrow_fancy_orienteering
  ) + 
  theme(legend.position = "bottom",
        legend.key.width = unit(0.5, "cm"),
        legend.text = element_text(size = 8),
        plot.title = element_text(lineheight = 0.8, face = "bold", size = 14),
        axis.text = element_text(size = 12),
        strip.background = element_rect(fill = "white"),
        axis.title = element_blank(),
        panel.background = element_rect(fill = "white"),
        panel.border = element_rect(fill = NA),
        panel.grid = element_line(colour = "grey80"), #element_blank(),
        legend.title = element_blank()
  )


pts_simu <- pts_sf %>% st_drop_geometry() %>% subset(Simulation == "Simulation 1")
pts_det <- pts_in %>% st_drop_geometry() %>% subset(Simulation == "Simulation 1")

g3 <- ggplot() +
  layer_spatial(land_ENA, fill = "black", color = NA) + 
  layer_spatial(strata_ENA, fill = NA, color = "black") + 
  layer_spatial(transects_ENA, color = "black") +
  geom_point(data = pts_simu, aes(x = x, y = y), color = "grey60", cex = 1.2, shape = 20) +
  geom_point(data = pts_det, aes(x = x.x, y = y.x), color = "red", cex = 1.5, shape = 20) +
  ggtitle("Presences (grey) and detections (red)") +
  layer_spatial(bbx_ENA, color = "black", size = 1, fill = NA)+
  annotation_scale(location = "bl", width_hint = 0.5) +
  annotation_north_arrow(location = "tr",
                         which_north = "true",
                         pad_x = unit(0.2, "cm"),
                         pad_y = unit(0.1, "cm"),
                         style = north_arrow_fancy_orienteering
  ) + 
  theme(legend.position = "bottom",
        legend.key.width = unit(0.5, "cm"),
        legend.text = element_text(size = 8),
        plot.title = element_text(lineheight = 0.8, face = "bold", size = 14),
        axis.text = element_text(size = 12),
        strip.background = element_rect(fill = "white"),
        axis.title = element_blank(),
        panel.background = element_rect(fill = "white"),
        panel.border = element_rect(fill = NA),
        panel.grid = element_line(colour = "grey80"), #element_blank(),
        
  )
ggarrange(g1, g2, g3, nrow = 1, ncol = 3, align = "hv", common.legend = T, legend = "right")
ggsave(paste(outdir, "ENA_sp025_Simulated_survey.png", sep = ""), device = "png", dpi = 300, height = 8, width = 18)



ggplot() +
  layer_spatial(land_ENA, fill = "black", color = NA) + 
  layer_spatial(strata_ENA, fill = NA, color = "black") + 
  layer_spatial(transects_ENA, color = "black") +
  geom_point(data = pts_simu, aes(x = x, y = y), color = "grey", cex = 0.8, alpha = 0.6, shape = 20) +
  geom_point(data = pts_det, aes(x = x.x, y = y.x), color = "red", cex = 0.8, shape = 20) +
  ggtitle("Detections") +
  layer_spatial(bbx_ENA, color = "black", size = 1, fill = NA)+
  annotation_scale(location = "bl", width_hint = 0.5) +
  annotation_north_arrow(location = "tr",
                         which_north = "true",
                         pad_x = unit(0.2, "cm"),
                         pad_y = unit(0.1, "cm"),
                         style = north_arrow_fancy_orienteering
  ) + theme_bw() +
  theme(legend.position = "bottom",
        legend.key.width = unit(0.5, "cm"),
        legend.text = element_text(size = 8),
        plot.title = element_text(lineheight = 0.8, face = "bold", size = 14),
        axis.text = element_text(size = 12),
        strip.background = element_rect(fill = "white"),
        axis.title = element_blank(),
        panel.background = element_rect(fill = "white"),
        panel.border = element_rect(fill = NA),
        panel.grid = element_line(colour = "grey80"), #element_blank(),
        
  ) 
ggsave(paste(outdir, "ENA_sp025_Simulated_obs.png", sep = ""), device = "png", dpi = 300, height = 8, width = 6)



