### simulate distribution of individuals
simul_spat <- function(occ_map, # predgrid & density_map must be projected
                       N, # estimated pop size
                       projection_espg = 2154, 
                       seed = NULL,
                       n_sim = 1
) {
    # base map to im object
    # https://rdrr.io/cran/maptools/man/as.ppp.html
    df_map <- terra::as.data.frame(occ_map, xy = T)
    df_map <- df_map %>% drop_na()
    
    X <- spatstat.geom::as.im(df_map)
    # use spatstat.geom::as.im() for class data.frame (sur data.frame(raster))
    
    ### generate observations according to an Inhomogeneous Poisson Point Process with spatstat IPP
    seed <- sample.int(1e6, 1)
    set.seed(seed)
    y_obs <- rpoispp_custom(lambda = X, drop = TRUE, N = N, nsim = n_sim)
    
    if(n_sim > 1) {
      y_obs <- purrr::map_df(y_obs,
                           ~tibble(x = .$x, y = .$y),
                           .id = "name")
    }

    pts <- obs <- data.frame(x = y_obs$x,
                             y = y_obs$y,
                             name = y_obs$name
    )
    
    pts <- pts %>%
      st_as_sf(coords = c("x", "y"), crs = st_crs(projection_espg)) %>%
      st_transform(crs = st_crs(4326)) %>%
      st_coordinates()
    obs$lon <- pts[, 1]
    obs$lat <- pts[, 2]
    # n_obs <- nrow(obs)
    
    # ### generate a mark: group size
    # omega <- rep(1, n_obs)
    # obs$size <- rpois(n_obs, lambda = (mean_group_size - 1) * omega) + 1
    return(obs)
}

# distance to line transects and detection process
detection_process <- function(pts, # spatial point (sf) projected
                              transects, # same projection as points
                              sigma = NULL, # esw in km ! 
                              verbose = FALSE) {
  ## compute distance with st_distance (use projected layers)
  dist_mat <- sf::st_distance(
    x = pts,
    y = transects) 
  pts$distance <- apply(dist_mat, 1, min)
  pts$distance_km <- pts$distance / 1e3
  pts$closest.seg <- apply(dist_mat, 1, function(x) min(which(x == min(x, na.rm = TRUE))))
  pts$segID <- transects$segID[pts$closest.seg]
  
  ## detection process
  if(is.null(sigma)) {
    if(verbose) {
      message("Strip-transect: perfect detection within the first 200 meters")
    }
    esw <- 0.2
    pts$detected <- ifelse(pts$distance_km <= esw, 1, 0)
  } else {
    if(verbose) {
      message("Line-transect: assuming a half-normal key for detection function")
    }
    esw <- (pnorm(+Inf, 0, sigma) - 0.5) / dnorm(0, 0, sigma)
    proba <- exp(- (pts$distance_km)^2 / (2 * sigma * sigma))
    pts$detected <- rbinom(nrow(pts), size = 1, prob = proba)
  }
  pts <- pts %>% sf::st_drop_geometry()
  
  return(pts)
}



rpoispp_custom <- function (lambda, lmax = NULL, win = owin(), ..., nsim = 1, 
          drop = TRUE, ex = NULL, warnwin = TRUE, N) 
{
  if (!missing(nsim)) {
    check.1.integer(nsim)
    stopifnot(nsim >= 1)
  }
  if (missing(lambda) && is.null(lmax) && missing(win) && 
      !is.null(ex)) {
    lambda <- intensity(unmark(ex))
    win <- Window(ex)
  }
  else {
    if (!(is.numeric(lambda) || is.function(lambda) || is.im(lambda))) 
      stop(paste(sQuote("lambda"), "must be a constant, a function or an image"))
    if (is.numeric(lambda) && !(length(lambda) == 1 && lambda >= 
                                0)) 
      stop(paste(sQuote("lambda"), "must be a single, nonnegative number"))
    if (!is.null(lmax)) {
      if (!is.numeric(lmax)) 
        stop("lmax should be a number")
      if (length(lmax) > 1) 
        stop("lmax should be a single number")
    }
    if (is.im(lambda)) {
      if (warnwin && !missing(win)) 
        warning("Argument win ignored", call. = FALSE)
      win <- rescue.rectangle(as.owin(lambda))
    }
    else {
      win <- as.owin(win)
    }
  }
  if (is.numeric(lambda)) 
    return(runifpoispp(lambda, win, nsim = nsim, drop = drop))
  if (is.null(lmax)) {
    imag <- as.im(lambda, win, ...)
    summ <- summary(imag)
    lmax <- summ$max + 0.05 * diff(summ$range)
  }
  if (is.function(lambda)) {
    result <- runifpoispp(lmax, win, nsim = nsim, drop = FALSE)
    for (isim in 1:nsim) {
      X <- result[[isim]]
      if (X$n > 0) {
        prob <- lambda(X$x, X$y, ...)/lmax
        u <- runif(X$n)
        retain <- (u <= prob)
        result[[isim]] <- X[retain]
      }
    }
    if (nsim == 1 && drop) 
      result <- result[[1L]]
    return(result)
  }
  if (is.im(lambda)) {
    if (spatstat.options("fastpois")) {
      # mu <- integral(lambda)
      mu <- N
      dx <- lambda$xstep/2
      dy <- lambda$ystep/2
      df <- as.data.frame(lambda)
      npix <- nrow(df)
      lpix <- df$value
      result <- vector(mode = "list", length = nsim)
      nn <- rpois(nsim, mu)
      if (!all(is.finite(nn))) 
        stop(paste("Unable to generate Poisson process with a mean of", 
                   mu, "points"))
      for (isim in seq_len(nsim)) {
        ni <- nn[isim]
        ii <- sample.int(npix, size = ni, replace = TRUE, 
                         prob = lpix)
        xx <- df$x[ii] + runif(ni, -dx, dx)
        yy <- df$y[ii] + runif(ni, -dy, dy)
        result[[isim]] <- ppp(xx, yy, window = win, 
                              check = FALSE)
      }
      result <- simulationresult(result, nsim, drop)
      return(result)
    }
    else {
      result <- runifpoispp(lmax, win, nsim = nsim, drop = FALSE)
      for (isim in 1:nsim) {
        X <- result[[isim]]
        if (X$n > 0) {
          prob <- lambda[X]/lmax
          u <- runif(X$n)
          retain <- (u <= prob)
          result[[isim]] <- X[retain]
        }
      }
      if (nsim == 1 && drop) 
        return(result[[1L]])
      return(result)
    }
  }
  stop(paste(sQuote("lambda"), "must be a constant, a function or an image"))
}
