##--------------------------------------------------------------------------------------------------------
## PROJECT : The effect of spatial resolution on SDM efficiency
## SCRIPT : Format survey data for modelling - Eastern North Atlantic
##
## Author : Charlotte Lambert
## Last update : Jan 2022
## R version 4.0.4 (2021-02-15) -- "Lost Library Book"
##--------------------------------------------------------------------------------------------------------


library(dplyr)
library(tidyverse)
library(raster)
library(terra)
library(maps)
library(maptools)
library(ggplot2)
library(ggthemes)
library(ggspatial)
library(ggpubr)
library(RColorBrewer)
library(reshape2)
library(lubridate)
library(viridis)
library(ncdf4)
library(sf)
library(spatstat)
library(spatstat.utils)
library(multidplyr)


#### ----------------------- ####
#### ---- SOURCES FILES ---- ####
#### ----------------------- ####
sourcedir <- "~/1_Source/Variables/"
spdir <- "~/2_Species_Simulations/"
surveydir <- "~/3_Survey_simulation/"
outdir <- "~/4_Formatting/"

### 1 - Open WE files ----------------------------------------------------------
bbx_ENA <- st_read(dsn = "~/1_source/GIS", layer = "BoundingBox_ENA")
land_ENA <- st_read(dsn = "~/1_source/GIS", layer = "Land_ENA_EPSG2154") %>%
  st_transform(st_crs("epsg:4326"))
strata_ENA <- st_read(dsn = "~/1_source/GIS", layer = "Strata_ENA_clipped_EPSG2154", type = 3) %>%
  st_transform(st_crs("epsg:4326"))
transects_ENA <- st_read(dsn = "~/1_source/GIS", layer = "Transects_ENA_EPSG2154", type = 3)  %>%
  st_transform(st_crs("epsg:4326"))
segments_ENA <- st_read(dsn = "~/1_source/GIS", layer = "Segments_ENA_EPSG2154", type = 3) %>%
  rename(transect = transct, TransectLengthKm = TrnsLnK, SegLengthKm = SgLngtK, Longitude = Longitd, Latitude = Latitud) %>%
  st_transform(st_crs("epsg:4326")) 


### 2 - Open species maps ------------------------------------------------------
ENAsp0083_suit <- terra::rast(paste(spdir, "Species0083_ENA_Suitability_EPSG2154.grd", sep = ""))
ENAsp0083_occ <- terra::rast(paste(spdir, "Species0083_ENA_ProbOcc_EPSG2154.grd", sep = ""))
ENAsp0083_occ[ENAsp0083_occ < 0] <- 0

ENAsp025_suit <- terra::rast(paste(spdir, "Species025_ENA_Suitability_EPSG2154.grd", sep = ""))
ENAsp025_occ <- terra::rast(paste(spdir, "Species025_ENA_ProbOcc_EPSG2154.grd", sep = ""))
ENAsp025_occ[ENAsp025_occ < 0] <- 0


### 3 - Open survey files ------------------------------------------------------
ENAsp0083_seg <- read.table("~/3_Survey_simulation/ENA_sp0083_Simulated_Survey_EPSG2154_effort_with_obs.csv", 
                        h = T, dec = ".", sep = ";")%>% rename(n_sp0083 = n_detected)
simu <- unique(ENAsp0083_seg$Simulation)

ENAsp025_seg <- read.table("~/3_Survey_simulation/ENA_sp025_Simulated_Survey_EPSG2154_effort_with_obs.csv", 
                        h = T, dec = ".", sep = ";")%>% rename(n_sp025 = n_detected)

ENA_seg <- left_join(ENAsp0083_seg, ENAsp025_seg, by = c("x", "y", "Longitude", "Latitude", "strata", "transect", "TransectLengthKm", "IdSeg", "SegID", "SegLengthKm", "Simulation"))


### 4 - Open variables rasters -------------------------------------------------
stack_ENA_0083 <- stack(paste(sourcedir, "EnvironmentalVariables_ENA_0083.envi", sep = ""))
stack_ENA_017 <- stack(paste(sourcedir, "EnvironmentalVariables_ENA_017.envi", sep = ""))
stack_ENA_025 <- stack(paste(sourcedir, "EnvironmentalVariables_ENA_025.envi", sep = ""))
stack_ENA_050 <- stack(paste(sourcedir, "EnvironmentalVariables_ENA_050.envi", sep = ""))
stack_ENA_100 <- stack(paste(sourcedir, "EnvironmentalVariables_ENA_100.envi", sep = ""))



#### ---------------------- ####
#### ----- Rasterize  ----- ####
#### ---------------------- ####

### Rasterize to 0.17deg -------------------------------------------------------
ENA_017 <- NULL
ENA_017 <- lapply(seq_along(simu), function(i){
  df <- ENA_seg %>% subset(Simulation == simu[i])
  df_eff <- vect(x = df, crs = st_crs(4326), geom = c("Longitude", "Latitude"))
  
  obs_sp0083 <- terra::rasterize(x = df_eff, field = "n_sp0083", y = rast(stack_ENA_017), fun = sum) 
  obs_sp025 <- terra::rasterize(x = df_eff, field = "n_sp025", y = rast(stack_ENA_017), fun = sum) 
  eff <- terra::rasterize(x = df_eff, field = "SegLengthKm", y = rast(stack_ENA_017), fun = sum) 
  
  df_fin <- left_join(as.data.frame(eff, xy = T),
                      as.data.frame(obs_sp0083, xy = T), by = c("x", "y")) %>%
    left_join(.,
              as.data.frame(obs_sp025, xy = T), by = c("x", "y")) %>%
    mutate(Simulation = simu[i])
  colnames(df_fin) <- c("Longitude", "Latitude", "effort", "n_sp0083", "n_sp025", "Simulation")
  return(df_fin)
}) %>% data.table::rbindlist()


### Rasterize to 0.25deg -------------------------------------------------------
ENA_025 <- NULL
ENA_025 <- lapply(seq_along(simu), function(i){
  df <- ENA_seg %>% subset(Simulation == simu[i])
  df_eff <- vect(x = df, crs = st_crs(4326), geom = c("Longitude", "Latitude"))
  
  obs_sp0083 <- terra::rasterize(x = df_eff, field = "n_sp0083", y = rast(stack_ENA_025), fun = sum) 
  obs_sp025 <- terra::rasterize(x = df_eff, field = "n_sp025", y = rast(stack_ENA_025), fun = sum) 
  eff <- terra::rasterize(x = df_eff, field = "SegLengthKm", y = rast(stack_ENA_025), fun = sum) 
  
  df_fin <- left_join(as.data.frame(eff, xy = T),
                      as.data.frame(obs_sp0083, xy = T), by = c("x", "y")) %>%
    left_join(.,
              as.data.frame(obs_sp025, xy = T), by = c("x", "y")) %>%
    mutate(Simulation = simu[i])
  colnames(df_fin) <- c("Longitude", "Latitude", "effort", "n_sp0083", "n_sp025", "Simulation")
  return(df_fin)
}) %>% data.table::rbindlist()


### Rasterize to 0.50deg -------------------------------------------------------
ENA_050 <- NULL
ENA_050 <- lapply(seq_along(simu), function(i){
  df <- ENA_seg %>% subset(Simulation == simu[i])
  df_eff <- vect(x = df, crs = st_crs(4326), geom = c("Longitude", "Latitude"))
  
  obs_sp0083 <- terra::rasterize(x = df_eff, field = "n_sp0083", y = rast(stack_ENA_050), fun = sum) 
  obs_sp025 <- terra::rasterize(x = df_eff, field = "n_sp025", y = rast(stack_ENA_050), fun = sum) 
  eff <- terra::rasterize(x = df_eff, field = "SegLengthKm", y = rast(stack_ENA_050), fun = sum) 
  
  df_fin <- left_join(as.data.frame(eff, xy = T),
                      as.data.frame(obs_sp0083, xy = T), by = c("x", "y")) %>%
    left_join(.,
              as.data.frame(obs_sp025, xy = T), by = c("x", "y")) %>%
    mutate(Simulation = simu[i])
  colnames(df_fin) <- c("Longitude", "Latitude", "effort", "n_sp0083", "n_sp025", "Simulation")
  return(df_fin)
}) %>% data.table::rbindlist()


### Rasterize to 1.00deg -------------------------------------------------------
ENA_100 <- NULL
ENA_100 <- lapply(seq_along(simu), function(i){
  df <- ENA_seg %>% subset(Simulation == simu[i])
  df_eff <- vect(x = df, crs = st_crs(4326), geom = c("Longitude", "Latitude"))
  
  obs_sp0083 <- terra::rasterize(x = df_eff, field = "n_sp0083", y = rast(stack_ENA_100), fun = sum) 
  obs_sp025 <- terra::rasterize(x = df_eff, field = "n_sp025", y = rast(stack_ENA_100), fun = sum) 
  eff <- terra::rasterize(x = df_eff, field = "SegLengthKm", y = rast(stack_ENA_100), fun = sum) 
  
  df_fin <- left_join(as.data.frame(eff, xy = T),
                      as.data.frame(obs_sp0083, xy = T), by = c("x", "y")) %>%
    left_join(.,
              as.data.frame(obs_sp025, xy = T), by = c("x", "y")) %>%
    mutate(Simulation = simu[i])
  colnames(df_fin) <- c("Longitude", "Latitude", "effort", "n_sp0083", "n_sp025", "Simulation")
  return(df_fin)
}) %>% data.table::rbindlist()




#### ----------------------- ####
#### ----- Plots maps  ----- ####
#### ----------------------- ####

# layout
layout <- theme_bw() +
  theme(legend.position = "bottom",
        legend.key.width = unit(0.5, "cm"),
        legend.text = element_text(size = 8),
        plot.title = element_text(lineheight = 0.8, face = "bold", size = 14),
        axis.text = element_text(size = 12),
        strip.background = element_rect(fill = "white"),
        axis.title = element_blank(),
        panel.background = element_rect(fill = "white"),
        panel.border = element_rect(fill = NA),
        panel.grid = element_line(colour = "grey80"), #element_blank(),
        legend.title = element_blank()
  )


### 1 - Plots efforts ----------------------------------------------------------
eff_017 <- ggplot() + 
  geom_raster(data = ENA_017 %>% subset(Simulation == "Simulation 1"), aes(x = Longitude, y = Latitude, fill = effort)) + 
  scale_fill_viridis(na.value = NA, option = "H") + 
  layer_spatial(land_ENA, fill = "black", color = NA) + 
  layer_spatial(strata_ENA, fill = NA, color = "black") + 
  layer_spatial(bbx_ENA, color = "black", size = 1, fill = NA)+
  annotation_scale(location = "bl", width_hint = 0.5) +
  annotation_north_arrow(location = "tr",
                         which_north = "true",
                         pad_x = unit(0.2, "cm"),
                         pad_y = unit(0.1, "cm"),
                         style = north_arrow_fancy_orienteering
  ) + layout + 
  ggtitle("0.17")

eff_025 <- ggplot() + 
  geom_raster(data = ENA_025 %>% subset(Simulation == "Simulation 1"), aes(x = Longitude, y = Latitude, fill = effort)) + 
  scale_fill_viridis(na.value = NA, option = "H") + 
  layer_spatial(land_ENA, fill = "black", color = NA) + 
  layer_spatial(strata_ENA, fill = NA, color = "black") + 
  layer_spatial(bbx_ENA, color = "black", size = 1, fill = NA)+
  annotation_scale(location = "bl", width_hint = 0.5) +
  annotation_north_arrow(location = "tr",
                         which_north = "true",
                         pad_x = unit(0.2, "cm"),
                         pad_y = unit(0.1, "cm"),
                         style = north_arrow_fancy_orienteering
  ) + layout + 
  ggtitle("0.25")

eff_050 <- ggplot() + 
  geom_raster(data = ENA_050 %>% subset(Simulation == "Simulation 1"), aes(x = Longitude, y = Latitude, fill = effort)) + 
  scale_fill_viridis(na.value = NA, option = "H") + 
  layer_spatial(land_ENA, fill = "black", color = NA) + 
  layer_spatial(strata_ENA, fill = NA, color = "black") + 
  layer_spatial(bbx_ENA, color = "black", size = 1, fill = NA)+
  annotation_scale(location = "bl", width_hint = 0.5) +
  annotation_north_arrow(location = "tr",
                         which_north = "true",
                         pad_x = unit(0.2, "cm"),
                         pad_y = unit(0.1, "cm"),
                         style = north_arrow_fancy_orienteering
  ) + layout + 
  ggtitle("0.50")

eff_100 <- ggplot() + 
  geom_raster(data = ENA_100 %>% subset(Simulation == "Simulation 1"), aes(x = Longitude, y = Latitude, fill = effort)) + 
  scale_fill_viridis(na.value = NA, option = "H") + 
  layer_spatial(land_ENA, fill = "black", color = NA) + 
  layer_spatial(strata_ENA, fill = NA, color = "black") + 
  layer_spatial(bbx_ENA, color = "black", size = 1, fill = NA)+
  annotation_scale(location = "bl", width_hint = 0.5) +
  annotation_north_arrow(location = "tr",
                         which_north = "true",
                         pad_x = unit(0.2, "cm"),
                         pad_y = unit(0.1, "cm"),
                         style = north_arrow_fancy_orienteering
  ) + layout + 
  ggtitle("1.00")

ggarrange(eff_017, eff_025, eff_050, eff_100, nrow = 1, ncol = 4, align = "hv") + bgcolor("white")
ggsave("~/4_Formatting/ENA_Rasterized_Effort.png", dpi = 350, height = 8, width = 15)


save.image("~/4_Formatting/ENA_Rasterize.RData")


### 2 - Plot Species 1 ---------------------------------------------------------
sp0083_017 <- ggplot() + 
  geom_raster(data = ENA_017 %>% subset(Simulation == "Simulation 1"), aes(x = Longitude, y = Latitude, fill = n_sp0083)) + 
  scale_fill_viridis(na.value = NA, option = "H") + 
  layer_spatial(land_ENA, fill = "black", color = NA) + 
  layer_spatial(strata_ENA, fill = NA, color = "black") + 
  layer_spatial(bbx_ENA, color = "black", size = 1, fill = NA)+
  annotation_scale(location = "bl", width_hint = 0.5) +
  annotation_north_arrow(location = "tr",
                         which_north = "true",
                         pad_x = unit(0.2, "cm"),
                         pad_y = unit(0.1, "cm"),
                         style = north_arrow_fancy_orienteering
  ) + layout + 
  ggtitle("0.17")

sp0083_025 <- ggplot() + 
  geom_raster(data = ENA_025 %>% subset(Simulation == "Simulation 1"), aes(x = Longitude, y = Latitude, fill = n_sp0083)) + 
  scale_fill_viridis(na.value = NA, option = "H") + 
  layer_spatial(land_ENA, fill = "black", color = NA) + 
  layer_spatial(strata_ENA, fill = NA, color = "black") + 
  layer_spatial(bbx_ENA, color = "black", size = 1, fill = NA)+
  annotation_scale(location = "bl", width_hint = 0.5) +
  annotation_north_arrow(location = "tr",
                         which_north = "true",
                         pad_x = unit(0.2, "cm"),
                         pad_y = unit(0.1, "cm"),
                         style = north_arrow_fancy_orienteering
  ) + layout + 
  ggtitle("0.25")

sp0083_050 <- ggplot() + 
  geom_raster(data = ENA_050 %>% subset(Simulation == "Simulation 1"), aes(x = Longitude, y = Latitude, fill = n_sp0083)) + 
  scale_fill_viridis(na.value = NA, option = "H") + 
  layer_spatial(land_ENA, fill = "black", color = NA) + 
  layer_spatial(strata_ENA, fill = NA, color = "black") + 
  layer_spatial(bbx_ENA, color = "black", size = 1, fill = NA)+
  annotation_scale(location = "bl", width_hint = 0.5) +
  annotation_north_arrow(location = "tr",
                         which_north = "true",
                         pad_x = unit(0.2, "cm"),
                         pad_y = unit(0.1, "cm"),
                         style = north_arrow_fancy_orienteering
  ) + layout + 
  ggtitle("0.50")

sp0083_100 <- ggplot() + 
  geom_raster(data = ENA_100 %>% subset(Simulation == "Simulation 1"), aes(x = Longitude, y = Latitude, fill = n_sp0083)) + 
  scale_fill_viridis(na.value = NA, option = "H") + 
  layer_spatial(land_ENA, fill = "black", color = NA) + 
  layer_spatial(strata_ENA, fill = NA, color = "black") + 
  layer_spatial(bbx_ENA, color = "black", size = 1, fill = NA)+
  annotation_scale(location = "bl", width_hint = 0.5) +
  annotation_north_arrow(location = "tr",
                         which_north = "true",
                         pad_x = unit(0.2, "cm"),
                         pad_y = unit(0.1, "cm"),
                         style = north_arrow_fancy_orienteering
  ) + layout + 
  ggtitle("1.00")

ggarrange(sp0083_017, sp0083_025, sp0083_050, sp0083_100, nrow = 1, ncol = 4, align = "hv") + bgcolor("white")
ggsave("~/4_Formatting/ENA_Rasterized_Species0083_NObs.png", dpi = 350, height = 8, width = 15)


### 3 - Plot Species 2 ---------------------------------------------------------
sp025_017 <- ggplot() + 
  geom_raster(data = ENA_017 %>% subset(Simulation == "Simulation 1"), aes(x = Longitude, y = Latitude, fill = n_sp025)) + 
  scale_fill_viridis(na.value = NA, option = "H") + 
  layer_spatial(land_ENA, fill = "black", color = NA) + 
  layer_spatial(strata_ENA, fill = NA, color = "black") + 
  layer_spatial(bbx_ENA, color = "black", size = 1, fill = NA)+
  annotation_scale(location = "bl", width_hint = 0.5) +
  annotation_north_arrow(location = "tr",
                         which_north = "true",
                         pad_x = unit(0.2, "cm"),
                         pad_y = unit(0.1, "cm"),
                         style = north_arrow_fancy_orienteering
  ) + layout + 
  ggtitle("0.17")

sp025_025 <- ggplot() + 
  geom_raster(data = ENA_025 %>% subset(Simulation == "Simulation 1"), aes(x = Longitude, y = Latitude, fill = n_sp025)) + 
  scale_fill_viridis(na.value = NA, option = "H") + 
  layer_spatial(land_ENA, fill = "black", color = NA) + 
  layer_spatial(strata_ENA, fill = NA, color = "black") + 
  layer_spatial(bbx_ENA, color = "black", size = 1, fill = NA)+
  annotation_scale(location = "bl", width_hint = 0.5) +
  annotation_north_arrow(location = "tr",
                         which_north = "true",
                         pad_x = unit(0.2, "cm"),
                         pad_y = unit(0.1, "cm"),
                         style = north_arrow_fancy_orienteering
  ) + layout + 
  ggtitle("0.25")

sp025_050 <- ggplot() + 
  geom_raster(data = ENA_050 %>% subset(Simulation == "Simulation 1"), aes(x = Longitude, y = Latitude, fill = n_sp025)) + 
  scale_fill_viridis(na.value = NA, option = "H") + 
  layer_spatial(land_ENA, fill = "black", color = NA) + 
  layer_spatial(strata_ENA, fill = NA, color = "black") + 
  layer_spatial(bbx_ENA, color = "black", size = 1, fill = NA)+
  annotation_scale(location = "bl", width_hint = 0.5) +
  annotation_north_arrow(location = "tr",
                         which_north = "true",
                         pad_x = unit(0.2, "cm"),
                         pad_y = unit(0.1, "cm"),
                         style = north_arrow_fancy_orienteering
  ) + layout + 
  ggtitle("0.50")

sp025_100 <- ggplot() + 
  geom_raster(data = ENA_100 %>% subset(Simulation == "Simulation 1"), aes(x = Longitude, y = Latitude, fill = n_sp025)) + 
  scale_fill_viridis(na.value = NA, option = "H") + 
  layer_spatial(land_ENA, fill = "black", color = NA) + 
  layer_spatial(strata_ENA, fill = NA, color = "black") + 
  layer_spatial(bbx_ENA, color = "black", size = 1, fill = NA)+
  annotation_scale(location = "bl", width_hint = 0.5) +
  annotation_north_arrow(location = "tr",
                         which_north = "true",
                         pad_x = unit(0.2, "cm"),
                         pad_y = unit(0.1, "cm"),
                         style = north_arrow_fancy_orienteering
  ) + layout + 
  ggtitle("1.00")

ggarrange(sp025_017, sp025_025, sp025_050, sp025_100, nrow = 1, ncol = 4, align = "hv") + bgcolor("white")
ggsave("~/4_Formatting/ENA_Rasterized_Species025_NObs.png", dpi = 350, height = 8, width = 15)


### 4 - Plot all ---------------------------------------------------------------
ggarrange(eff_017, eff_025, eff_050, eff_100,
          sp0083_017, sp0083_025, sp0083_050, sp0083_100,
          sp025_017, sp025_025, sp025_050, sp025_100, nrow = 3, ncol = 4, align = "hv") + 
  bgcolor("white") + border("white")
ggsave("~/4_Formatting/ENA_Rasterized_tot.png", dpi = 350, height = 18, width = 15)

save.image("~/4_Formatting/ENA_Rasterize.RData")



#### ------------------------------ ####
#### ----- Extract covariates ----- ####
#### ------------------------------ ####

## segments
ENA_seg_cov0083 <- raster::extract(stack_ENA_0083, y = ENA_seg[, c("Longitude", "Latitude")])
ENA_seg_0083 <- cbind(ENA_seg, ENA_seg_cov0083[,1:13])

ENA_seg_cov017 <- raster::extract(stack_ENA_017, y = ENA_seg[, c("Longitude", "Latitude")])
ENA_seg_017 <- cbind(ENA_seg, ENA_seg_cov017[,1:13])

ENA_seg_cov025 <- raster::extract(stack_ENA_025, y = ENA_seg[, c("Longitude", "Latitude")])
ENA_seg_025 <- cbind(ENA_seg, ENA_seg_cov025[,1:13])

ENA_seg_cov050 <- raster::extract(stack_ENA_050, y = ENA_seg[, c("Longitude", "Latitude")])
ENA_seg_050 <- cbind(ENA_seg, ENA_seg_cov050[,1:13])

ENA_seg_cov100 <- raster::extract(stack_ENA_100, y = ENA_seg[, c("Longitude", "Latitude")])
ENA_seg_100 <- cbind(ENA_seg, ENA_seg_cov100[,1:13])

## 017
ENA_017_cov <- raster::extract(stack_ENA_017, y = ENA_017[, c("Longitude", "Latitude")])
ENA_017 <- cbind(ENA_017, ENA_017_cov[,1:13])

## 025
ENA_025_cov <- raster::extract(stack_ENA_025, y = ENA_025[, c("Longitude", "Latitude")])
ENA_025 <- cbind(ENA_025, ENA_025_cov[,1:13])

## 050
ENA_050_cov <- raster::extract(stack_ENA_050, y = ENA_050[, c("Longitude", "Latitude")])
ENA_050 <- cbind(ENA_050, ENA_050_cov[,1:13])

## 100
ENA_100_cov <- raster::extract(stack_ENA_100, y = ENA_100[, c("Longitude", "Latitude")])
ENA_100 <- cbind(ENA_100, ENA_100_cov[,1:13])



save.image("~/4_Formatting/ENA_Rasterize.RData")



#### ------------------------------ ####
#### ----- Saturate variables ----- ####
#### ------------------------------ ####

# function
saturate <- function(x, threshold = NULL, lower = FALSE, alpha = 0.01, both = FALSE) {
  if(both) {
    if(is.null(threshold)) {
      threshold <- quantile(x, c(alpha/2, 1-alpha/2), na.rm = TRUE)
    }
    if(length(threshold) != 2){
      stop("must provide a lower and upper thresholds")
    }
    x <- ifelse(x < min(threshold), min(threshold), x)
    x <- ifelse(x > max(threshold), max(threshold), x)
  }
  else{
    if(is.null(threshold)) {
      threshold.lower <- quantile(x, alpha/2, na.rm = TRUE)
      threshold.upper <- quantile(x, 1-alpha/2, na.rm = TRUE)
      if(lower) { x <- ifelse(x < threshold.lower, threshold.lower, x) }
      else { x <- ifelse(x > threshold.upper, threshold.upper, x) }
    } else {
      if(lower) { x <- ifelse(x < threshold, threshold, x) }
      else { x <- ifelse(x > threshold, threshold, x) }
    }
    
  }
  return(x)
}
vars <- names(stack_ENA_0083)[1:9]



### 1 - Segments 0.083 --------------------------------------------------------
ENA_seg_0083_b <- lapply(seq_along(simu), function(i){
    df <- ENA_seg_0083 %>% subset(Simulation == simu[i])
    
    df[, "CurrentSpeed"] <- saturate(df[, "CurrentSpeed"], both = F, alpha = 0.002)
    df[, "EKE"] <- saturate(df[, "EKE"], both = F, alpha = 0.0015)
    df[, "MLD"] <- saturate(df[, "MLD"], both = F, alpha = 0.0008)
    df[, "NPP"] <- saturate(df[, "NPP"], both = F, alpha = 0.004)
    df[, "Slope"] <- saturate(df[, "Slope"], both = F, alpha = 0.005)
    df[, "SSH"] <- saturate(df[, "SSH"], both = T, alpha = 0.005)
    df[, "Temp"] <- saturate(df[, "Temp"], both = F, lower = T, alpha = 0.002)
    df[, "ZEu"] <- saturate(df[, "ZEu"], both = T, alpha = 0.002)
    
    return(df)
  }) %>% data.table::rbindlist()



### 2 - Segments 0.17 ---------------------------------------------------------
ENA_seg_017_b <- lapply(seq_along(simu), function(i){
  df <- ENA_seg_017 %>% subset(Simulation == simu[i])
  
  df[, "CurrentSpeed"] <- saturate(df[, "CurrentSpeed"], both = F, alpha = 0.0025)
  df[, "EKE"] <- saturate(df[, "EKE"], both = F, alpha = 0.003)
  df[, "NPP"] <- saturate(df[, "NPP"], both = F, alpha = 0.003)
  df[, "Slope"] <- saturate(df[, "Slope"], both = F, alpha = 0.002)
  df[, "SSH"] <- saturate(df[, "SSH"], both = T, alpha = 0.0035)
  df[, "Temp"] <- saturate(df[, "Temp"], both = F, lower = T, alpha = 0.002)
  df[, "ZEu"] <- saturate(df[, "ZEu"], both = T, alpha = 0.002)
  
  return(df)
}) %>% data.table::rbindlist()



### 3 - Segments 0.25 --------------------------------------------------------
ENA_seg_025_b <- lapply(seq_along(simu), function(i){
  df <- ENA_seg_025 %>% subset(Simulation == simu[i])
  
  df[, "CurrentSpeed"] <- saturate(df[, "CurrentSpeed"], both = F, alpha = 0.0025)
  df[, "EKE"] <- saturate(df[, "EKE"], both = F, alpha = 0.0055)
  df[, "MLD"] <- saturate(df[, "MLD"], both = F, alpha = 0.0007)
  df[, "NPP"] <- saturate(df[, "NPP"], both = F, alpha = 0.005)
  df[, "Slope"] <- saturate(df[, "Slope"], both = F, alpha = 0.001)
  df[, "SSH"] <- saturate(df[, "SSH"], both = F, lower = T, alpha = 0.006)
  df[, "Temp"] <- saturate(df[, "Temp"], both = T, alpha = 0.004)
  df[, "ZEu"] <- saturate(df[, "ZEu"], both = F, alpha = 0.001)
  
  return(df)
}) %>% data.table::rbindlist()



### 4 - Segments 0.50 ---------------------------------------------------------
ENA_seg_050_b <- lapply(seq_along(simu), function(i){
  df <- ENA_seg_050 %>% subset(Simulation == simu[i])
  
  df[, "CurrentSpeed"] <- saturate(df[, "CurrentSpeed"], both = F, alpha = 0.005)
  df[, "EKE"] <- saturate(df[, "EKE"], both = F, alpha = 0.0065)
  df[, "NPP"] <- saturate(df[, "NPP"], both = F, alpha = 0.003)
  df[, "SSH"] <- saturate(df[, "SSH"], both = T, alpha = 0.006)
  df[, "Temp"] <- saturate(df[, "Temp"], both = T, alpha = 0.005)
  
  return(df)
}) %>% data.table::rbindlist()



### 5 - Segments 1.00 ---------------------------------------------------------
ENA_seg_100_b <- lapply(seq_along(simu), function(i){
  df <- ENA_seg_100 %>% subset(Simulation == simu[i])
  
  df[, "CurrentSpeed"] <- saturate(df[, "CurrentSpeed"], both = F, alpha = 0.0025)
  df[, "EKE"] <- saturate(df[, "EKE"], both = F, alpha = 0.0055)
  df[, "NPP"] <- saturate(df[, "NPP"], both = F, alpha = 0.003)
  df[, "Slope"] <- saturate(df[, "Slope"], both = F, alpha = 0.003)
  df[, "SSH"] <- saturate(df[, "SSH"], both = F, lower = T, alpha = 0.008)
  df[, "Temp"] <- saturate(df[, "Temp"], both = T, alpha = 0.006)
  df[, "ZEu"] <- saturate(df[, "ZEu"], both = F, lower = T, alpha = 0.001)
  
  return(df)
}) %>% data.table::rbindlist()



### 6 - Rasters 0.17 --------------------------------------------------------
ENA_017_b <- lapply(seq_along(simu), function(i){
  df <- as.data.frame(ENA_017) %>% subset(Simulation == simu[i])
  
  df[, "CurrentSpeed"] <- saturate(df[, "CurrentSpeed"], both = F, alpha = 0.002)
  df[, "EKE"] <- saturate(df[, "EKE"], both = F, alpha = 0.0025)
  df[, "NPP"] <- saturate(df[, "NPP"], both = F, alpha = 0.002)
  df[, "Slope"] <- saturate(df[, "Slope"], both = F, alpha = 0.001)
  df[, "SSH"] <- saturate(df[, "SSH"], both = T, alpha = 0.003)
  df[, "Temp"] <- saturate(df[, "Temp"], both = F, lower = T, alpha = 0.001)
  df[, "ZEu"] <- saturate(df[, "ZEu"], both = T, alpha = 0.001)
  
  return(df)
}) %>% data.table::rbindlist()



### 7 - Rasters 0.25 --------------------------------------------------------
ENA_025_b <- lapply(seq_along(simu), function(i){
  df <- as.data.frame(ENA_025) %>% subset(Simulation == simu[i])
  
  df[, "CurrentSpeed"] <- saturate(df[, "CurrentSpeed"], both = F, alpha = 0.002)
  df[, "EKE"] <- saturate(df[, "EKE"], both = F, alpha = 0.002)
  df[, "NPP"] <- saturate(df[, "NPP"], both = F, alpha = 0.002)
  df[, "SSH"] <- saturate(df[, "SSH"], both = T, alpha = 0.002)
  df[, "Temp"] <- saturate(df[, "Temp"], both = F, lower = T, alpha = 0.002)
  df[, "ZEu"] <- saturate(df[, "ZEu"], both = F, alpha = 0.001)
  
  return(df)
}) %>% data.table::rbindlist()



### 8 - Rasters 0.50 --------------------------------------------------------
ENA_050_b <- lapply(seq_along(simu), function(i){
  df <- as.data.frame(ENA_050) %>% subset(Simulation == simu[i])
  
  df[, "CurrentSpeed"] <- saturate(df[, "CurrentSpeed"], both = F, alpha = 0.006)
  df[, "EKE"] <- saturate(df[, "EKE"], both = F, alpha = 0.006)
  df[, "NPP"] <- saturate(df[, "NPP"], both = F, alpha = 0.005)
  df[, "SSH"] <- saturate(df[, "SSH"], both = T, alpha = 0.005)
  df[, "Temp"] <- saturate(df[, "Temp"], both = F, lower = T, alpha = 0.003)
  
  return(df)
}) %>% data.table::rbindlist()



### 9 - Rasters 1.00 --------------------------------------------------------
ENA_100_b <- lapply(seq_along(simu), function(i){
  df <- as.data.frame(ENA_100) %>% subset(Simulation == simu[i])
  
  df[, "CurrentSpeed"] <- saturate(df[, "CurrentSpeed"], both = F, alpha = 0.005)
  df[, "EKE"] <- saturate(df[, "EKE"], both = F, alpha = 0.006)
  df[, "Slope"] <- saturate(df[, "Slope"], both = F, alpha = 0.003)
  df[, "SSH"] <- saturate(df[, "SSH"], both = F, lower = T, alpha = 0.02)
  df[, "Temp"] <- saturate(df[, "Temp"], both = F, lower = T, alpha = 0.03)
  df[, "Temp"] <- saturate(df[, "Temp"], both = F, alpha = 0.003)
  df[, "ZEu"] <- saturate(df[, "ZEu"], both = F, lower = T, alpha = 0.002)
  
  return(df)
}) %>% data.table::rbindlist()



#### ------------------ ####
#### ----- Export ----- ####
#### ------------------ ####

write.table(ENA_seg_0083_b, paste(outdir, "ENA_seg_cov_0083.csv", sep = ""), dec = ".", sep = ";", row.names = F)
write.table(ENA_seg_017_b, paste(outdir, "ENA_seg_cov_017.csv", sep = ""), dec = ".", sep = ";", row.names = F)
write.table(ENA_seg_025_b, paste(outdir, "ENA_seg_cov_025.csv", sep = ""), dec = ".", sep = ";", row.names = F)
write.table(ENA_seg_050_b, paste(outdir, "ENA_seg_cov_050.csv", sep = ""), dec = ".", sep = ";", row.names = F)
write.table(ENA_seg_100_b, paste(outdir, "ENA_seg_cov_100.csv", sep = ""), dec = ".", sep = ";", row.names = F)

write.table(ENA_017_b, paste(outdir, "ENA_rast_cov_017.csv", sep = ""), dec = ".", sep = ";", row.names = F)
write.table(ENA_025_b, paste(outdir, "ENA_rast_cov_025.csv", sep = ""), dec = ".", sep = ";", row.names = F)
write.table(ENA_050_b, paste(outdir, "ENA_rast_cov_050.csv", sep = ""), dec = ".", sep = ";", row.names = F)
write.table(ENA_100_b, paste(outdir, "ENA_rast_cov_100.csv", sep = ""), dec = ".", sep = ";", row.names = F)




