##--------------------------------------------------------------------------------------------------------
## PROJECT : The effect of spatial resolution on SDM efficiency
## SCRIPT : Format survey data for modelling - Indian Ocean
##
## Author : Charlotte Lambert
## Last update : Jan 2022
## R version 4.0.4 (2021-02-15) -- "Lost Library Book"
##--------------------------------------------------------------------------------------------------------


library(dplyr)
library(tidyverse)
library(raster)
library(terra)
library(maps)
library(maptools)
library(ggplot2)
library(ggthemes)
library(ggspatial)
library(ggpubr)
library(RColorBreIOr)
library(reshape2)
library(lubridate)
library(viridis)
library(ncdf4)
library(sf)
library(spatstat)
library(spatstat.utils)
library(multidplyr)


#### ----------------------- ####
#### ---- SOURCES FILES ---- ####
#### ----------------------- ####
sourcedir <- "~/1_source/Variables/"
spdir <- "~/2_Species_simulations/"
surveydir <- "~/3_Survey_simulation/"
outdir <- "~/4_Formatting/"


### 1 - Open IO files ----------------------------------------------------------
bbx_WIO <- st_read(dsn = "~/1_source/GIS", layer = "BoundingBox_WIO")
land_WIO <- st_read(dsn = "~/1_source/GIS", layer = "Land_WIO_EPSG2737")  %>%
  st_transform(st_crs("epsg:4326"))
strata_WIO <- st_read(dsn = "~/1_source/GIS", layer = "Strata_WIO_clipped_EPSG2737", type = 3) %>%
  st_transform(st_crs("epsg:4326"))
transects_WIO <- st_read(dsn = "~/1_source/GIS", layer = "Transects_WIO_EPSG2737", type = 3)  %>%
  st_transform(st_crs("epsg:4326"))
segments_WIO <- st_read(dsn = "~/1_source/GIS", layer = "Segments_WIO_EPSG2737", type = 3) %>%
  rename(transect = transct, TransectLengthKm = TrnsLnK, SegLengthKm = SgLngtK, Longitude = Longitd, Latitude = Latitud) %>%
  st_transform(st_crs("epsg:4326"))


### 2 - Open species maps ------------------------------------------------------
WIOsp025_suit <- terra::rast(paste(spdir, "Species025_WIO_Suitability_EPSG2737.grd", sep = ""))
WIOsp025_occ <- terra::rast(paste(spdir, "Species025_WIO_ProbOcc_EPSG2737.grd", sep = ""))
WIOsp025_occ[WIOsp025_occ < 0] <- 0

WIOsp0083_suit <- terra::rast(paste(spdir, "Species0083_WIO_Suitability_EPSG2737.grd", sep = ""))
WIOsp0083_occ <- terra::rast(paste(spdir, "Species0083_WIO_ProbOcc_EPSG2737.grd", sep = ""))
WIOsp0083_occ[WIOsp0083_occ < 0] <- 0


### 3 - Open survey files ------------------------------------------------------
WIOsp025_seg <- read.table("~/3_Survey_simulation/WIO_sp025_Simulated_Survey_EPSG2737_effort_with_obs.csv", 
                        h = T, dec = ".", sep = ";")%>% rename(n_sp025 = n_detected)
simu <- unique(WIOsp025_seg$Simulation)

WIOsp0083_seg <- read.table("~/3_Survey_simulation/WIO_sp0083_Simulated_Survey_EPSG2737_effort_with_obs.csv", 
                        h = T, dec = ".", sep = ";")%>% rename(n_sp0083 = n_detected)

WIO_seg <- left_join(WIOsp025_seg, WIOsp0083_seg, by = c("x", "y", "Longitude", "Latitude", "strata", "transect", "TransectLengthKm", "IdSeg", "SegID", "SegLengthKm", "Simulation"))


### 4 - Open variables rasters -------------------------------------------------
stack_WIO_0083 <- stack(paste(sourcedir, "EnvironmentalVariables_WIO_0083.envi", sep = ""))
stack_WIO_017 <- stack(paste(sourcedir, "EnvironmentalVariables_WIO_017.envi", sep = ""))
stack_WIO_025 <- stack(paste(sourcedir, "EnvironmentalVariables_WIO_025.envi", sep = ""))
stack_WIO_050 <- stack(paste(sourcedir, "EnvironmentalVariables_WIO_050.envi", sep = ""))
stack_WIO_100 <- stack(paste(sourcedir, "EnvironmentalVariables_WIO_100.envi", sep = ""))



#### ---------------------- ####
#### ----- Rasterize  ----- ####
#### ---------------------- ####

### Rasterize to 0.17deg -------------------------------------------------------
WIO_017 <- NULL
WIO_017 <- lapply(seq_along(simu), function(i){
  df <- WIO_seg %>% subset(Simulation == simu[i])
  df_eff <- vect(x = df, crs = st_crs(4326), geom = c("Longitude", "Latitude"))
  
  obs_sp025 <- terra::rasterize(x = df_eff, field = "n_sp025", y = rast(stack_WIO_017), fun = sum) 
  obs_sp0083 <- terra::rasterize(x = df_eff, field = "n_sp0083", y = rast(stack_WIO_017), fun = sum) 
  eff <- terra::rasterize(x = df_eff, field = "SegLengthKm", y = rast(stack_WIO_017), fun = sum) 
  
  df_fin <- left_join(as.data.frame(eff, xy = T),
                      as.data.frame(obs_sp025, xy = T), by = c("x", "y")) %>%
    left_join(.,
              as.data.frame(obs_sp0083, xy = T), by = c("x", "y")) %>%
    mutate(Simulation = simu[i])
  colnames(df_fin) <- c("Longitude", "Latitude", "effort", "n_sp025", "n_sp0083", "Simulation")
  return(df_fin)
}) %>% data.table::rbindlist()


### Rasterize to 0.25deg -------------------------------------------------------
WIO_025 <- NULL
WIO_025 <- lapply(seq_along(simu), function(i){
  df <- WIO_seg %>% subset(Simulation == simu[i])
  df_eff <- vect(x = df, crs = st_crs(4326), geom = c("Longitude", "Latitude"))
  
  obs_sp025 <- terra::rasterize(x = df_eff, field = "n_sp025", y = rast(stack_WIO_025), fun = sum) 
  obs_sp0083 <- terra::rasterize(x = df_eff, field = "n_sp0083", y = rast(stack_WIO_025), fun = sum) 
  eff <- terra::rasterize(x = df_eff, field = "SegLengthKm", y = rast(stack_WIO_025), fun = sum) 
  
  df_fin <- left_join(as.data.frame(eff, xy = T),
                      as.data.frame(obs_sp025, xy = T), by = c("x", "y")) %>%
    left_join(.,
              as.data.frame(obs_sp0083, xy = T), by = c("x", "y")) %>%
    mutate(Simulation = simu[i])
  colnames(df_fin) <- c("Longitude", "Latitude", "effort", "n_sp025", "n_sp0083", "Simulation")
  return(df_fin)
}) %>% data.table::rbindlist()


### Rasterize to 0.50deg -------------------------------------------------------
WIO_050 <- NULL
WIO_050 <- lapply(seq_along(simu), function(i){
  df <- WIO_seg %>% subset(Simulation == simu[i])
  df_eff <- vect(x = df, crs = st_crs(4326), geom = c("Longitude", "Latitude"))
  
  obs_sp025 <- terra::rasterize(x = df_eff, field = "n_sp025", y = rast(stack_WIO_050), fun = sum) 
  obs_sp0083 <- terra::rasterize(x = df_eff, field = "n_sp0083", y = rast(stack_WIO_050), fun = sum) 
  eff <- terra::rasterize(x = df_eff, field = "SegLengthKm", y = rast(stack_WIO_050), fun = sum) 
  
  df_fin <- left_join(as.data.frame(eff, xy = T),
                      as.data.frame(obs_sp025, xy = T), by = c("x", "y")) %>%
    left_join(.,
              as.data.frame(obs_sp0083, xy = T), by = c("x", "y")) %>%
    mutate(Simulation = simu[i])
  colnames(df_fin) <- c("Longitude", "Latitude", "effort", "n_sp025", "n_sp0083", "Simulation")
  return(df_fin)
}) %>% data.table::rbindlist()


### Rasterize to 1.00deg -------------------------------------------------------
WIO_100 <- NULL
WIO_100 <- lapply(seq_along(simu), function(i){
  df <- WIO_seg %>% subset(Simulation == simu[i])
  df_eff <- vect(x = df, crs = st_crs(4326), geom = c("Longitude", "Latitude"))
  
  obs_sp025 <- terra::rasterize(x = df_eff, field = "n_sp025", y = rast(stack_WIO_100), fun = sum) 
  obs_sp0083 <- terra::rasterize(x = df_eff, field = "n_sp0083", y = rast(stack_WIO_100), fun = sum) 
  eff <- terra::rasterize(x = df_eff, field = "SegLengthKm", y = rast(stack_WIO_100), fun = sum) 
  
  df_fin <- left_join(as.data.frame(eff, xy = T),
                      as.data.frame(obs_sp025, xy = T), by = c("x", "y")) %>%
    left_join(.,
              as.data.frame(obs_sp0083, xy = T), by = c("x", "y")) %>%
    mutate(Simulation = simu[i])
  colnames(df_fin) <- c("Longitude", "Latitude", "effort", "n_sp025", "n_sp0083", "Simulation")
  return(df_fin)
}) %>% data.table::rbindlist()




#### ----------------------- ####
#### ----- Plots maps  ----- ####
#### ----------------------- ####

# layout
layout <- theme_bw() +
  theme(legend.position = "bottom",
        legend.key.width = unit(0.5, "cm"),
        legend.text = element_text(size = 8),
        plot.title = element_text(lineheight = 0.8, face = "bold", size = 14),
        axis.text = element_text(size = 12),
        strip.background = element_rect(fill = "white"),
        axis.title = element_blank(),
        panel.background = element_rect(fill = "white"),
        panel.border = element_rect(fill = NA),
        panel.grid = element_line(colour = "grey80"), #element_blank(),
        legend.title = element_blank()
  )


### 1 - Plots efforts ----------------------------------------------------------
eff_017 <- ggplot() + 
  geom_raster(data = WIO_017 %>% subset(Simulation == "Simulation 1"), aes(x = Longitude, y = Latitude, fill = effort)) + 
  scale_fill_viridis(na.value = NA, option = "H") + 
  layer_spatial(land_WIO, fill = "black", color = NA) + 
  layer_spatial(strata_WIO, fill = NA, color = "black") + 
  layer_spatial(bbx_WIO, color = "black", size = 1, fill = NA)+
  annotation_scale(location = "bl", width_hint = 0.5) +
  annotation_north_arrow(location = "tr",
                         which_north = "true",
                         pad_x = unit(0.2, "cm"),
                         pad_y = unit(0.1, "cm"),
                         style = north_arrow_fancy_orienteering
  ) + layout + 
  ggtitle("0.17")

eff_025 <- ggplot() + 
  geom_raster(data = WIO_025 %>% subset(Simulation == "Simulation 1"), aes(x = Longitude, y = Latitude, fill = effort)) + 
  scale_fill_viridis(na.value = NA, option = "H") + 
  layer_spatial(land_WIO, fill = "black", color = NA) + 
  layer_spatial(strata_WIO, fill = NA, color = "black") + 
  layer_spatial(bbx_WIO, color = "black", size = 1, fill = NA)+
  annotation_scale(location = "bl", width_hint = 0.5) +
  annotation_north_arrow(location = "tr",
                         which_north = "true",
                         pad_x = unit(0.2, "cm"),
                         pad_y = unit(0.1, "cm"),
                         style = north_arrow_fancy_orienteering
  ) + layout + 
  ggtitle("0.25")

eff_050 <- ggplot() + 
  geom_raster(data = WIO_050 %>% subset(Simulation == "Simulation 1"), aes(x = Longitude, y = Latitude, fill = effort)) + 
  scale_fill_viridis(na.value = NA, option = "H") + 
  layer_spatial(land_WIO, fill = "black", color = NA) + 
  layer_spatial(strata_WIO, fill = NA, color = "black") + 
  layer_spatial(bbx_WIO, color = "black", size = 1, fill = NA)+
  annotation_scale(location = "bl", width_hint = 0.5) +
  annotation_north_arrow(location = "tr",
                         which_north = "true",
                         pad_x = unit(0.2, "cm"),
                         pad_y = unit(0.1, "cm"),
                         style = north_arrow_fancy_orienteering
  ) + layout + 
  ggtitle("0.50")

eff_100 <- ggplot() + 
  geom_raster(data = WIO_100 %>% subset(Simulation == "Simulation 1"), aes(x = Longitude, y = Latitude, fill = effort)) + 
  scale_fill_viridis(na.value = NA, option = "H") + 
  layer_spatial(land_WIO, fill = "black", color = NA) + 
  layer_spatial(strata_WIO, fill = NA, color = "black") + 
  layer_spatial(bbx_WIO, color = "black", size = 1, fill = NA)+
  annotation_scale(location = "bl", width_hint = 0.5) +
  annotation_north_arrow(location = "tr",
                         which_north = "true",
                         pad_x = unit(0.2, "cm"),
                         pad_y = unit(0.1, "cm"),
                         style = north_arrow_fancy_orienteering
  ) + layout + 
  ggtitle("1.00")

ggarrange(eff_017, eff_025, eff_050, eff_100, nrow = 1, ncol = 4, align = "hv") + bgcolor("white")
ggsave("~/4_Formatting/WIO_Rasterized_Effort.png", dpi = 350, height = 8, width = 15)


save.image("~/4_Formatting/WIO_Rasterize.RData")


### 2 - Plot Species 1 ---------------------------------------------------------
sp025_017 <- ggplot() + 
  geom_raster(data = WIO_017 %>% subset(Simulation == "Simulation 1"), aes(x = Longitude, y = Latitude, fill = n_sp025)) + 
  scale_fill_viridis(na.value = NA, option = "H") + 
  layer_spatial(land_WIO, fill = "black", color = NA) + 
  layer_spatial(strata_WIO, fill = NA, color = "black") + 
  layer_spatial(bbx_WIO, color = "black", size = 1, fill = NA)+
  annotation_scale(location = "bl", width_hint = 0.5) +
  annotation_north_arrow(location = "tr",
                         which_north = "true",
                         pad_x = unit(0.2, "cm"),
                         pad_y = unit(0.1, "cm"),
                         style = north_arrow_fancy_orienteering
  ) + layout + 
  ggtitle("0.17")

sp025_025 <- ggplot() + 
  geom_raster(data = WIO_025 %>% subset(Simulation == "Simulation 1"), aes(x = Longitude, y = Latitude, fill = n_sp025)) + 
  scale_fill_viridis(na.value = NA, option = "H") + 
  layer_spatial(land_WIO, fill = "black", color = NA) + 
  layer_spatial(strata_WIO, fill = NA, color = "black") + 
  layer_spatial(bbx_WIO, color = "black", size = 1, fill = NA)+
  annotation_scale(location = "bl", width_hint = 0.5) +
  annotation_north_arrow(location = "tr",
                         which_north = "true",
                         pad_x = unit(0.2, "cm"),
                         pad_y = unit(0.1, "cm"),
                         style = north_arrow_fancy_orienteering
  ) + layout + 
  ggtitle("0.25")

sp025_050 <- ggplot() + 
  geom_raster(data = WIO_050 %>% subset(Simulation == "Simulation 1"), aes(x = Longitude, y = Latitude, fill = n_sp025)) + 
  scale_fill_viridis(na.value = NA, option = "H") + 
  layer_spatial(land_WIO, fill = "black", color = NA) + 
  layer_spatial(strata_WIO, fill = NA, color = "black") + 
  layer_spatial(bbx_WIO, color = "black", size = 1, fill = NA)+
  annotation_scale(location = "bl", width_hint = 0.5) +
  annotation_north_arrow(location = "tr",
                         which_north = "true",
                         pad_x = unit(0.2, "cm"),
                         pad_y = unit(0.1, "cm"),
                         style = north_arrow_fancy_orienteering
  ) + layout + 
  ggtitle("0.50")

sp025_100 <- ggplot() + 
  geom_raster(data = WIO_100 %>% subset(Simulation == "Simulation 1"), aes(x = Longitude, y = Latitude, fill = n_sp025)) + 
  scale_fill_viridis(na.value = NA, option = "H") + 
  layer_spatial(land_WIO, fill = "black", color = NA) + 
  layer_spatial(strata_WIO, fill = NA, color = "black") + 
  layer_spatial(bbx_WIO, color = "black", size = 1, fill = NA)+
  annotation_scale(location = "bl", width_hint = 0.5) +
  annotation_north_arrow(location = "tr",
                         which_north = "true",
                         pad_x = unit(0.2, "cm"),
                         pad_y = unit(0.1, "cm"),
                         style = north_arrow_fancy_orienteering
  ) + layout + 
  ggtitle("1.00")

ggarrange(sp025_017, sp025_025, sp025_050, sp025_100, nrow = 1, ncol = 4, align = "hv") + bgcolor("white")
ggsave("~/4_Formatting/WIO_Rasterized_Species025_NObs.png", dpi = 350, height = 8, width = 15)


### 3 - Plot Species 2 ---------------------------------------------------------
sp0083_017 <- ggplot() + 
  geom_raster(data = WIO_017 %>% subset(Simulation == "Simulation 1"), aes(x = Longitude, y = Latitude, fill = n_sp0083)) + 
  scale_fill_viridis(na.value = NA, option = "H") + 
  layer_spatial(land_WIO, fill = "black", color = NA) + 
  layer_spatial(strata_WIO, fill = NA, color = "black") + 
  layer_spatial(bbx_WIO, color = "black", size = 1, fill = NA)+
  annotation_scale(location = "bl", width_hint = 0.5) +
  annotation_north_arrow(location = "tr",
                         which_north = "true",
                         pad_x = unit(0.2, "cm"),
                         pad_y = unit(0.1, "cm"),
                         style = north_arrow_fancy_orienteering
  ) + layout + 
  ggtitle("0.17")

sp0083_025 <- ggplot() + 
  geom_raster(data = WIO_025 %>% subset(Simulation == "Simulation 1"), aes(x = Longitude, y = Latitude, fill = n_sp0083)) + 
  scale_fill_viridis(na.value = NA, option = "H") + 
  layer_spatial(land_WIO, fill = "black", color = NA) + 
  layer_spatial(strata_WIO, fill = NA, color = "black") + 
  layer_spatial(bbx_WIO, color = "black", size = 1, fill = NA)+
  annotation_scale(location = "bl", width_hint = 0.5) +
  annotation_north_arrow(location = "tr",
                         which_north = "true",
                         pad_x = unit(0.2, "cm"),
                         pad_y = unit(0.1, "cm"),
                         style = north_arrow_fancy_orienteering
  ) + layout + 
  ggtitle("0.25")

sp0083_050 <- ggplot() + 
  geom_raster(data = WIO_050 %>% subset(Simulation == "Simulation 1"), aes(x = Longitude, y = Latitude, fill = n_sp0083)) + 
  scale_fill_viridis(na.value = NA, option = "H") + 
  layer_spatial(land_WIO, fill = "black", color = NA) + 
  layer_spatial(strata_WIO, fill = NA, color = "black") + 
  layer_spatial(bbx_WIO, color = "black", size = 1, fill = NA)+
  annotation_scale(location = "bl", width_hint = 0.5) +
  annotation_north_arrow(location = "tr",
                         which_north = "true",
                         pad_x = unit(0.2, "cm"),
                         pad_y = unit(0.1, "cm"),
                         style = north_arrow_fancy_orienteering
  ) + layout + 
  ggtitle("0.50")

sp0083_100 <- ggplot() + 
  geom_raster(data = WIO_100 %>% subset(Simulation == "Simulation 1"), aes(x = Longitude, y = Latitude, fill = n_sp0083)) + 
  scale_fill_viridis(na.value = NA, option = "H") + 
  layer_spatial(land_WIO, fill = "black", color = NA) + 
  layer_spatial(strata_WIO, fill = NA, color = "black") + 
  layer_spatial(bbx_WIO, color = "black", size = 1, fill = NA)+
  annotation_scale(location = "bl", width_hint = 0.5) +
  annotation_north_arrow(location = "tr",
                         which_north = "true",
                         pad_x = unit(0.2, "cm"),
                         pad_y = unit(0.1, "cm"),
                         style = north_arrow_fancy_orienteering
  ) + layout + 
  ggtitle("1.00")

ggarrange(sp0083_017, sp0083_025, sp0083_050, sp0083_100, nrow = 1, ncol = 4, align = "hv") + bgcolor("white")
ggsave("~/4_Formatting/WIO_Rasterized_Species0083_NObs.png", dpi = 350, height = 8, width = 15)


### 4 - Plot all ---------------------------------------------------------------
ggarrange(eff_017, eff_025, eff_050, eff_100,
          sp0083_017, sp0083_025, sp0083_050, sp0083_100, 
          sp025_017, sp025_025, sp025_050, sp025_100, nrow = 3, ncol = 4, align = "hv") + bgcolor("white")
ggsave("~/4_Formatting/WIO_Rasterized_tot.png", dpi = 350, height = 18, width = 15)

save.image("~/4_Formatting/WIO_Rasterize.RData")



#### ------------------------------ ####
#### ----- Extract covariates ----- ####
#### ------------------------------ ####

## segments
WIO_seg_cov0083 <- raster::extract(stack_WIO_0083, y = WIO_seg[, c("Longitude", "Latitude")])
WIO_seg_0083 <- cbind(WIO_seg, WIO_seg_cov0083[,1:13])

WIO_seg_cov017 <- raster::extract(stack_WIO_017, y = WIO_seg[, c("Longitude", "Latitude")])
WIO_seg_017 <- cbind(WIO_seg, WIO_seg_cov017[,1:13])

WIO_seg_cov025 <- raster::extract(stack_WIO_025, y = WIO_seg[, c("Longitude", "Latitude")])
WIO_seg_025 <- cbind(WIO_seg, WIO_seg_cov025[,1:13])

WIO_seg_cov050 <- raster::extract(stack_WIO_050, y = WIO_seg[, c("Longitude", "Latitude")])
WIO_seg_050 <- cbind(WIO_seg, WIO_seg_cov050[,1:13])

WIO_seg_cov100 <- raster::extract(stack_WIO_100, y = WIO_seg[, c("Longitude", "Latitude")])
WIO_seg_100 <- cbind(WIO_seg, WIO_seg_cov100[,1:13])

## 017
WIO_017_cov <- raster::extract(stack_WIO_017, y = WIO_017[, c("Longitude", "Latitude")])
WIO_017 <- cbind(WIO_017, WIO_017_cov[,1:13])

## 025
WIO_025_cov <- raster::extract(stack_WIO_025, y = WIO_025[, c("Longitude", "Latitude")])
WIO_025 <- cbind(WIO_025, WIO_025_cov[,1:13])

## 050
WIO_050_cov <- raster::extract(stack_WIO_050, y = WIO_050[, c("Longitude", "Latitude")])
WIO_050 <- cbind(WIO_050, WIO_050_cov[,1:13])

## 100
WIO_100_cov <- raster::extract(stack_WIO_100, y = WIO_100[, c("Longitude", "Latitude")])
WIO_100 <- cbind(WIO_100, WIO_100_cov[,1:13])



save.image("~/4_Formatting/WIO_Rasterize.RData")



#### ------------------------------ ####
#### ----- Saturate variables ----- ####
#### ------------------------------ ####

# function
saturate <- function(x, threshold = NULL, lower = FALSE, alpha = 0.01, both = FALSE) {
  if(both) {
    if(is.null(threshold)) {
      threshold <- quantile(x, c(alpha/2, 1-alpha/2), na.rm = TRUE)
    }
    if(length(threshold) != 2){
      stop("must provide a lower and upper thresholds")
    }
    x <- ifelse(x < min(threshold), min(threshold), x)
    x <- ifelse(x > max(threshold), max(threshold), x)
  }
  else{
    if(is.null(threshold)) {
      threshold.lower <- quantile(x, alpha/2, na.rm = TRUE)
      threshold.upper <- quantile(x, 1-alpha/2, na.rm = TRUE)
      if(lower) { x <- ifelse(x < threshold.lower, threshold.lower, x) }
      else { x <- ifelse(x > threshold.upper, threshold.upper, x) }
    } else {
      if(lower) { x <- ifelse(x < threshold, threshold, x) }
      else { x <- ifelse(x > threshold, threshold, x) }
    }
    
  }
  return(x)
}
vars <- names(stack_WIO_0083)[1:9]


### 1 - Segments 0.083 --------------------------------------------------------
WIO_seg_0083_b <- lapply(seq_along(simu), function(i){
  df <- WIO_seg_0083 %>% subset(Simulation == simu[i])
  
  df[, "CurrentSpeed"] <- saturate(df[, "CurrentSpeed"], both = F, alpha = 0.001)
  df[, "EKE"] <- saturate(df[, "EKE"], both = F, alpha = 0.001)
  df[, "MLD"] <- saturate(df[, "MLD"], both = F, alpha = 0.001)
  df[, "NPP"] <- saturate(df[, "NPP"], both = F, alpha = 0.002)
  df[, "Slope"] <- saturate(df[, "Slope"], both = F, alpha = 0.002)
  df[, "Temp"] <- saturate(df[, "Temp"], both = F, lower = T, alpha = 0.002)
  df[, "ZEu"] <- saturate(df[, "ZEu"], both = T, alpha = 0.001)
  
  return(df)
}) %>% data.table::rbindlist()



### 2 - Segments 0.17 ---------------------------------------------------------
WIO_seg_017_b <- lapply(seq_along(simu), function(i){
  df <- WIO_seg_017 %>% subset(Simulation == simu[i])
  
  df[, "MLD"] <- saturate(df[, "MLD"], both = F, alpha = 0.003)
  df[, "NPP"] <- saturate(df[, "NPP"], both = F, alpha = 0.003)
  df[, "Slope"] <- saturate(df[, "Slope"], both = F, alpha = 0.001)
  df[, "Temp"] <- saturate(df[, "Temp"], both = F, lower = T, alpha = 0.002)
  df[, "Temp"] <- saturate(df[, "Temp"], both = F, alpha = 0.001)
  df[, "ZEu"] <- saturate(df[, "ZEu"], both = T, alpha = 0.001)
  
  return(df)
}) %>% data.table::rbindlist()



### 3 - Segments 0.25 --------------------------------------------------------
WIO_seg_025_b <- lapply(seq_along(simu), function(i){
  df <- WIO_seg_025 %>% subset(Simulation == simu[i])
  
  df[, "EKE"] <- saturate(df[, "EKE"], both = F, alpha = 0.005)
  df[, "MLD"] <- saturate(df[, "MLD"], both = F, alpha = 0.002)
  # df[, "NPP"] <- saturate(df[, "NPP"], both = F, alpha = 0.001)
  df[, "Slope"] <- saturate(df[, "Slope"], both = F, alpha = 0.002)
  df[, "Temp"] <- saturate(df[, "Temp"], both = T, alpha = 0.004)
  
  return(df)
}) %>% data.table::rbindlist()



### 4 - Segments 0.50 ---------------------------------------------------------
WIO_seg_050_b <- lapply(seq_along(simu), function(i){
  df <- WIO_seg_050 %>% subset(Simulation == simu[i])
  
  df[, "CurrentSpeed"] <- saturate(df[, "CurrentSpeed"], both = F, alpha = 0.007)
  df[, "EKE"] <- saturate(df[, "EKE"], both = F, alpha = 0.01)
  df[, "MLD"] <- saturate(df[, "MLD"], both = F, alpha = 0.008)
  df[, "MLD"] <- saturate(df[, "MLD"], both = F, lower = T, alpha = 0.002)
  df[, "NPP"] <- saturate(df[, "NPP"], both = F, alpha = 0.008)
  df[, "Slope"] <- saturate(df[, "Slope"], both = F, alpha = 0.03)
  df[, "SSH"] <- saturate(df[, "SSH"], both = T, alpha = 0.02)
  df[, "Temp"] <- saturate(df[, "Temp"], both = T, alpha = 0.005)
  df[, "ZEu"] <- saturate(df[, "ZEu"], both = F, lower = T, alpha = 0.001)
  
  return(df)
}) %>% data.table::rbindlist()



### 5 - Segments 1.00 ---------------------------------------------------------
WIO_seg_100_b <- lapply(seq_along(simu), function(i){
  df <- WIO_seg_100 %>% subset(Simulation == simu[i])
  
  df[, "CurrentSpeed"] <- saturate(df[, "CurrentSpeed"], both = F, alpha = 0.09)
  df[, "EKE"] <- saturate(df[, "EKE"], both = F, alpha = 0.1)
  df[, "MLD"] <- saturate(df[, "MLD"], both = F, alpha = 0.01)
  df[, "NPP"] <- saturate(df[, "NPP"], both = F, alpha = 0.003)
  df[, "Slope"] <- saturate(df[, "Slope"], both = F, alpha = 0.02)
  df[, "SSH"] <- saturate(df[, "SSH"], both = T, alpha = 0.02)
  df[, "Temp"] <- saturate(df[, "Temp"], both = T, alpha = 0.02)
  df[, "ZEu"] <- saturate(df[, "ZEu"], both = F, lower = T, alpha = 0.02)
  
  return(df)
}) %>% data.table::rbindlist()



### 6 - Rasters 0.17 --------------------------------------------------------
WIO_017_b <- lapply(seq_along(simu), function(i){
  df <- as.data.frame(WIO_017) %>% subset(Simulation == simu[i])
  
  df[, "EKE"] <- saturate(df[, "EKE"], both = F, alpha = 0.002)
  df[, "MLD"] <- saturate(df[, "MLD"], both = F, alpha = 0.002)
  df[, "NPP"] <- saturate(df[, "NPP"], both = F, alpha = 0.0045)
  df[, "Slope"] <- saturate(df[, "Slope"], both = F, alpha = 0.002)
  df[, "Temp"] <- saturate(df[, "Temp"], both = F, alpha = 0.001)
  df[, "Temp"] <- saturate(df[, "Temp"], both = F, lower = T, alpha = 0.002)
  df[, "ZEu"] <- saturate(df[, "ZEu"], both = T, alpha = 0.001)
  
  return(df)
}) %>% data.table::rbindlist()



### 7 - Rasters 0.25 --------------------------------------------------------
WIO_025_b <- lapply(seq_along(simu), function(i){
  df <- as.data.frame(WIO_025) %>% subset(Simulation == simu[i])
  
  df[, "EKE"] <- saturate(df[, "EKE"], both = F, alpha = 0.005)
  df[, "MLD"] <- saturate(df[, "MLD"], both = F, alpha = 0.002)
  df[, "NPP"] <- saturate(df[, "NPP"], both = F, alpha = 0.001)
  df[, "Slope"] <- saturate(df[, "Slope"], both = F, alpha = 0.002)
  df[, "Temp"] <- saturate(df[, "Temp"], both = F, lower = T, alpha = 0.003)
  
  return(df)
}) %>% data.table::rbindlist()



### 8 - Rasters 0.50 --------------------------------------------------------
WIO_050_b <- lapply(seq_along(simu), function(i){
  df <- as.data.frame(WIO_050) %>% subset(Simulation == simu[i])
  
  df[, "CurrentSpeed"] <- saturate(df[, "CurrentSpeed"], both = F, alpha = 0.005)
  df[, "EKE"] <- saturate(df[, "EKE"], both = F, alpha = 0.006)
  df[, "MLD"] <- saturate(df[, "MLD"], both = F, alpha = 0.006)
  df[, "NPP"] <- saturate(df[, "NPP"], both = F, alpha = 0.03)
  df[, "Slope"] <- saturate(df[, "Slope"], both = F, alpha = 0.02)
  # df[, "SSH"] <- saturate(df[, "SSH"], both = T, alpha = 0.005)
  df[, "Temp"] <- saturate(df[, "Temp"], both = F, lower = T, alpha = 0.005)
  df[, "ZEu"] <- saturate(df[, "ZEu"], both = F, lower = T, alpha = 0.001)
  
  return(df)
}) %>% data.table::rbindlist()



### 9 - Rasters 1.00 --------------------------------------------------------
WIO_100_b <- lapply(seq_along(simu), function(i){
  df <- as.data.frame(WIO_100) %>% subset(Simulation == simu[i])
  
  # df[, "CurrentSpeed"] <- saturate(df[, "CurrentSpeed"], both = F, alpha = 0.01)
  # df[, "EKE"] <- saturate(df[, "EKE"], both = F, alpha = 0.01)
  df[, "MLD"] <- saturate(df[, "MLD"], both = F, alpha = 0.01)
  df[, "Slope"] <- saturate(df[, "Slope"], both = F, alpha = 0.008)
  df[, "SSH"] <- saturate(df[, "SSH"], both = F, alpha = 0.02)
  df[, "SSH"] <- saturate(df[, "SSH"], both = F, lower = T, alpha = 0.005)
  df[, "Temp"] <- saturate(df[, "Temp"], both = F, lower = T, alpha = 0.03)
  df[, "Temp"] <- saturate(df[, "Temp"], both = F, alpha = 0.004)
  df[, "ZEu"] <- saturate(df[, "ZEu"], both = F, lower = T, alpha = 0.005)
  
  return(df)
}) %>% data.table::rbindlist()


#### ------------------ ####
#### ----- Export ----- ####
#### ------------------ ####

write.table(WIO_seg_0083_b, paste(outdir, "WIO_seg_cov_0083.csv", sep = ""), dec = ".", sep = ";", row.names = F)
write.table(WIO_seg_017_b, paste(outdir, "WIO_seg_cov_017.csv", sep = ""), dec = ".", sep = ";", row.names = F)
write.table(WIO_seg_025_b, paste(outdir, "WIO_seg_cov_025.csv", sep = ""), dec = ".", sep = ";", row.names = F)
write.table(WIO_seg_050_b, paste(outdir, "WIO_seg_cov_050.csv", sep = ""), dec = ".", sep = ";", row.names = F)
write.table(WIO_seg_100_b, paste(outdir, "WIO_seg_cov_100.csv", sep = ""), dec = ".", sep = ";", row.names = F)

write.table(WIO_017_b, paste(outdir, "WIO_rast_cov_017.csv", sep = ""), dec = ".", sep = ";", row.names = F)
write.table(WIO_025_b, paste(outdir, "WIO_rast_cov_025.csv", sep = ""), dec = ".", sep = ";", row.names = F)
write.table(WIO_050_b, paste(outdir, "WIO_rast_cov_050.csv", sep = ""), dec = ".", sep = ";", row.names = F)
write.table(WIO_100_b, paste(outdir, "WIO_rast_cov_100.csv", sep = ""), dec = ".", sep = ";", row.names = F)





