##--------------------------------------------------------------------------------------------------------
## PROJECT : The effect of spatial resolution on SDM efficiency
## SCRIPT : Generate survey
##
## Author : Charlotte Lambert
## Last update : Jan 2022
## R version 4.0.4 (2021-02-15) -- "Lost Library Book"
##--------------------------------------------------------------------------------------------------------


library(dplyr)
library(tidyverse)
library(terra)
library(maps)
library(maptools)
library(ggplot2)
library(ggthemes)
library(ggspatial)
library(ggpubr)
library(RColorBrewer)
library(reshape2)
library(lubridate)
library(viridis)
library(ncdf4)
library(sf)
library(spatstat)
library(spatstat.utils)
library(multidplyr)
library(layer)

#### ----------------------- ####
#### ---- SOURCES FILES ---- ####
#### ----------------------- ####
datadir <- "~/4_Formatting/"

### 1 - Open effort files - ENA ------------------------------------------------
seg0083_ENA <- read.table(paste(datadir, "ENA_seg_cov_0083.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Seg_0083", region = "ENA") %>% rename(effort = SegLengthKm, simulation=)
seg017_ENA <- read.table(paste(datadir, "ENA_seg_cov_017.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Seg_017", region = "ENA") %>% rename(effort = SegLengthKm)
seg025_ENA <- read.table(paste(datadir, "ENA_seg_cov_025.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Seg_025", region = "ENA") %>% rename(effort = SegLengthKm)
seg050_ENA <- read.table(paste(datadir, "ENA_seg_cov_050.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Seg_050", region = "ENA") %>% rename(effort = SegLengthKm)
seg100_ENA <- read.table(paste(datadir, "ENA_seg_cov_100.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Seg_100", region = "ENA") %>% rename(effort = SegLengthKm)

rast017_ENA <- read.table(paste(datadir, "ENA_rast_cov_017.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Rast_017", region = "ENA")
rast025_ENA <- read.table(paste(datadir, "ENA_rast_cov_025.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Rast_025", region = "ENA")
rast050_ENA <- read.table(paste(datadir, "ENA_rast_cov_050.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Rast_050", region = "ENA")
rast100_ENA <- read.table(paste(datadir, "ENA_rast_cov_100.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Rast_100", region = "ENA")

vars <- c("Bathy", "Slope", "Temp", "SSH", "NPP", "MLD", "ZEu", "CurrentSpeed", "EKE")

data_ENA <- rbind(seg0083_ENA[,c("Longitude", "Latitude", "effort", "n_sp0083", "n_sp025",
                                 "Simulation", "sampling_resolution", "region", vars)],
                  seg017_ENA[,c("Longitude", "Latitude", "effort", "n_sp0083", "n_sp025",
                                "Simulation", "sampling_resolution", "region", vars)],
                  seg025_ENA[,c("Longitude", "Latitude", "effort", "n_sp0083", "n_sp025",
                                "Simulation", "sampling_resolution", "region", vars)],
                  seg050_ENA[,c("Longitude", "Latitude", "effort", "n_sp0083", "n_sp025",
                                "Simulation", "sampling_resolution", "region", vars)],
                  seg100_ENA[,c("Longitude", "Latitude", "effort", "n_sp0083", "n_sp025",
                                "Simulation", "sampling_resolution", "region", vars)],
                  rast017_ENA[,c("Longitude", "Latitude", "effort", "n_sp0083", "n_sp025",
                                 "Simulation", "sampling_resolution", "region", vars)],
                  rast025_ENA[,c("Longitude", "Latitude", "effort", "n_sp0083", "n_sp025",
                                 "Simulation", "sampling_resolution", "region", vars)],
                  rast050_ENA[,c("Longitude", "Latitude", "effort", "n_sp0083", "n_sp025",
                                 "Simulation", "sampling_resolution", "region", vars)],
                  rast100_ENA[,c("Longitude", "Latitude", "effort", "n_sp0083", "n_sp025",
                                 "Simulation", "sampling_resolution", "region", vars)])

rm(seg0083_ENA, seg017_ENA, seg025_ENA, seg050_ENA, seg100_ENA, rast017_ENA, rast025_ENA, rast050_ENA, rast100_ENA)


### 2 - Open effort files - WIO ------------------------------------------------
seg0083_WIO <- read.table(paste(datadir, "WIO_seg_cov_0083.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Seg_0083", region = "WIO") %>% rename(effort = SegLengthKm, simulation=)
seg017_WIO <- read.table(paste(datadir, "WIO_seg_cov_017.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Seg_017", region = "WIO") %>% rename(effort = SegLengthKm)
seg025_WIO <- read.table(paste(datadir, "WIO_seg_cov_025.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Seg_025", region = "WIO") %>% rename(effort = SegLengthKm)
seg050_WIO <- read.table(paste(datadir, "WIO_seg_cov_050.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Seg_050", region = "WIO") %>% rename(effort = SegLengthKm)
seg100_WIO <- read.table(paste(datadir, "WIO_seg_cov_100.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Seg_100", region = "WIO") %>% rename(effort = SegLengthKm)

rast017_WIO <- read.table(paste(datadir, "WIO_rast_cov_017.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Rast_017", region = "WIO")
rast025_WIO <- read.table(paste(datadir, "WIO_rast_cov_025.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Rast_025", region = "WIO")
rast050_WIO <- read.table(paste(datadir, "WIO_rast_cov_050.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Rast_050", region = "WIO")
rast100_WIO <- read.table(paste(datadir, "WIO_rast_cov_100.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Rast_100", region = "WIO")

data_WIO <- rbind(seg0083_WIO[,c("Longitude", "Latitude", "effort", "n_sp025", "n_sp0083",
                                 "Simulation", "sampling_resolution", "region", vars)],
                  seg017_WIO[,c("Longitude", "Latitude", "effort", "n_sp025", "n_sp0083",
                                "Simulation", "sampling_resolution", "region", vars)],
                  seg025_WIO[,c("Longitude", "Latitude", "effort", "n_sp025", "n_sp0083",
                                "Simulation", "sampling_resolution", "region", vars)],
                  seg050_WIO[,c("Longitude", "Latitude", "effort", "n_sp025", "n_sp0083",
                                "Simulation", "sampling_resolution", "region", vars)],
                  seg100_WIO[,c("Longitude", "Latitude", "effort", "n_sp025", "n_sp0083",
                                "Simulation", "sampling_resolution", "region", vars)],
                  rast017_WIO[,c("Longitude", "Latitude", "effort", "n_sp025", "n_sp0083",
                                 "Simulation", "sampling_resolution", "region", vars)],
                  rast025_WIO[,c("Longitude", "Latitude", "effort", "n_sp025", "n_sp0083",
                                 "Simulation", "sampling_resolution", "region", vars)],
                  rast050_WIO[,c("Longitude", "Latitude", "effort", "n_sp025", "n_sp0083",
                                 "Simulation", "sampling_resolution", "region", vars)],
                  rast100_WIO[,c("Longitude", "Latitude", "effort", "n_sp025", "n_sp0083",
                                 "Simulation", "sampling_resolution", "region", vars)])

rm(seg0083_WIO, seg017_WIO, seg025_WIO, seg050_WIO, seg100_WIO, rast017_WIO, rast025_WIO, rast050_WIO, rast100_WIO)



### 2 - Summarize --------------------------------------------------------------
ENA_sp0083 <- data_ENA %>% group_by(region, Simulation, sampling_resolution) %>% 
  subset(n_sp0083 != 0) %>%
  summarize(sum_N = sum(n_sp0083, na.rm = T),
            count = length(n_sp0083)) %>%
  mutate(Species = "Species 0.083") %>% ungroup()
ENA_sp025 <- data_ENA %>% group_by(region, Simulation, sampling_resolution) %>% 
  subset(n_sp025 != 0) %>%
  summarize(sum_N = sum(n_sp025, na.rm = T),
            count = length(n_sp025)) %>%
  mutate(Species = "Species 0.25") %>% ungroup()

WIO_sp0083 <- data_WIO %>% group_by(region, Simulation, sampling_resolution) %>% 
  subset(n_sp0083 != 0) %>%
  summarize(sum_N = sum(n_sp0083, na.rm = T),
            count = length(n_sp0083)) %>%
  mutate(Species = "Species 0.083") %>% ungroup()
WIO_sp025 <- data_WIO %>% group_by(region, Simulation, sampling_resolution) %>% 
  subset(n_sp025 != 0) %>%
  summarize(sum_N = sum(n_sp025, na.rm = T),
            count = length(n_sp025)) %>%
  mutate(Species = "Species 0.25") %>% ungroup()

ENA <- data_ENA %>% group_by(region, Simulation, sampling_resolution) %>% 
  summarize(sum_effort = sum(effort, na.rm = T),
            count_effort = length(effort)) %>% ungroup()

WIO <- data_WIO %>% group_by(region, Simulation, sampling_resolution) %>% 
  summarize(sum_effort = sum(effort, na.rm = T),
            count_effort = length(effort)) %>% ungroup()



### 3 - Plot -------------------------------------------------------------------
# join tables
smry <- rbind(left_join(ENA_sp0083, ENA), 
              left_join(ENA_sp025, ENA), 
              left_join(WIO_sp0083, WIO), 
              left_join(WIO_sp025, WIO)) %>% 
  mutate(Type = purrr::map(str_split(sampling_resolution, "_"), `[[`, 1) %>% unlist(),
         Resolution = purrr::map(str_split(sampling_resolution, "_"), `[[`,2) %>% unlist())

smry %>% group_by(Type, Resolution) %>% summarize(mean = mean(count/count_effort),
                                                                   min = min(count/count_effort),
                                                                   max = max(count/count_effort))
View(smry %>% group_by(Species, region, Type, Resolution) %>% summarize(mean = mean(count/count_effort),
                                                  min = min(count/count_effort),
                                                  max = max(count/count_effort)))



# format
smry <- smry %>% mutate(
  Resolution = case_when(
    Resolution == "0083" ~ "0.083",
    Resolution == "017" ~ "0.17",
    Resolution == "025" ~ "0.25",
    Resolution == "050" ~ "0.50",
    Resolution == "100" ~ "1.00"
  )
)
smry$Resolution <- factor(smry$Resolution, levels = c("0.083","0.17","0.25","0.50","1.00"))

ggplot(smry) + geom_boxplot(aes(x = Resolution, y = (count/count_effort)*100, color = Type), 
                                  outlier.size = 0.9, outlier.alpha = 0.8, outlier.stroke = 0.1,
                                  size = 0.4, width = 0.7) + 
  theme_bw() +  coord_cartesian(ylim = c(0,70)) +
  theme(axis.title = element_text(size = 9),
        strip.text = element_text(face = "bold", hjust = 0, size = 10),
        legend.position = "bottom") +
  facet_grid(Species~region) + 
  scale_color_colorblind(labels = c("Raster", "Segment")) +
  ylab("Percentage of sampling unit wih sightings")




