##--------------------------------------------------------------------------------------------------------
## PROJECT : The effect of spatial resolution on SDM efficiency
## SCRIPT : Model single variables
##
## Author : Charlotte Lambert
## Last update : Jan 2022
## R version 4.0.4 (2021-02-15) -- "Lost Library Book"
##--------------------------------------------------------------------------------------------------------


library(dplyr)
library(tidyverse)
library(raster)
library(terra)
library(maps)
library(maptools)
library(ggplot2)
library(ggthemes)
library(ggspatial)
library(ggpubr)
library(RColorBrewer)
library(reshape2)
library(lubridate)
library(viridis)
library(ncdf4)
library(sf)
library(spatstat)
library(spatstat.utils)
library(multidplyr)
library(mgcv)

#### ----------------------- ####
#### ---- SOURCES FILES ---- ####
#### ----------------------- ####
datadir <- "~/4_Formatting/"
outdir <- "~/5_Modelling/01_Single_variable_models/"

seg0083_WIO <- read.table(paste(datadir, "WIO_seg_cov_0083.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Seg_0083", region = "WE") %>% rename(effort = SegLengthKm, simulation=)
seg017_WIO <- read.table(paste(datadir, "WIO_seg_cov_017.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Seg_017", region = "WE") %>% rename(effort = SegLengthKm)
seg025_WIO <- read.table(paste(datadir, "WIO_seg_cov_025.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Seg_025", region = "WE") %>% rename(effort = SegLengthKm)
seg050_WIO <- read.table(paste(datadir, "WIO_seg_cov_050.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Seg_050", region = "WE") %>% rename(effort = SegLengthKm)
seg100_WIO <- read.table(paste(datadir, "WIO_seg_cov_100.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Seg_100", region = "WE") %>% rename(effort = SegLengthKm)

rast017_WIO <- read.table(paste(datadir, "WIO_rast_cov_017.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Rast_017", region = "WE")
rast025_WIO <- read.table(paste(datadir, "WIO_rast_cov_025.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Rast_025", region = "WE")
rast050_WIO <- read.table(paste(datadir, "WIO_rast_cov_050.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Rast_050", region = "WE")
rast100_WIO <- read.table(paste(datadir, "WIO_rast_cov_100.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Rast_100", region = "WE")

vars <- c("Bathy", "Slope", "Temp", "SSH", "NPP", "MLD", "ZEu", "CurrentSpeed", "EKE")

data_WIO <- rbind(seg0083_WIO[,c("Longitude", "Latitude", "effort", "n_sp025", "n_sp0083",
                               "Simulation", "sampling_resolution", "region", vars)],
                 seg017_WIO[,c("Longitude", "Latitude", "effort", "n_sp025", "n_sp0083",
                              "Simulation", "sampling_resolution", "region", vars)],
                 seg025_WIO[,c("Longitude", "Latitude", "effort", "n_sp025", "n_sp0083",
                              "Simulation", "sampling_resolution", "region", vars)],
                 seg050_WIO[,c("Longitude", "Latitude", "effort", "n_sp025", "n_sp0083",
                              "Simulation", "sampling_resolution", "region", vars)],
                 seg100_WIO[,c("Longitude", "Latitude", "effort", "n_sp025", "n_sp0083",
                              "Simulation", "sampling_resolution", "region", vars)],
                 rast017_WIO[,c("Longitude", "Latitude", "effort", "n_sp025", "n_sp0083",
                               "Simulation", "sampling_resolution", "region", vars)],
                 rast025_WIO[,c("Longitude", "Latitude", "effort", "n_sp025", "n_sp0083",
                               "Simulation", "sampling_resolution", "region", vars)],
                 rast050_WIO[,c("Longitude", "Latitude", "effort", "n_sp025", "n_sp0083",
                               "Simulation", "sampling_resolution", "region", vars)],
                 rast100_WIO[,c("Longitude", "Latitude", "effort", "n_sp025", "n_sp0083",
                               "Simulation", "sampling_resolution", "region", vars)])

rm(seg0083_WIO, seg017_WIO, seg025_WIO, seg050_WIO, seg100_WIO, rast017_WIO, rast025_WIO, rast050_WIO, rast100_WIO)



#### ---------------------- ####
#### ------ MODELLING ----- ####
#### ---------------------- ####

### 1 - Computation ------------------------------------------------------------
simu <- unique(data_WIO$Simulation)
reso <- unique(data_WIO$sampling_resolution)

fit_all_single <- function(data, variables, simu, reso){
  data <- data %>% pivot_longer(cols = all_of(variables), names_to = "Variable", values_to = "value") %>%
    drop_na(value) %>% dplyr::select(all_of(c("Longitude", "Latitude", "effort",
                                              "n_sp025", "n_sp0083", "Simulation",
                                              "sampling_resolution", "region", 
                                              "Variable", "value")))
  result_table <- NULL
  pred_table <- NULL
  
  for(i in seq_along(variables)){
    for(j in seq_along(simu)){
      for(k in seq_along(reso)){
        subdata <- data %>% subset(Variable == variables[i] 
                                   & Simulation == simu[j] 
                                   & sampling_resolution == reso[k])
        mod_sp025 <- gam(n_sp025 ~ s(value, k = 4), family = tw(), data = subdata, offset = log(2*effort*0.2))
        mod_sp0083 <- gam(n_sp0083 ~ s(value, k = 4), family = tw(), data = subdata, offset = log(2*effort*0.2))
                
        edf_sp025 <- sum( mod_sp025$edf[ mod_sp025$smooth[[i]]$first.para:mod_sp025$smooth[[i]]$last.para ] )
        edf_sp0083 <- sum( mod_sp0083$edf[ mod_sp0083$smooth[[i]]$first.para:mod_sp0083$smooth[[i]]$last.para ] )
        
        dev_table <- data.frame(Variable = variables[i],
                                Simulation = simu[j],
                                sampling_resolution = reso[k],
                                Deviance = c(summary(mod_sp025)$dev.expl,
                                             summary(mod_sp0083)$dev.expl),
                                Rsquare = c(summary(mod_sp025)$r.sq,
                                            summary(mod_sp0083)$r.sq),
                                AIC = c(mod_sp025$aic,
                                        mod_sp0083$aic),
                                Sample_size = c(summary(mod_sp025)$n,
                                                summary(mod_sp0083)$n),
                                # p_value = c(summary(mod_sp025)$dev.expl,
                                #             summary(mod_sp0083)$dev.expl),
                                RMSE = c(sqrt(mean(residuals.gam(mod_sp025, type = "response")^2)),
                                         sqrt(mean(residuals.gam(mod_sp0083, type = "response")^2))  ),
                                edf = c(round(edf_sp025, digits = 2),
                                        round(edf_sp0083, digits = 2)),
                                Species = c("sp025", "sp0083"))
        result_table <- rbind(result_table, dev_table)
        
        pred <- cbind(subdata, 
                      predict(mod_sp025, newdata = subdata, type = "terms", se.fit = T),
                      predict(mod_sp0083, newdata = subdata, type = "terms", se.fit = T))
        names(pred)[11:14] <- c("n_sp025_fit" , "n_sp025_sefit", "n_sp0083_fit", "n_sp0083_sefit")
        # plot(pred$n_sp025_fit ~ pred$value)
        # points(pred$n_sp025_fit + pred$n_sp025_sefit ~ pred$value, col = "green")
        # points(pred$n_sp025_fit - pred$n_sp025_sefit ~ pred$value, col = "green")
        pred_table <- rbind(pred_table, pred)
        
      }    
    }
  }
  return(list(Data = pred_table, Deviance = result_table))
}


toto <- pbapply::pblapply(seq_along(vars), function(i){
  writeLines(paste("Processing", vars[i], sep = " "))
  WIO_seg0083 <- fit_all_single(data = data_WIO, variables = vars[i], reso = reso[1], simu = simu)
  WIO_seg017 <- fit_all_single(data = data_WIO, variables = vars[i], reso = reso[2], simu = simu)
  WIO_seg025 <- fit_all_single(data = data_WIO, variables = vars[i], reso = reso[3], simu = simu)
  WIO_seg050 <- fit_all_single(data = data_WIO, variables = vars[i], reso = reso[4], simu = simu)
  WIO_seg100 <- fit_all_single(data = data_WIO, variables = vars[i], reso = reso[5], simu = simu)
  WIO_rast017 <- fit_all_single(data = data_WIO, variables = vars[i], reso = reso[6], simu = simu)
  WIO_rast025 <- fit_all_single(data = data_WIO, variables = vars[i], reso = reso[7], simu = simu)
  WIO_rast050 <- fit_all_single(data = data_WIO, variables = vars[i], reso = reso[8], simu = simu)
  WIO_rast100 <- fit_all_single(data = data_WIO, variables = vars[i], reso = reso[9], simu = simu)
  
  single_var <- rbind(WIO_seg0083$Data,
                      WIO_seg017$Data,
                      WIO_seg025$Data,
                      WIO_seg050$Data,
                      WIO_seg100$Data,
                      WIO_rast017$Data,
                      WIO_rast025$Data,
                      WIO_rast050$Data,
                      WIO_rast100$Data     )
  deviances <- rbind(WIO_seg0083$Deviance,
                     WIO_seg017$Deviance,
                     WIO_seg025$Deviance,
                     WIO_seg050$Deviance,
                     WIO_seg100$Deviance,
                     WIO_rast017$Deviance,
                     WIO_rast025$Deviance,
                     WIO_rast050$Deviance,
                     WIO_rast100$Deviance     )
  rm(WIO_seg0083,
     WIO_seg017,
     WIO_seg025,
     WIO_seg050,
     WIO_seg100,
     WIO_rast017,
     WIO_rast025,
     WIO_rast050,
     WIO_rast100)
  return(list(Predictions = single_var, Model_parameters = deviances))
})


save.image(paste(outdir, "WIO_Single_vars.RData", sep =""))



### 2 - Process files ----------------------------------------------------------
single_var_pred <- toto %>% 
  purrr::map(`[[`, 1) %>% 
  data.table::rbindlist()

deviances <- toto %>% 
  purrr::map(`[[`, 2) %>% 
  data.table::rbindlist() %>%
  mutate(Region = "IO",
         Species = case_when(
           Species == "sp0083" ~ "Sp_0083", #sp1
           Species == "sp025" ~ "Sp_025", #sp2
         ))

save.image(paste(outdir, "WIO_Single_vars.RData", sep =""))

write.table(deviances, "~/Project_Spatial_Resolution/5_Modelling/01_Single_variable_models/Deviances_WIO.csv",
            sep = ";", dec = ".", row.names = F)



### 3 - Fit overall gam to simulation-specific curves --------------------------
smry_curve <- NULL
for(i in seq_along(vars)){
  for(k in seq_along(reso)){
    subdata <- single_var_pred %>% subset(Variable == vars[i] 
                                          & sampling_resolution == reso[k])
    mod_sp025 <- gam(n_sp025_fit ~ s(value, k = 4), data = subdata)
    mod_sp0083 <- gam(n_sp0083_fit ~ s(value, k = 4), data = subdata)
    
    
    result_table <- data.frame(Variable = vars[i],
                               sampling_resolution = reso[k],
                               Deviance = c(summary(mod_sp025)$dev.expl,
                                            summary(mod_sp0083)$dev.expl),
                               Rsquare = c(summary(mod_sp025)$r.sq,
                                           summary(mod_sp0083)$r.sq),
                               RMSE = c(sqrt(mean(residuals.gam(mod_sp025, type = "response")^2)),
                                        sqrt(mean(residuals.gam(mod_sp0083, type = "response")^2))  ),
                               Species = c("sp025", "sp0083"),
                               Region = rep("WIO", 2))
    smry_curve <- rbind(smry_curve, result_table)
  }
}

save.image(paste(outdir, "WIO_Single_vars.RData", sep =""))

write.table(smry_curve, "~/Project_Spatial_Resolution/5_Modelling/01_Single_variable_models/Summary_curves_WIO.csv",
            sep = ";", dec = ".", row.names = F)


### 4 - Plots curves -----------------------------------------------------------
# alleviate memory
rm(data_WIO, toto)

### sp 1
ggplot(single_var_pred %>% subset(Variable == "Bathy")) + 
  geom_line(aes(x = value, y = n_sp025_fit, group = Simulation), colour = "#ebc546", size = 0.1) +
  facet_grid(sampling_resolution~Variable, scales = "free") + theme_bw() + 
  geom_rug(data = single_var_pred %>% subset(Variable == "Bathy" & Simulation == "Simulation 1"), 
           aes(x = value, y = n_sp025_fit), sides = "b", alpha = 1/2, position = "jitter")
ggsave(paste(outdir, "WIO_sp025_Bathy_curves.png", sep = ""), height = 10, width = 4, dpi= 350)

ggplot(single_var_pred %>% subset(Variable == "CurrentSpeed")) + 
  geom_line(aes(x = value, y = n_sp025_fit, group = Simulation), color = "#ebc546", size = 0.1) +
  facet_grid(sampling_resolution~Variable, scales = "free") + theme_bw() + 
  geom_rug(data = single_var_pred %>% subset(Variable == "CurrentSpeed" & Simulation == "Simulation 1"), 
           aes(x = value, y = n_sp025_fit), sides = "b", alpha = 1/2, position = "jitter")
ggsave(paste(outdir, "WIO_sp025_CurrentSpeed_curves.png", sep = ""), height = 10, width = 4, dpi= 350)

ggplot(single_var_pred %>% subset(Variable == "EKE")) + 
  geom_line(aes(x = value, y = n_sp025_fit, group = Simulation), color = "#ebc546", size = 0.1) +
  facet_grid(sampling_resolution~Variable, scales = "free") + theme_bw() + 
  geom_rug(data = single_var_pred %>% subset(Variable == "EKE" & Simulation == "Simulation 1"), 
           aes(x = value, y = n_sp025_fit), sides = "b", alpha = 1/2, position = "jitter")
ggsave(paste(outdir, "WIO_sp025_EKE_curves.png", sep = ""), height = 10, width = 4, dpi= 350)

ggplot(single_var_pred %>% subset(Variable == "MLD")) + 
  geom_line(aes(x = value, y = n_sp025_fit, group = Simulation), color = "#ebc546", size = 0.1) +
  facet_grid(sampling_resolution~Variable, scales = "free") + theme_bw() + 
  geom_rug(data = single_var_pred %>% subset(Variable == "MLD" & Simulation == "Simulation 1"), 
           aes(x = value, y = n_sp025_fit), sides = "b", alpha = 1/2, position = "jitter")
ggsave(paste(outdir, "WIO_sp025_MLD_curves.png", sep = ""), height = 10, width = 4, dpi= 350)

ggplot(single_var_pred %>% subset(Variable == "NPP")) + 
  geom_line(aes(x = value, y = n_sp025_fit, group = Simulation), color = "#ebc546", size = 0.1) +
  facet_grid(sampling_resolution~Variable, scales = "free") + theme_bw() + 
  geom_rug(data = single_var_pred %>% subset(Variable == "NPP" & Simulation == "Simulation 1"), 
           aes(x = value, y = n_sp025_fit), sides = "b", alpha = 1/2, position = "jitter")
ggsave(paste(outdir, "WIO_sp025_NPP_curves.png", sep = ""), height = 10, width = 4, dpi= 350)

ggplot(single_var_pred %>% subset(Variable == "Slope")) + 
  geom_line(aes(x = value, y = n_sp025_fit, group = Simulation), color = "#ebc546", size = 0.1) +
  facet_grid(sampling_resolution~Variable, scales = "free") + theme_bw() + 
  geom_rug(data = single_var_pred %>% subset(Variable == "Slope" & Simulation == "Simulation 1"), 
           aes(x = value, y = n_sp025_fit), sides = "b", alpha = 1/2, position = "jitter")
ggsave(paste(outdir, "WIO_sp025_Slope_curves.png", sep = ""), height = 10, width = 4, dpi= 350)

ggplot(single_var_pred %>% subset(Variable == "SSH")) + 
  geom_line(aes(x = value, y = n_sp025_fit, group = Simulation), color = "#ebc546", size = 0.1) +
  facet_grid(sampling_resolution~Variable, scales = "free") + theme_bw() + 
  geom_rug(data = single_var_pred %>% subset(Variable == "SSH" & Simulation == "Simulation 1"), 
           aes(x = value, y = n_sp025_fit), sides = "b", alpha = 1/2, position = "jitter")
ggsave(paste(outdir, "WIO_sp025_SSH_curves.png", sep = ""), height = 10, width = 4, dpi= 350)

ggplot(single_var_pred %>% subset(Variable == "Temp")) + 
  geom_line(aes(x = value, y = n_sp025_fit, group = Simulation), color = "#ebc546", size = 0.1) +
  facet_grid(sampling_resolution~Variable, scales = "free") + theme_bw() + 
  geom_rug(data = single_var_pred %>% subset(Variable == "Temp" & Simulation == "Simulation 1"), 
           aes(x = value, y = n_sp025_fit), sides = "b", alpha = 1/2, position = "jitter")
ggsave(paste(outdir, "WIO_sp025_Temp_curves.png", sep = ""), height = 10, width = 4, dpi= 350)

ggplot(single_var_pred %>% subset(Variable == "ZEu")) + 
  geom_line(aes(x = value, y = n_sp025_fit, group = Simulation), color = "#ebc546", size = 0.1) +
  facet_grid(sampling_resolution~Variable, scales = "free") + theme_bw() + 
  geom_rug(data = single_var_pred %>% subset(Variable == "ZEu" & Simulation == "Simulation 1"), 
           aes(x = value, y = n_sp025_fit), sides = "b", alpha = 1/2, position = "jitter")
ggsave(paste(outdir, "WIO_sp025_ZEu_curves.png", sep = ""), height = 10, width = 4, dpi= 350)


### Sp 2
ggplot(single_var_pred %>% subset(Variable == "Bathy")) + 
  geom_line(aes(x = value, y = n_sp0083_fit, group = Simulation), color = "#9d7eb0", size = 0.1) +
  facet_grid(sampling_resolution~Variable, scales = "free") + theme_bw() + 
  geom_rug(data = single_var_pred %>% subset(Variable == "Bathy" & Simulation == "Simulation 1"), 
           aes(x = value, y = n_sp0083_fit), sides = "b", alpha = 1/2, position = "jitter")
ggsave(paste(outdir, "WIO_sp0083_Bathy_curves.png", sep = ""), height = 10, width = 4, dpi= 350)

ggplot(single_var_pred %>% subset(Variable == "CurrentSpeed")) + 
  geom_line(aes(x = value, y = n_sp0083_fit, group = Simulation), color = "#9d7eb0", size = 0.1) +
  facet_grid(sampling_resolution~Variable, scales = "free") + theme_bw() + 
  geom_rug(data = single_var_pred %>% subset(Variable == "CurrentSpeed" & Simulation == "Simulation 1"), 
           aes(x = value, y = n_sp0083_fit), sides = "b", alpha = 1/2, position = "jitter")
ggsave(paste(outdir, "WIO_sp0083_CurrentSpeed_curves.png", sep = ""), height = 10, width = 4, dpi= 350)

ggplot(single_var_pred %>% subset(Variable == "EKE")) + 
  geom_line(aes(x = value, y = n_sp0083_fit, group = Simulation), color = "#9d7eb0", size = 0.1) +
  facet_grid(sampling_resolution~Variable, scales = "free") + theme_bw() + 
  geom_rug(data = single_var_pred %>% subset(Variable == "EKE" & Simulation == "Simulation 1"), 
           aes(x = value, y = n_sp0083_fit), sides = "b", alpha = 1/2, position = "jitter")
ggsave(paste(outdir, "WIO_sp0083_EKE_curves.png", sep = ""), height = 10, width = 4, dpi= 350)

ggplot(single_var_pred %>% subset(Variable == "MLD")) + 
  geom_line(aes(x = value, y = n_sp0083_fit, group = Simulation), color = "#9d7eb0", size = 0.1) +
  facet_grid(sampling_resolution~Variable, scales = "free") + theme_bw() + 
  geom_rug(data = single_var_pred %>% subset(Variable == "MLD" & Simulation == "Simulation 1"), 
           aes(x = value, y = n_sp0083_fit), sides = "b", alpha = 1/2, position = "jitter")
ggsave(paste(outdir, "WIO_sp0083_MLD_curves.png", sep = ""), height = 10, width = 4, dpi= 350)

ggplot(single_var_pred %>% subset(Variable == "NPP")) + 
  geom_line(aes(x = value, y = n_sp0083_fit, group = Simulation), color = "#9d7eb0", size = 0.1) +
  facet_grid(sampling_resolution~Variable, scales = "free") + theme_bw() + 
  geom_rug(data = single_var_pred %>% subset(Variable == "NPP" & Simulation == "Simulation 1"), 
           aes(x = value, y = n_sp0083_fit), sides = "b", alpha = 1/2, position = "jitter")
ggsave(paste(outdir, "WIO_sp0083_NPP_curves.png", sep = ""), height = 10, width = 4, dpi= 350)

ggplot(single_var_pred %>% subset(Variable == "Slope")) + 
  geom_line(aes(x = value, y = n_sp0083_fit, group = Simulation), color = "#9d7eb0", size = 0.1) +
  facet_grid(sampling_resolution~Variable, scales = "free") + theme_bw() + 
  geom_rug(data = single_var_pred %>% subset(Variable == "Slope" & Simulation == "Simulation 1"), 
           aes(x = value, y = n_sp0083_fit), sides = "b", alpha = 1/2, position = "jitter")
ggsave(paste(outdir, "WIO_sp0083_Slope_curves.png", sep = ""), height = 10, width = 4, dpi= 350)

ggplot(single_var_pred %>% subset(Variable == "SSH")) + 
  geom_line(aes(x = value, y = n_sp0083_fit, group = Simulation), color = "#9d7eb0", size = 0.1) +
  facet_grid(sampling_resolution~Variable, scales = "free") + theme_bw() + 
  geom_rug(data = single_var_pred %>% subset(Variable == "SSH" & Simulation == "Simulation 1"), 
           aes(x = value, y = n_sp0083_fit), sides = "b", alpha = 1/2, position = "jitter")
ggsave(paste(outdir, "WIO_sp0083_SSH_curves.png", sep = ""), height = 10, width = 4, dpi= 350)

ggplot(single_var_pred %>% subset(Variable == "Temp")) + 
  geom_line(aes(x = value, y = n_sp0083_fit, group = Simulation), color = "#9d7eb0", size = 0.1) +
  facet_grid(sampling_resolution~Variable, scales = "free") + theme_bw() + 
  geom_rug(data = single_var_pred %>% subset(Variable == "Temp" & Simulation == "Simulation 1"), 
           aes(x = value, y = n_sp0083_fit), sides = "b", alpha = 1/2, position = "jitter")
ggsave(paste(outdir, "WIO_sp0083_Temp_curves.png", sep = ""), height = 10, width = 4, dpi= 350)

ggplot(single_var_pred %>% subset(Variable == "ZEu")) + 
  geom_line(aes(x = value, y = n_sp0083_fit, group = Simulation), color = "#9d7eb0", size = 0.1) +
  facet_grid(sampling_resolution~Variable, scales = "free") + theme_bw() + 
  geom_rug(data = single_var_pred %>% subset(Variable == "ZEu" & Simulation == "Simulation 1"), 
           aes(x = value, y = n_sp0083_fit), sides = "b", alpha = 1/2, position = "jitter")
ggsave(paste(outdir, "WIO_sp0083_ZEu_curves.png", sep = ""), height = 10, width = 4, dpi= 350)


