##--------------------------------------------------------------------------------------------------------
## PROJECT : The effect of spatial resolution on SDM efficiency
## SCRIPT : Model single variables
##
## Author : Charlotte Lambert
## Last update : Jan 2022
## R version 4.0.4 (2021-02-15) -- "Lost Library Book"
##--------------------------------------------------------------------------------------------------------


library(dplyr)
library(tidyverse)
library(ggplot2)
library(ggthemes)
library(ggpubr)
library(RColorBrewer)
library(viridis)
library(sf)
library(mgcv)
library(ggalt)
library(grid)
library(gridExtra)

#### ----------------------- ####
#### ---- SOURCES FILES ---- ####
#### ----------------------- ####
outdir <- "~/5_Modelling/01_Single_variable_models/"

### 1 - Open files -------------------------------------------------------------
vars <- c("Bathy", "Slope", "Temp", "SSH", "NPP", "MLD", "ZEu", "CurrentSpeed", "EKE")

dev_ENA <- read.table(paste(outdir, "Deviances_ENA.csv", sep = ""), h = T, dec = ".", sep = ";")  %>% 
  mutate(Type = purrr::map(str_split(sampling_resolution, "_"), `[[`, 1) %>% unlist(),
         Resolution = purrr::map(str_split(sampling_resolution, "_"), `[[`,2) %>% unlist())
dev_WIO <- read.table(paste(outdir, "Deviances_WIO.csv", sep = ""), h = T, dec = ".", sep = ";") %>% 
  mutate(Region = "WIO") %>% 
  mutate(Type = purrr::map(str_split(sampling_resolution, "_"), `[[`, 1) %>% unlist(),
         Resolution = purrr::map(str_split(sampling_resolution, "_"), `[[`,2) %>% unlist())

smry_curves_ENA <- read.table(paste(outdir, "Summary_curves_ENA.csv", sep = ""), h = T, dec = ".", sep = ";")
smry_curves_WIO <- read.table(paste(outdir, "Summary_curves_WIO.csv", sep = ""), h = T, dec = ".", sep = ";")


### 2 - Format files -----------------------------------------------------------
deviances <- full_join(dev_ENA, dev_WIO) %>%
  mutate(Species = case_when(
    Species == "Sp_0083" ~ "Species 0.083",
    Species == "Sp_025" ~ "Species 0.25"
  ))

smry_curves <- full_join(smry_curves_ENA, smry_curves_WIO) %>% 
  mutate(Type = purrr::map(str_split(sampling_resolution, "_"), `[[`, 1) %>% unlist(),
         Resolution = purrr::map(str_split(sampling_resolution, "_"), `[[`,2) %>% unlist()) %>%
  mutate(Species = case_when(
    Species == "Sp_0083" ~ "Species 0.083",
    Species == "Sp_025" ~ "Species 0.25"
  ))



dev_rast <- deviances %>% subset(Type == "Rast") %>% 
  rename(Dev_rast = Deviance,
         edf_rast = edf,
         RMSE_rast = RMSE,
         Rsqu_rast = Rsquare) %>%
  select(! c("Type", "sampling_resolution", "AIC", "Sample_size")) 
dev_seg <- deviances %>% subset(Type == "Seg" & Resolution != "0083") %>% 
  rename(Dev_seg = Deviance,
         edf_seg = edf,
         RMSE_seg = RMSE,
         Rsqu_seg = Rsquare) %>%
  select(! c("Type", "sampling_resolution", "AIC", "Sample_size")) 
dd <- left_join(dev_seg,
                dev_rast) %>%
  mutate(Diff_dev = Dev_seg - Dev_rast,
         Diff_edf = edf_seg - edf_rast,
         Diff_RMSE = RMSE_seg - RMSE_rast,
         Diff_Rsquare = Rsqu_seg - Rsqu_rast)


smry_param <- deviances %>% 
  group_by(Region, Species, Variable, Type, Resolution) %>%
  summarize(sd_dev = sd(Deviance, na.rm = T),
            sd_Rsquare = sd(Rsquare, na.rm = T),
            sd_AIC = sd(AIC, na.rm = T),
            sd_RMSE = sd(RMSE, na.rm = T),
            mean_dev = mean(Deviance, na.rm = T),
            mean_Rsquare = mean(Rsquare, na.rm = T),
            mean_AIC = mean(AIC, na.rm = T),
            mean_RMSE = mean(RMSE, na.rm = T))




#### ------------------------------------ ####
#### ----- Final summary of results ----- ####
#### ------------------------------------ ####

# prepare g1
p1 <-   ggplot(deviances) + 
  geom_boxplot(aes(y = Variable, x = Deviance, color = Type), 
               outlier.size = 0.9, outlier.alpha = 0.8, outlier.stroke = 0.1,
               size = 0.4, width = 0.7) + 
  theme_bw() +  
  theme(axis.title.x = element_text(size = 9),
        axis.title.y = element_blank(),
        strip.text = element_text(face = "bold", hjust = 0, size = 10.5),
        strip.background = element_rect(fill = NA, color = NA)) +
  facet_grid(Species~Region) + scale_color_colorblind(labels = c("Raster", "Segment")) +
  labs(title = "A - Mean explained deviances", x = "Deviance")

# prepare g2
p2 <- ggplot(dd) + 
      geom_boxplot(aes(x = Species, y = Diff_dev, color = Resolution), 
                   outlier.size = 0.9, outlier.alpha = 0.8, outlier.stroke = 0.1,
                   size = 0.4, width = 0.5) + 
      scale_color_colorblind(labels = c("0.17", "0.25", "0.50", "1.00")) + 
      ylab("Segment - Raster") +
      facet_wrap("Region") + theme_bw() + ggtitle("B - Difference in model deviances") +
      theme(axis.title.y = element_text(size = 9),
            axis.title.x = element_blank(),
            strip.text = element_text(face = "bold", hjust = 0, size = 10.5),
            strip.background = element_rect(fill = NA, color = NA)) 

# prepare g3
p3 <- ggplot(dd) + 
      geom_boxplot(aes(x = Species, y = Diff_edf, color = Resolution), 
                   outlier.size = 0.9, outlier.alpha = 0.8, outlier.stroke = 0.1,
                   size = 0.4, width = 0.5)  + 
      scale_color_colorblind(labels = c("0.17", "0.25", "0.50", "1.00")) + 
      ylab("Segment - Raster") +
      facet_wrap("Region") + theme_bw() + ggtitle("C - Difference in curves complexity") +
      theme(axis.title.y = element_text(size = 9),
            axis.title.x = element_blank(),
            strip.text = element_text(face = "bold", hjust = 0, size = 10.5),
            strip.background = element_rect(fill = NA, color = NA)) 

# prepare g4
p4 <- ggplot(dd) + 
      geom_boxplot(aes(x = Species, y = Diff_RMSE, color = Resolution), 
                   outlier.size = 0.9, outlier.alpha = 0.8, outlier.stroke = 0.1, 
                   size = 0.4, width = 0.6)  + 
      scale_color_colorblind(labels = c("0.17", "0.25", "0.50", "1.00")) + 
      ylab("Segment - Raster") +
      facet_wrap("Region") + theme_bw() + ggtitle("D - Difference in model RMSE") +
      theme(axis.title.y = element_text(size = 9),
            axis.title.x = element_blank(),
            strip.text = element_text(face = "bold", hjust = 0, size = 10.5),
            strip.background = element_rect(fill = NA, color = NA) )


## arrange the final plot
ggarrange(
  p1,
  ggarrange(
    p2, p3, p4, ncol = 1, nrow = 3, common.legend = T, align = "hv", legend = "bottom")
  , nrow = 1, ncol = 2, legend = "bottom", widths = c(1,0.9)) + 
  bgcolor("white") + border("white")
ggsave(paste(outdir, "0_Single_var_results.png", sep = ""), height = 9, width = 9, dpi= 350)

