##--------------------------------------------------------------------------------------------------------
## PROJECT : The effect of spatial resolution on SDM efficiency
## SCRIPT : Model selection procedure
##
## Author : Charlotte Lambert
## Last update : March 2022
## R version 4.0.4 (2021-02-15) -- "Lost Library Book"
##--------------------------------------------------------------------------------------------------------


library(dplyr)
library(tidyverse)
library(raster)
library(terra)
library(maps)
library(maptools)
library(ggplot2)
library(ggthemes)
library(ggspatial)
library(ggpubr)
library(RColorBrewer)
library(reshape2)
library(lubridate)
library(viridis)
library(ncdf4)
library(sf)
library(spatstat)
library(spatstat.utils)
library(multidplyr)
library(mgcv)
library(data.table)
library(qpcR)
library(itsadug)

#### ----------------------- ####
#### ---- SOURCES FILES ---- ####
#### ----------------------- ####
datadir <- "~/4_Formatting/"
outdir <- "~/5_Modelling/02_Model_selection/"

seg0083_ENA <- read.table(paste(datadir, "ENA_seg_cov_0083.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Seg_0083", region = "ENA") %>% rename(effort = SegLengthKm, simulation=)
seg017_ENA <- read.table(paste(datadir, "ENA_seg_cov_017.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Seg_017", region = "ENA") %>% rename(effort = SegLengthKm)
seg025_ENA <- read.table(paste(datadir, "ENA_seg_cov_025.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Seg_025", region = "ENA") %>% rename(effort = SegLengthKm)
seg050_ENA <- read.table(paste(datadir, "ENA_seg_cov_050.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Seg_050", region = "ENA") %>% rename(effort = SegLengthKm)
seg100_ENA <- read.table(paste(datadir, "ENA_seg_cov_100.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Seg_100", region = "ENA") %>% rename(effort = SegLengthKm)

rast017_ENA <- read.table(paste(datadir, "ENA_rast_cov_017.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Rast_017", region = "ENA")
rast025_ENA <- read.table(paste(datadir, "ENA_rast_cov_025.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Rast_025", region = "ENA")
rast050_ENA <- read.table(paste(datadir, "ENA_rast_cov_050.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Rast_050", region = "ENA")
rast100_ENA <- read.table(paste(datadir, "ENA_rast_cov_100.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Rast_100", region = "ENA")

vars <- c("Bathy", "Slope", "Temp", "SSH", "NPP", "MLD", "ZEu", "CurrentSpeed", "EKE")

data_ENA <- rbind(seg0083_ENA[,c("Longitude", "Latitude", "effort", "n_sp0083", "n_sp025",
                               "Simulation", "sampling_resolution", "region", vars)],
                 seg017_ENA[,c("Longitude", "Latitude", "effort", "n_sp0083", "n_sp025",
                              "Simulation", "sampling_resolution", "region", vars)],
                 seg025_ENA[,c("Longitude", "Latitude", "effort", "n_sp0083", "n_sp025",
                              "Simulation", "sampling_resolution", "region", vars)],
                 seg050_ENA[,c("Longitude", "Latitude", "effort", "n_sp0083", "n_sp025",
                              "Simulation", "sampling_resolution", "region", vars)],
                 seg100_ENA[,c("Longitude", "Latitude", "effort", "n_sp0083", "n_sp025",
                              "Simulation", "sampling_resolution", "region", vars)],
                 rast017_ENA[,c("Longitude", "Latitude", "effort", "n_sp0083", "n_sp025",
                               "Simulation", "sampling_resolution", "region", vars)],
                 rast025_ENA[,c("Longitude", "Latitude", "effort", "n_sp0083", "n_sp025",
                               "Simulation", "sampling_resolution", "region", vars)],
                 rast050_ENA[,c("Longitude", "Latitude", "effort", "n_sp0083", "n_sp025",
                               "Simulation", "sampling_resolution", "region", vars)],
                 rast100_ENA[,c("Longitude", "Latitude", "effort", "n_sp0083", "n_sp025",
                               "Simulation", "sampling_resolution", "region", vars)])

rm(seg0083_ENA, seg017_ENA, seg025_ENA, seg050_ENA, seg100_ENA, rast017_ENA, rast025_ENA, rast050_ENA, rast100_ENA)

source(paste0(outdir, "fctGAM_202202.r"))
source(paste0(outdir, "contrib_function_202202.r"))


#### ---------------------- ####
#### ------ MODELLING ----- ####
#### ---------------------- ####

### 1 - Function ---------------------------------------------------------------
simu <- unique(data_ENA$Simulation)
reso <- unique(data_ENA$sampling_resolution)

fit_all_single <- function(data, simu, resolution, variables){
  data <- data %>% drop_na(all_of(variables)) %>% subset(sampling_resolution == resolution) %>% rename(Effort = effort)
  
  res <- pbapply::pblapply(seq_along(simu), function(j){
      subdata <- data %>% subset(Simulation == simu[j])
      #### SP 0083
      # model selection
      fit_best_sp0083 <- fit_all_gam(envdata = subdata, esw = 0.2, outcome = "n_sp0083",
                              predictors = variables, 
                              rescale = F, weight_by_g_naught = 1, family = "tweedie", 
                              max_cor = 0.5, nb_min_pred = 1, nb_max_pred = 4, complexity = 4)
      
      # compute AIC weights and likelihoods of all tested models
      fit_best_sp0083_ord <- fit_best_sp0083[order(fit_best_sp0083$AIC),]
      aics_sp0083 <- akaike.weights(fit_best_sp0083_ord[,3])
      fit_best_sp0083_ord$Delta.AIC <- aics_sp0083$deltaAIC
      fit_best_sp0083_ord$rel.likelihood <- aics_sp0083$rel.LL
      fit_best_sp0083_ord$Akaike.weight <- aics_sp0083$weights
      fit_best_sp0083_ord <- fit_best_sp0083_ord %>%
        mutate(Region = "ENA", 
               Species = "Sp_0083", 
               Simulation = simu[j], 
               sampling_resolution = resolution)
      
      # compute AICw of variables
      table_sp0083 <- data.frame(var = NA, count = NA, percent = NA, akaike.weight = NA)
      for (i in 1:length(variables)) {
        select <- fit_best_sp0083_ord[fit_best_sp0083_ord$model %like% variables[i], ]
        table_sp0083[i,1] <- variables[i]
        table_sp0083[i,2] <- nrow(select) 
        table_sp0083[i,3] <- nrow(select)/nrow(fit_best_sp0083_ord)*100
        table_sp0083[i,4] <- sum(select$Akaike.weight)*100
      }
      table_sp0083 <- table_sp0083[order(-table_sp0083$akaike.weight),] %>% 
        mutate(Region = "ENA", 
               Species = "Sp_0083", 
               Simulation = simu[j], 
               sampling_resolution = resolution)
      
      # fit best model
      mod_sp0083 <- mgcv::gam(as.formula(fit_best_sp0083_ord[1,1]), 
                              data = subdata, offset = log(2*0.2*Effort), 
                              family = tw(), method = "REML")
      
      # compute contribution of selected variables
      if( stringr::str_detect(fit_best_sp0083_ord[1,1], "[+]") == TRUE ){
        contrib_sp0083 <- contrib_func(model = mod_sp0083, data = subdata)
        contrib_sp0083 <- as.data.frame(t(contrib_sp0083)) %>% rownames_to_column() %>%
          rename(Variable = rowname, Contribution = '1') %>%
          mutate(Region = "ENA",
                 Species = "Sp_0083",
                 Simulation = simu[j],
                 sampling_resolution = resolution)
      }
      if( stringr::str_detect(fit_best_sp0083_ord[1,1], "[+]") == FALSE ){
        contrib_sp0083 <- NULL
      }
      
      
      #### SP 025
      # model selection
      fit_best_sp025 <- fit_all_gam(envdata = subdata, esw = 0.2, outcome = "n_sp025",
                                     predictors = variables, 
                                     rescale = F, weight_by_g_naught = 1, family = "tweedie", 
                                     max_cor = 0.5, nb_min_pred = 1, nb_max_pred = 4, complexity = 4)
      
      # compute AIC weights and likelihoods of all tested models
      fit_best_sp025_ord <- fit_best_sp025[order(fit_best_sp025$AIC),]
      aics_sp025 <- akaike.weights(fit_best_sp025_ord[,3])
      fit_best_sp025_ord$Delta.AIC <- aics_sp025$deltaAIC
      fit_best_sp025_ord$rel.likelihood <- aics_sp025$rel.LL
      fit_best_sp025_ord$Akaike.weight <- aics_sp025$weights
      fit_best_sp025_ord <- fit_best_sp025_ord %>%
        mutate(Region = "ENA", 
               Species = "Sp_025", 
               Simulation = simu[j], 
               sampling_resolution = resolution)
      
      # compute AICw of variables
      table_sp025 <- data.frame(var = NA, count = NA, percent = NA, akaike.weight = NA)
      for (i in 1:length(variables)) {
        select <- fit_best_sp025_ord[fit_best_sp025_ord$model %like% variables[i], ]
        table_sp025[i,1] <- variables[i]
        table_sp025[i,2] <- nrow(select) 
        table_sp025[i,3] <- nrow(select)/nrow(fit_best_sp025_ord)*100
        table_sp025[i,4] <- sum(select$Akaike.weight)*100
      }
      table_sp025 <- table_sp025[order(-table_sp025$akaike.weight),] %>% 
        mutate(Region = "ENA", 
               Species = "Sp_025", 
               Simulation = simu[j], 
               sampling_resolution = resolution)
      
      # fit best model
      mod_sp025 <- mgcv::gam(as.formula(fit_best_sp025_ord[1,1]), 
                              data = subdata, offset = log(2*0.2*Effort), 
                              family = tw(), method = "REML")
      
      # compute contribution of selected variables
      if( stringr::str_detect(fit_best_sp025_ord[1,1], "[+]") == TRUE ){
        contrib_sp025 <- contrib_func(model = mod_sp025, data = subdata)
        contrib_sp025 <- as.data.frame(t(contrib_sp025)) %>% rownames_to_column() %>%
          rename(Variable = rowname, Contribution = '1') %>%
          mutate(Region = "ENA",
                 Species = "Sp_025",
                 Simulation = simu[j],
                 sampling_resolution = resolution)
      }
      if( stringr::str_detect(fit_best_sp025_ord[1,1], "[+]") == FALSE ){
        contrib_sp025 <- NULL
      }
    
      #### Join results
      result_table <- rbind(fit_best_sp0083_ord[1,], fit_best_sp025_ord[1,])
      var_weight_table <- rbind(table_sp0083, table_sp025)
      contrib_table <- rbind(contrib_sp0083, contrib_sp025)
      return(list(result_table, var_weight_table, contrib_table))
  })
  # concatenate results from simulations
  result_table <- res %>% 
    purrr::map(`[[`, 1) %>% 
    data.table::rbindlist()
  var_weight_table <- res %>% 
    purrr::map(`[[`, 2) %>% 
    data.table::rbindlist()
  contrib_table <- res %>%
    purrr::map(`[[`, 3) %>%
    data.table::rbindlist()
  
  return(list(Model_summary = result_table, Variables_summary = var_weight_table, Contribution_summary = contrib_table))
}



### 2 - Computation ------------------------------------------------------------
Model_summary <- NULL
Variables_summary <- NULL
Contribution_summary <- NULL

for(i in seq_along(reso)){
  writeLines(paste("Processing", reso[i], sep = " "))
  res <- fit_all_single(data = data_ENA, variables = vars, resolution = reso[i], simu = simu)
  Model_summary <- rbind(Model_summary, res$Model_summary)
  Variables_summary <- rbind(Variables_summary, res$Variables_summary)
  Contribution_summary <- rbind(Contribution_summary, res$Contribution_summary)
  
  save.image(paste(outdir, "ENA_Model_Selection.RData", sep =""))
}


save.image(paste(outdir, "ENA_Model_Selection.RData", sep =""))

write.table(Contribution_summary, 
            paste0(outdir, "ENA_Contributions_summary.csv"),
            sep = ";", dec = ".", row.names = F)
write.table(Model_summary, 
            paste0(outdir, "ENA_Model_summary.csv"),
            sep = ";", dec = ".", row.names = F)
write.table(Variables_summary, 
            paste0(outdir, "ENA_Variables_summary.csv"),
            sep = ";", dec = ".", row.names = F)


ggplot(Contribution_summary) + geom_boxplot(aes(x = Variable, y = Contribution, fill = sampling_resolution)) + 
  facet_grid(Region~Species)

ggplot(Model_summary) + geom_boxplot(aes(y = Akaike.weight, x = Species, fill = sampling_resolution))

ggplot(Variables_summary) + geom_boxplot(aes(x = var, y = akaike.weight, fill = sampling_resolution)) + 
  facet_grid(Region~Species)

# how many times a variable is selected
count_var <- Contribution_summary %>% group_by(Region, Species, sampling_resolution, Variable) %>% summarize(count = n())
ggplot(Contribution_summary) + geom_bar(aes(x = Variable, fill = sampling_resolution), position = "dodge", stat = "count") + facet_grid(Region~Species)


