##--------------------------------------------------------------------------------------------------------
## PROJECT : The effect of spatial resolution on SDM efficiency
## SCRIPT : Plot best model curves
##
## Author : Charlotte Lambert
## Last update : July 2022
## R version 4.0.4 (2021-02-15) -- "Lost Library Book"
##--------------------------------------------------------------------------------------------------------


library(dplyr)
library(tidyverse)
library(raster)
library(terra)
library(maps)
library(maptools)
library(ggplot2)
library(ggthemes)
library(ggspatial)
library(ggpubr)
library(RColorBrewer)
library(reshape2)
library(lubridate)
library(viridis)
library(ncdf4)
library(sf)
library(spatstat)
library(spatstat.utils)
library(multidplyr)
library(mgcv)
library(data.table)
library(qpcR)
library(itsadug)

#### ----------------------- ####
#### ---- SOURCES FILES ---- ####
#### ----------------------- ####
datadir <- "~/4_Formatting/"
outdir <- "~/5_Modelling/02_Model_selection/"

seg0083_WIO <- read.table(paste(datadir, "WIO_seg_cov_0083.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Seg_0083", region = "WIO") %>% rename(effort = SegLengthKm, simulation=)
seg017_WIO <- read.table(paste(datadir, "WIO_seg_cov_017.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Seg_017", region = "WIO") %>% rename(effort = SegLengthKm)
seg025_WIO <- read.table(paste(datadir, "WIO_seg_cov_025.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Seg_025", region = "WIO") %>% rename(effort = SegLengthKm)
seg050_WIO <- read.table(paste(datadir, "WIO_seg_cov_050.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Seg_050", region = "WIO") %>% rename(effort = SegLengthKm)
seg100_WIO <- read.table(paste(datadir, "WIO_seg_cov_100.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Seg_100", region = "WIO") %>% rename(effort = SegLengthKm)

rast017_WIO <- read.table(paste(datadir, "WIO_rast_cov_017.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Rast_017", region = "WIO")
rast025_WIO <- read.table(paste(datadir, "WIO_rast_cov_025.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Rast_025", region = "WIO")
rast050_WIO <- read.table(paste(datadir, "WIO_rast_cov_050.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Rast_050", region = "WIO")
rast100_WIO <- read.table(paste(datadir, "WIO_rast_cov_100.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Rast_100", region = "WIO")

vars <- c("Bathy", "Slope", "Temp", "SSH", "NPP", "MLD", "ZEu", "CurrentSpeed", "EKE")

data_WIO <- rbind(seg0083_WIO[,c("Longitude", "Latitude", "effort", "n_sp0083", "n_sp025",
                                 "Simulation", "sampling_resolution", "region", vars)],
                  seg017_WIO[,c("Longitude", "Latitude", "effort", "n_sp0083", "n_sp025",
                                "Simulation", "sampling_resolution", "region", vars)],
                  seg025_WIO[,c("Longitude", "Latitude", "effort", "n_sp0083", "n_sp025",
                                "Simulation", "sampling_resolution", "region", vars)],
                  seg050_WIO[,c("Longitude", "Latitude", "effort", "n_sp0083", "n_sp025",
                                "Simulation", "sampling_resolution", "region", vars)],
                  seg100_WIO[,c("Longitude", "Latitude", "effort", "n_sp0083", "n_sp025",
                                "Simulation", "sampling_resolution", "region", vars)],
                  rast017_WIO[,c("Longitude", "Latitude", "effort", "n_sp0083", "n_sp025",
                                 "Simulation", "sampling_resolution", "region", vars)],
                  rast025_WIO[,c("Longitude", "Latitude", "effort", "n_sp0083", "n_sp025",
                                 "Simulation", "sampling_resolution", "region", vars)],
                  rast050_WIO[,c("Longitude", "Latitude", "effort", "n_sp0083", "n_sp025",
                                 "Simulation", "sampling_resolution", "region", vars)],
                  rast100_WIO[,c("Longitude", "Latitude", "effort", "n_sp0083", "n_sp025",
                                 "Simulation", "sampling_resolution", "region", vars)])

rm(seg0083_WIO, seg017_WIO, seg025_WIO, seg050_WIO, seg100_WIO, rast017_WIO, rast025_WIO, rast050_WIO, rast100_WIO)


Model_summary <- read.table(paste0(outdir, "WIO_Model_summary.csv"), sep = ";", dec = ".", h = T)


#### ----------------------------- ####
#### ----- EXTRACTING CURVES ----- ####
#### ----------------------------- ####

### 1 - Function ---------------------------------------------------------------
simu <- unique(data_WIO$Simulation)
reso <- unique(data_WIO$sampling_resolution)


fit_curves <- function(data, simu, resolution, variables, model_summary, species){
  data <- data %>% drop_na(all_of(variables)) %>% subset(sampling_resolution == resolution) %>% rename(Effort = effort)
  
  res <- pbapply::pblapply(seq_along(simu), function(j){
    subdata <- data %>% subset(Simulation == simu[j])
    
    formula <- model_summary %>% subset(Species == species & sampling_resolution == resolution & Simulation == simu[j]) %>% dplyr::select(all_of("model"))
    
    if( stringr::str_detect(formula[1,1], "[+]") == TRUE ){ 
      mod <- mgcv::gam(as.formula(formula[1,1]), data = subdata, offset = log(2*0.2*subdata$Effort), family = tw(), method = "REML")
    
      pred <- predict(mod, newdata = subdata, type = "terms")
      
      out <- subdata %>%
        mutate(Species = species, 
               Simulation = simu[j], 
               sampling_resolution = resolution)
      out <- cbind(out, pred)
      return(out)
    }
  })
  res <- res[!unlist(lapply(res,is.null))] # remove any null element in the list
  curves <- res %>% plyr::join_all(., type = "full",
                                   by = c("Longitude", "Latitude", "Effort", "region", "Species", "Simulation", "sampling_resolution", "n_sp0083", "n_sp025"))
  return(curves)
}


  
### 2 - Computation ------------------------------------------------------------
## Sp 0083
for(i in seq_along(reso)){ 
  writeLines(paste("Processing", reso[i], sep = " "))
  res <- fit_curves(data = data_WIO, variables = vars, model_summary = Model_summary, 
                    resolution = reso[i], simu = simu, species = "Sp_0083")
  assign(paste0("WIO_Sp0083_", reso[i]), res)
}
save.image(paste(outdir, "WIO_plot_curves.RData", sep =""))


## Sp 025
for(i in seq_along(reso)){ 
  writeLines(paste("Processing", reso[i], sep = " "))
  res <- fit_curves(data = data_WIO, variables = vars, model_summary = Model_summary, 
                    resolution = reso[i], simu = simu, species = "Sp_025")
  assign(paste0("WIO_Sp025_", reso[i]), res)
}
save.image(paste(outdir, "WIO_plot_curves.RData", sep =""))


### 3 - Plot -------------------------------------------------------------------
'%!in%' <- function(x,y)!('%in%'(x,y))

## Sp 0083
for(r in 1:length(reso)){
  curves <- get(paste0("WIO_Sp0083_", reso[r]))
  
  png(paste0(outdir, "WIO_Sp0083_BestModCurves_", reso[r], ".png"), height = 4.5, width = 25, units = "cm", res = 350)
  par(mfrow = c(1,9), mar = c(1.5,1.5,2,0.5), tcl = -0.2, mgp = c(3,0.5,0), cex = 0.6)
  for(j in 1:length(vars)){
    if( paste0("s(", vars[j], ")") %!in% colnames(curves)){
      plot.new()
      mtext(vars[j], side = 3)
    } else {
      plot(curves[,vars[j]], curves[,paste0("s(", vars[j], ")")],
         type = "n", xlab = "", ylab = "", main = "")
      mtext(vars[j], side = 3)
      for(i in 1:100){
        points(curves[which(curves$Simulation == simu[i]), vars[j]], 
               curves[which(curves$Simulation == simu[i]), paste0("s(", vars[j], ")")],
        col = "#9d7eb0", pch = 19, cex = 0.3)
      } # with lines : one by one ok ; when running the for loop, connect all line ends
      rug(data_WIO[which(data_WIO$Simulation == "Simulation 1" & 
                           data_WIO$sampling_resolution == reso[r]), 
                   vars[j]], 
          col = "black", side = 1)
    }
  }
  dev.off()
}

## Sp 025
for(r in 1:length(reso)){
  curves <- get(paste0("WIO_Sp025_", reso[r]))
  
  png(paste0(outdir, "WIO_Sp025_BestModCurves_", reso[r], ".png"), height = 4.5, width = 25, units = "cm", res = 350)
  par(mfrow = c(1,9), mar = c(1.5,1.5,2,0.5), tcl = -0.2, mgp = c(3,0.5,0), cex = 0.6)
  for(j in 1:length(vars)){
    if( paste0("s(", vars[j], ")") %!in% colnames(curves)){
      plot.new()
      mtext(vars[j], side = 3)
    } else {
      plot(curves[,vars[j]], curves[,paste0("s(", vars[j], ")")],
           type = "n", xlab = "", ylab = "", main = "")
      mtext(vars[j], side = 3)
      for(i in 1:100){
        points(curves[which(curves$Simulation == simu[i]), vars[j]], 
               curves[which(curves$Simulation == simu[i]), paste0("s(", vars[j], ")")],
               col = "#ED8080C5", pch = 19, cex = 0.3)
      } # with lines : one by one ok ; when running the for loop, connect all line ends
      rug(data_WIO[which(data_WIO$Simulation == "Simulation 1" & 
                           data_WIO$sampling_resolution == reso[r]), 
                   vars[j]], 
          col = "black", side = 1)
    }
  }
  dev.off()
}





