##--------------------------------------------------------------------------------------------------------
## PROJECT : The effect of spatial resolution on SDM efficiency
## SCRIPT : Model selection
##
## Author : Charlotte Lambert
## Last update : Apr 2022
## R version 4.0.4 (2021-02-15) -- "Lost Library Book"
##--------------------------------------------------------------------------------------------------------


library(dplyr)
library(tidyverse)
library(ggplot2)
library(ggthemes)
library(ggpubr)
library(RColorBrewer)
library(viridis)
library(sf)
library(mgcv)
library(ggmosaic)
library(ggalt)
library(grid)
library(gridExtra)
library(stringr)

#### ----------------------- ####
#### ---- SOURCES FILES ---- ####
#### ----------------------- ####
outdir <- "~/5_Modelling/02_Model_selection/"

### 1 - Open files -------------------------------------------------------------
vars <- c("Bathy", "Slope", "Temp", "SSH", "NPP", "MLD", "ZEu", "CurrentSpeed", "EKE")

contrib_ENA <- read.table(paste(outdir, "ENA_Contributions_summary.csv", sep = ""), h = T, dec = ".", sep = ";") 
contrib_WIO <- read.table(paste(outdir, "WIO_Contributions_summary.csv", sep = ""), h = T, dec = ".", sep = ";") %>%
  mutate(Region = "WIO")

model_summary_ENA <- read.table(paste(outdir, "ENA_Model_summary.csv", sep = ""), h = T, dec = ".", sep = ";")
model_summary_WIO <- read.table(paste(outdir, "WIO_Model_summary.csv", sep = ""), h = T, dec = ".", sep = ";")

var_summary_ENA <- read.table(paste(outdir, "ENA_Variables_summary.csv", sep = ""), h = T, dec = ".", sep = ";")
var_summary_WIO <- read.table(paste(outdir, "WIO_Variables_summary.csv", sep = ""), h = T, dec = ".", sep = ";")


### 2 - Format files -----------------------------------------------------------
contrib <- full_join(contrib_ENA, contrib_WIO) %>%
  mutate(Type = purrr::map(str_split(sampling_resolution, "_"), `[[`, 1) %>% unlist(),
         Resolution = purrr::map(str_split(sampling_resolution, "_"), `[[`,2) %>% unlist(),
         Species = case_when(
                      Species == "Sp_0083" ~ "Species 0.083",
                      Species == "Sp_025" ~ "Species 0.25"))

model_summary <- full_join(model_summary_ENA, model_summary_WIO) %>%
 mutate(Type = purrr::map(str_split(sampling_resolution, "_"), `[[`, 1) %>% unlist(),
         Resolution = purrr::map(str_split(sampling_resolution, "_"), `[[`,2) %>% unlist(),
         N_cov = stringr::str_count(model, "[+]"),
         Species = case_when(
           Species == "Sp_0083" ~ "Species 0.083",
           Species == "Sp_025" ~ "Species 0.25"))

var_summary <- full_join(var_summary_ENA, var_summary_WIO) %>%
  mutate(Type = purrr::map(str_split(sampling_resolution, "_"), `[[`, 1) %>% unlist(),
         Resolution = purrr::map(str_split(sampling_resolution, "_"), `[[`,2) %>% unlist(),
         Species = case_when(
           Species == "Sp_0083" ~ "Species 0.083",
           Species == "Sp_025" ~ "Species 0.25"))


 
### 3 - Number of null models as best ones --------------------------------------
model_summary %>% 
  group_by(Region, Species, Resolution, Type, N_cov) %>% 
  tally() %>% arrange(N_cov) -> model_smry_n_cov
# 53 simulations en ENA pour la sp 0083 avec modle nul (pour res rast 025)
# 82 simulations en WIO pour la sp 0083 avec modle nul (pour res rast 017)
model_summary %>% 
  group_by(Region, Species, Resolution, Type, N_cov) %>% 
  tally() %>% arrange(N_cov) %>% 
  write.table(paste0(outdir, "Number_of_variables_per_models.csv"), dec = ".", sep = ";", row.names = F)


### 4 - Differences between sampling type ---------------------------------------
dev_rast <- model_summary %>% subset(Type == "Rast") %>% 
  rename(Dev_rast = ExpDev,
         RMSE_rast = RMSE,
         AIC_rast = AIC,
         AICw_rast = Akaike.weight,
         Ncov_rast = N_cov) %>%
  select(! c("Type", "sampling_resolution", "ResDev", "NulDev", "Convergence", "model", "Delta.AIC", "rel.likelihood")) 
dev_seg <- model_summary %>% subset(Type == "Seg" & Resolution != "0083") %>% 
  rename(Dev_seg = ExpDev,
         RMSE_seg = RMSE,
         AIC_seg = AIC,
         AICw_seg = Akaike.weight,
         Ncov_seg = N_cov) %>%
  select(! c("Type", "sampling_resolution", "ResDev", "NulDev", "Convergence", "model", "Delta.AIC", "rel.likelihood")) 
dd <- left_join(dev_seg,
                dev_rast, 
                by = c("Region", "Species", "Simulation", "Resolution") ) %>%
  mutate(Diff_dev = Dev_seg - Dev_rast,
         Diff_RMSE = RMSE_seg - RMSE_rast,
         Diff_AIC = AIC_seg - AIC_rast,
         Diff_AICw = AICw_seg - AICw_rast,
         Diff_Ncov = Ncov_seg - Ncov_rast)



### 5 - Variables in selected best models -------------------------------------
# summarize
smry_contrib <- contrib %>% group_by(Region, Species, Type, Resolution, Variable) %>% 
  summarize(mean = mean(Contribution), count = n()) %>% ungroup()

# add non selected variables (Nas and 0s)
smry_contrib <- rbind(smry_contrib, 
                      # ENA sp 0083
                      data.frame(Region = "ENA", Species = "Species 0.083", Type = "Rast", Resolution = "0083", Variable = "s(ZEu)", mean = 0, count = NA),
                      data.frame(Region = "ENA", Species = "Species 0.083", Type = "Rast", Resolution = "0083", Variable = "s(Temp)", mean = 0, count = NA),
                      data.frame(Region = "ENA", Species = "Species 0.083", Type = "Rast", Resolution = "0083", Variable = "s(SSH)", mean = 0, count = NA),
                      data.frame(Region = "ENA", Species = "Species 0.083", Type = "Rast", Resolution = "025", Variable = "s(SSH)", mean = 0, count = 0),
                      data.frame(Region = "ENA", Species = "Species 0.083", Type = "Rast", Resolution = "0083", Variable = "s(Slope)", mean = 0, count = NA),
                      data.frame(Region = "ENA", Species = "Species 0.083", Type = "Rast", Resolution = "0083", Variable = "s(MLD)", mean = 0, count = NA),
                      data.frame(Region = "ENA", Species = "Species 0.083", Type = "Rast", Resolution = "017", Variable = "s(MLD)", mean = 0, count = 0),
                      data.frame(Region = "ENA", Species = "Species 0.083", Type = "Rast", Resolution = "025", Variable = "s(MLD)", mean = 0, count = 0),
                      data.frame(Region = "ENA", Species = "Species 0.083", Type = "Rast", Resolution = "050", Variable = "s(MLD)", mean = 0, count = 0),
                      data.frame(Region = "ENA", Species = "Species 0.083", Type = "Rast", Resolution = "0083", Variable = "s(EKE)", mean = 0, count = NA),
                      data.frame(Region = "ENA", Species = "Species 0.083", Type = "Rast", Resolution = "0083", Variable = "s(CurrentSpeed)", mean = 0, count = NA),
                      data.frame(Region = "ENA", Species = "Species 0.083", Type = "Rast", Resolution = "0083", Variable = "s(Bathy)", mean = 0, count = NA),
                      data.frame(Region = "ENA", Species = "Species 0.083", Type = "Rast", Resolution = "017", Variable = "s(Bathy)", mean = 0, count = 0),
                      data.frame(Region = "ENA", Species = "Species 0.083", Type = "Rast", Resolution = "025", Variable = "s(Bathy)", mean = 0, count = 0),
                      data.frame(Region = "ENA", Species = "Species 0.083", Type = "Seg", Resolution = "0083", Variable = "s(Bathy)", mean = 0, count = 0),
                      data.frame(Region = "ENA", Species = "Species 0.083", Type = "Seg", Resolution = "017", Variable = "s(Bathy)", mean = 0, count = 0),
                      data.frame(Region = "ENA", Species = "Species 0.083", Type = "Seg", Resolution = "025", Variable = "s(Bathy)", mean = 0, count = 0),
                      data.frame(Region = "ENA", Species = "Species 0.083", Type = "Seg", Resolution = "050", Variable = "s(Bathy)", mean = 0, count = 0),
                      data.frame(Region = "ENA", Species = "Species 0.083", Type = "Seg", Resolution = "100", Variable = "s(Bathy)", mean = 0, count = 0),
                      # ENA sp 025
                      data.frame(Region = "ENA", Species = "Species 0.25", Type = "Rast", Resolution = "0083", Variable = "s(ZEu)", mean = 0, count = NA),
                      data.frame(Region = "ENA", Species = "Species 0.25", Type = "Rast", Resolution = "050", Variable = "s(ZEu)", mean = 0, count = 0),
                      data.frame(Region = "ENA", Species = "Species 0.25", Type = "Rast", Resolution = "100", Variable = "s(ZEu)", mean = 0, count = 0),
                      data.frame(Region = "ENA", Species = "Species 0.25", Type = "Rast", Resolution = "0083", Variable = "s(Temp)", mean = 0, count = NA),
                      data.frame(Region = "ENA", Species = "Species 0.25", Type = "Rast", Resolution = "025", Variable = "s(SSH)", mean = 0, count = 0),
                      data.frame(Region = "ENA", Species = "Species 0.25", Type = "Rast", Resolution = "100", Variable = "s(SSH)", mean = 0, count = 0),
                      data.frame(Region = "ENA", Species = "Species 0.25", Type = "Rast", Resolution = "0083", Variable = "s(Slope)", mean = 0, count = NA),
                      data.frame(Region = "ENA", Species = "Species 0.25", Type = "Rast", Resolution = "0083", Variable = "s(NPP)", mean = 0, count = NA),
                      data.frame(Region = "ENA", Species = "Species 0.25", Type = "Rast", Resolution = "0083", Variable = "s(MLD)", mean = 0, count = NA),
                      data.frame(Region = "ENA", Species = "Species 0.25", Type = "Rast", Resolution = "0083", Variable = "s(EKE)", mean = 0, count = NA),
                      data.frame(Region = "ENA", Species = "Species 0.25", Type = "Rast", Resolution = "0083", Variable = "s(CurrentSpeed)", mean = 0, count = NA),
                      data.frame(Region = "ENA", Species = "Species 0.25", Type = "Rast", Resolution = "0083", Variable = "s(Bathy)", mean = 0, count = NA),
                      data.frame(Region = "ENA", Species = "Species 0.25", Type = "Rast", Resolution = "025", Variable = "s(Bathy)", mean = 0, count = 0),
                      # data.frame(Region = "ENA", Species = "Species 0.25", Type = "Rast", Resolution = "050", Variable = "s(Bathy)", mean = 0, count = 0),
                      data.frame(Region = "ENA", Species = "Species 0.25", Type = "Rast", Resolution = "100", Variable = "s(Bathy)", mean = 0, count = 0),
                      data.frame(Region = "ENA", Species = "Species 0.25", Type = "Seg", Resolution = "0083", Variable = "s(MLD)", mean = 0, count = 0),
                      data.frame(Region = "ENA", Species = "Species 0.25", Type = "Seg", Resolution = "100", Variable = "s(MLD)", mean = 0, count = 0),
                      # WIO sp 0083
                      data.frame(Region = "WIO", Species = "Species 0.083", Type = "Rast", Resolution = "0083", Variable = "s(ZEu)", mean = 0, count = NA),
                      data.frame(Region = "WIO", Species = "Species 0.083", Type = "Rast", Resolution = "017", Variable = "s(ZEu)", mean = 0, count = 0),
                      data.frame(Region = "WIO", Species = "Species 0.083", Type = "Rast", Resolution = "0083", Variable = "s(Temp)", mean = 0, count = NA),
                      data.frame(Region = "WIO", Species = "Species 0.083", Type = "Rast", Resolution = "017", Variable = "s(Temp)", mean = 0, count = 0),
                      data.frame(Region = "WIO", Species = "Species 0.083", Type = "Rast", Resolution = "0083", Variable = "s(SSH)", mean = 0, count = NA),
                      data.frame(Region = "WIO", Species = "Species 0.083", Type = "Rast", Resolution = "0083", Variable = "s(Slope)", mean = 0, count = NA),
                      data.frame(Region = "WIO", Species = "Species 0.083", Type = "Rast", Resolution = "0083", Variable = "s(NPP)", mean = 0, count = NA),
                      data.frame(Region = "WIO", Species = "Species 0.083", Type = "Rast", Resolution = "017", Variable = "s(NPP)", mean = 0, count = 0),
                      data.frame(Region = "WIO", Species = "Species 0.083", Type = "Rast", Resolution = "0083", Variable = "s(MLD)", mean = 0, count = NA),
                      data.frame(Region = "WIO", Species = "Species 0.083", Type = "Rast", Resolution = "100", Variable = "s(MLD)", mean = 0, count = 0),
                      data.frame(Region = "WIO", Species = "Species 0.083", Type = "Rast", Resolution = "0083", Variable = "s(EKE)", mean = 0, count = NA),
                      data.frame(Region = "WIO", Species = "Species 0.083", Type = "Rast", Resolution = "017", Variable = "s(EKE)", mean = 0, count = 0),
                      data.frame(Region = "WIO", Species = "Species 0.083", Type = "Rast", Resolution = "0083", Variable = "s(CurrentSpeed)", mean = 0, count = NA),
                      data.frame(Region = "WIO", Species = "Species 0.083", Type = "Rast", Resolution = "017", Variable = "s(CurrentSpeed)", mean = 0, count = 0),
                      data.frame(Region = "WIO", Species = "Species 0.083", Type = "Rast", Resolution = "0083", Variable = "s(Bathy)", mean = 0, count = NA),
                      data.frame(Region = "WIO", Species = "Species 0.083", Type = "Rast", Resolution = "017", Variable = "s(Bathy)", mean = 0, count = 0),
                      data.frame(Region = "WIO", Species = "Species 0.083", Type = "Rast", Resolution = "050", Variable = "s(Bathy)", mean = 0, count = 0),
                      data.frame(Region = "WIO", Species = "Species 0.083", Type = "Seg", Resolution = "100", Variable = "s(NPP)", mean = 0, count = 0),
                      data.frame(Region = "WIO", Species = "Species 0.083", Type = "Seg", Resolution = "025", Variable = "s(MLD)", mean = 0, count = 0),
                      data.frame(Region = "WIO", Species = "Species 0.083", Type = "Seg", Resolution = "100", Variable = "s(MLD)", mean = 0, count = 0),
                      # WIO sp 025
                      data.frame(Region = "WIO", Species = "Species 0.25", Type = "Rast", Resolution = "0083", Variable = "s(ZEu)", mean = 0, count = NA),
                      data.frame(Region = "WIO", Species = "Species 0.25", Type = "Rast", Resolution = "050", Variable = "s(ZEu)", mean = 0, count = 0),
                      data.frame(Region = "WIO", Species = "Species 0.25", Type = "Rast", Resolution = "0083", Variable = "s(Temp)", mean = 0, count = NA),
                      data.frame(Region = "WIO", Species = "Species 0.25", Type = "Rast", Resolution = "0083", Variable = "s(SSH)", mean = 0, count = NA),
                      data.frame(Region = "WIO", Species = "Species 0.25", Type = "Rast", Resolution = "0083", Variable = "s(Slope)", mean = 0, count = NA),
                      data.frame(Region = "WIO", Species = "Species 0.25", Type = "Rast", Resolution = "0083", Variable = "s(NPP)", mean = 0, count = NA),
                      data.frame(Region = "WIO", Species = "Species 0.25", Type = "Rast", Resolution = "0083", Variable = "s(MLD)", mean = 0, count = NA),
                      data.frame(Region = "WIO", Species = "Species 0.25", Type = "Rast", Resolution = "100", Variable = "s(MLD)", mean = 0, count = 0),
                      data.frame(Region = "WIO", Species = "Species 0.25", Type = "Rast", Resolution = "0083", Variable = "s(EKE)", mean = 0, count = NA),
                      data.frame(Region = "WIO", Species = "Species 0.25", Type = "Rast", Resolution = "0083", Variable = "s(CurrentSpeed)", mean = 0, count = NA),
                      data.frame(Region = "WIO", Species = "Species 0.25", Type = "Rast", Resolution = "0083", Variable = "s(Bathy)", mean = 0, count = NA)
)


# rename variables
smry_contrib$Variable2 <- sapply( str_split(smry_contrib$Variable, pattern = "[()]"), "[[", 2)




#### --------------- ####
#### ---- Plots ---- ####
#### --------------- ####

## 1 - Deviances ---------------------------------------------------------------
## rmse
p0 <- ggplot(model_summary) + geom_point(aes(y = RMSE, x = Resolution, color = Type), 
                                   size = 0.8, shape = 19,
                                   position = position_jitterdodge(jitter.width = 0.2, dodge.width = 0.3)) + 
  facet_grid(Species~Region) + theme_bw() + scale_color_colorblind() +
  labs(title = "C - RMSE of selected best models") 

g0 <- ggplot_gtable(ggplot_build(p0))
strip_both <- which(grepl('strip-', g0$layout$name))
fills_both <- c("#79CCC8AC", "#197f96a3", "#A181A3C5", "#ED8080C5")
k <- 1
for (i in strip_both) {
  j <- which(grepl('rect', g0$grobs[[i]]$grobs[[1]]$childrenOrder))
  g0$grobs[[i]]$grobs[[1]]$children[[j]]$gp$fill <- fills_both[k]
  k <- k+1
}
grid::grid.draw(g0)


## dev
p1 <- ggplot(model_summary) + 
  geom_point(aes(y = ExpDev, x = Resolution, color = Type), 
             size = 0.8, shape = 19,
             position = position_jitterdodge(jitter.width = 0.2, dodge.width = 0.3)) +
  facet_grid(Species~Region) + theme_bw() + scale_color_colorblind() +
  labs(title = "A - D* of selected best models", y = "Deviance") +
  theme(axis.title = element_text(size = 9),
      strip.text = element_text(face = "bold", hjust = 0, size = 10.5),
      legend.position = "right") 

g1 <- ggplot_gtable(ggplot_build(p1))
strip_both <- which(grepl('strip-', g1$layout$name))
fills_both <- c("#79CCC8AC", "#197f96a3", "#A181A3C5", "#ED8080C5")
k <- 1
for (i in strip_both) {
  j <- which(grepl('rect', g1$grobs[[i]]$grobs[[1]]$childrenOrder))
  g1$grobs[[i]]$grobs[[1]]$children[[j]]$gp$fill <- fills_both[k]
  k <- k+1
}
grid::grid.draw(g1)


## n cov
p2 <- ggplot(model_smry_n_cov) + 
  geom_point(aes(x = Resolution, y = N_cov, color = Type, size = n), 
             position = position_dodge(0.5)) +
  facet_grid(Species~Region) + theme_bw() + scale_color_colorblind()+
  labs(title = "B - Number of variables in selected best models", 
       size = "Proportion of\nmodels (%)") +
  ylab("Number of covariates") + 
  theme(axis.title = element_text(size = 9),
        strip.text = element_text(face = "bold", hjust = 0, size = 10.5),
        legend.position = "right") 

g2 <- ggplot_gtable(ggplot_build(p2) )
k <- 1
for (i in strip_both) {
  j <- which(grepl('rect', g2$grobs[[i]]$grobs[[1]]$childrenOrder))
  g2$grobs[[i]]$grobs[[1]]$children[[j]]$gp$fill <- fills_both[k]
  k <- k+1
}
grid::grid.draw(g2)

## both
ggarrange(
  as_ggplot(g1),as_ggplot(g2),as_ggplot(g0),
  ncol = 3, nrow = 1, align = "hv") + 
  bgcolor("white") + border("white")
ggsave(paste0(outdir, "0_Best_mods.png"), height = 5, width = 20, dpi = 350)  



## 2 - Deviances ---------------------------------------------------------------
model_summary <- model_summary %>% mutate(
  Resolution = case_when(
    Resolution == "0083" ~ "0.083",
    Resolution == "017" ~ "0.17",
    Resolution == "025" ~ "0.25",
    Resolution == "050" ~ "0.50",
    Resolution == "100" ~ "1.00"
  )
)
model_summary$Resolution <- factor(model_summary$Resolution, levels = c("0.083","0.17","0.25","0.50","1.00"))
# ENA
p1b <- ggplot(model_summary %>% subset(Region == "ENA")) + 
    geom_point(aes(y = ExpDev, x = N_cov, color = Type), 
               size = 0.8, shape = 19,
               position = position_jitterdodge(jitter.width = 0.2, dodge.width = 0.3)) +
    scale_color_colorblind(labels = c("Raster", "Segment")) + 
  facet_grid(Resolution~Species) + theme_bw() + ylim(c(0,100))+
  labs(title = "A - ENA", x = "Number of covariates", y = "Explained deviance (%)") +
  theme(axis.title = element_text(size = 9),
        strip.text = element_text(face = "bold", hjust = 0, size = 10.5),
        legend.position = "bottom") 

g1b <- ggplot_gtable(ggplot_build(p1b))
g1b$grobs[[26]]$grobs[[1]]$children[[1]]$gp$fill <- "#A181A3C5" # strip x 1
g1b$grobs[[27]]$grobs[[1]]$children[[1]]$gp$fill <- "#ED8080C5" # strip x 2
g1b$grobs[[28]]$grobs[[1]]$children[[1]]$gp$fill <- "white" # strip x 2
g1b$grobs[[29]]$grobs[[1]]$children[[1]]$gp$fill <- "white" # strip x 2
g1b$grobs[[30]]$grobs[[1]]$children[[1]]$gp$fill <- "white" # strip x 2
g1b$grobs[[31]]$grobs[[1]]$children[[1]]$gp$fill <- "white" # strip x 2
g1b$grobs[[32]]$grobs[[1]]$children[[1]]$gp$fill <- "white" # strip x 2
grid::grid.draw(g1b)


# WIO 
p2b <- ggplot(model_summary %>% subset(Region == "WIO")) + 
    geom_point(aes(y = ExpDev, x = N_cov, color = Type), 
               size = 0.8, shape = 19,
               position = position_jitterdodge(jitter.width = 0.2, dodge.width = 0.3))+
  scale_color_colorblind(labels = c("Raster", "Segment")) + 
  facet_grid(Resolution~Species) + theme_bw() + ylim(c(0,100))+
  labs(title = "B - WIO", x = "Number of covariates", y = "Explained deviance (%)") +
  theme(axis.title = element_text(size = 9),
        strip.text = element_text(face = "bold", hjust = 0, size = 10.5),
        legend.position = "bottom") 

g2b <- ggplot_gtable(ggplot_build(p2b))
g2b$grobs[[26]]$grobs[[1]]$children[[1]]$gp$fill <- "#A181A3C5" # strip x 1
g2b$grobs[[27]]$grobs[[1]]$children[[1]]$gp$fill <- "#ED8080C5" # strip x 2
g2b$grobs[[28]]$grobs[[1]]$children[[1]]$gp$fill <- "white" # strip x 2
g2b$grobs[[29]]$grobs[[1]]$children[[1]]$gp$fill <- "white" # strip x 2
g2b$grobs[[30]]$grobs[[1]]$children[[1]]$gp$fill <- "white" # strip x 2
g2b$grobs[[31]]$grobs[[1]]$children[[1]]$gp$fill <- "white" # strip x 2
g2b$grobs[[32]]$grobs[[1]]$children[[1]]$gp$fill <- "white" # strip x 2
grid::grid.draw(g2b)


# arrange
ggarrange(
  as_ggplot(g1b), as_ggplot(g2b),
  ncol = 2, nrow = 1, align = "hv") + 
  bgcolor("white") + border("white")
ggsave(paste(outdir, "0_Dev_vs_Ncov.png", sep = ""), height = 6, width = 8, dpi= 350)



## 2 - Number of selections ----------------------------------------------------
smry_contrib <- smry_contrib %>% mutate(
  Resolution = case_when(
    Resolution == "0083" ~ "0.083",
    Resolution == "017" ~ "0.17",
    Resolution == "025" ~ "0.25",
    Resolution == "050" ~ "0.50",
    Resolution == "100" ~ "1.00"
  )
)
smry_contrib$Resolution <- factor(smry_contrib$Resolution, levels = c("0.083","0.17","0.25","0.50","1.00"))

# set plots
p3 <- ggplot() + 
    geom_point(data = smry_contrib %>% subset(Region == "ENA" & Type == "Seg"), 
               aes( y = -count, x = Variable2, color = Type), position = position_dodge(0.5)) +
    geom_linerange(data = smry_contrib %>% subset(Region == "ENA" & Type == "Seg"), 
                   aes( ymin = 0, ymax = -count, x = Variable2, color = Type), position = position_dodge(0.5)) +
    geom_point(data = smry_contrib %>% subset(Region == "ENA" & Type == "Rast"), 
               aes( y = count, x = Variable2, color = Type), position = position_dodge(0.5)) +
    geom_linerange(data = smry_contrib %>% subset(Region == "ENA" & Type == "Rast"), 
                   aes( ymin = count, ymax = 0, x = Variable2, color = Type), position = position_dodge(0.5)) +
    facet_grid(Resolution~Species, scales = "free") + coord_flip() + 
    geom_hline(yintercept = 0, color = "black") + scale_color_colorblind(labels = c("Raster", "Segment"))+
    scale_y_continuous(labels = abs, limits = c(-100,100)) +
    theme_bw() + #scale_color_colorblind() + 
    ylab("Number of simulations") +
    labs(title = "A - ENA", x = NULL) + 
    theme(axis.title.y = element_text(size = 9), 
        axis.title.x = element_text(),
        strip.text = element_text(face = "bold", hjust = 0, size = 10.5),
        legend.position = "bottom") 

p4 <- ggplot() + 
    geom_point(data = smry_contrib %>% subset(Region == "WIO" & Type == "Seg"), 
               aes( y = -count, x = Variable2, color = Type), position = position_dodge(0.5)) +
    geom_linerange(data = smry_contrib %>% subset(Region == "WIO" & Type == "Seg"), 
                   aes( ymin = 0, ymax = -count, x = Variable2, color = Type), position = position_dodge(0.5)) +
    geom_point(data = smry_contrib %>% subset(Region == "WIO" & Type == "Rast"), 
               aes( y = count, x = Variable2, color = Type), position = position_dodge(0.5)) +
    geom_linerange(data = smry_contrib %>% subset(Region == "WIO" & Type == "Rast"), 
                   aes( ymin = count, ymax = 0, x = Variable2, color = Type), position = position_dodge(0.5)) +
    facet_grid(Resolution~Species, scales = "free") + coord_flip() + 
    geom_hline(yintercept = 0, color = "black") + scale_color_colorblind(labels = c("Raster", "Segment")) +
    scale_y_continuous(labels = abs, limits = c(-100,100)) +
    theme_bw() + #scale_color_colorblind() + 
    ylab("Number of simulations") +
    labs(title = "B - WIO", x = NULL)+ 
    theme(axis.title.y = element_text(size = 9), 
        axis.title.x = element_text(),
        strip.text = element_text(face = "bold", hjust = 0, size = 10.5),
        legend.position = "bottom") 


# change background
g3 <- ggplot_gtable(ggplot_build(p3))
g3$grobs[[26]]$grobs[[1]]$children[[1]]$gp$fill <- "#A181A3C5" # strip x 1
g3$grobs[[27]]$grobs[[1]]$children[[1]]$gp$fill <- "#ED8080C5" # strip x 2
g3$grobs[[28]]$grobs[[1]]$children[[1]]$gp$fill <- "white" # strip x 2
g3$grobs[[29]]$grobs[[1]]$children[[1]]$gp$fill <- "white" # strip x 2
g3$grobs[[30]]$grobs[[1]]$children[[1]]$gp$fill <- "white" # strip x 2
g3$grobs[[31]]$grobs[[1]]$children[[1]]$gp$fill <- "white" # strip x 2
g3$grobs[[32]]$grobs[[1]]$children[[1]]$gp$fill <- "white" # strip x 2
grid::grid.draw(g3)

g4 <- ggplot_gtable(ggplot_build(p4))
g4$grobs[[26]]$grobs[[1]]$children[[1]]$gp$fill <- "#A181A3C5" # strip x 1
g4$grobs[[27]]$grobs[[1]]$children[[1]]$gp$fill <- "#ED8080C5" # strip x 2
g4$grobs[[28]]$grobs[[1]]$children[[1]]$gp$fill <- "white" # strip x 2
g4$grobs[[29]]$grobs[[1]]$children[[1]]$gp$fill <- "white" # strip x 2
g4$grobs[[30]]$grobs[[1]]$children[[1]]$gp$fill <- "white" # strip x 2
g4$grobs[[31]]$grobs[[1]]$children[[1]]$gp$fill <- "white" # strip x 2
g4$grobs[[32]]$grobs[[1]]$children[[1]]$gp$fill <- "white" # strip x 2
grid::grid.draw(g4)


# arrange
ggarrange(
  as_ggplot(g3), as_ggplot(g4),
    ncol = 2, nrow = 1, align = "hv") + 
  bgcolor("white") + border("white")
ggsave(paste(outdir, "0_N_var.png", sep = ""), height = 9, width = 10, dpi= 350)




## 4 - Contributions -----------------------------------------------------------
p5 <- ggplot() + 
    geom_boxplot(data = contrib %>% subset(Region == "ENA" & Type == "Seg"), 
                 aes(x = Variable, y = -Contribution, color = Type)) + 
    geom_boxplot(data = contrib %>% subset(Region == "ENA" & Type == "Rast"), 
                 aes(x = Variable, y = Contribution, color = Type)) + 
    facet_grid(Resolution~Species) + theme_bw() + 
    geom_hline(yintercept = 0, color = "black") + 
    scale_color_colorblind() + labs(title = "A - ENA", x = NULL, y = "Contribution") + 
  coord_flip() +
  scale_y_continuous(labels = abs, limits = c(-1,1)) +
  theme(axis.title.y = element_text(size = 9), 
        axis.title.x = element_text(),
        strip.text = element_text(face = "bold", hjust = 0, size = 10.5),
        legend.position = "bottom") 

p6 <- ggplot() + 
    geom_boxplot(data = contrib %>% subset(Region == "WIO" & Type == "Seg"), 
                 aes(x = Variable, y = -Contribution, color = Type)) + 
    geom_boxplot(data = contrib %>% subset(Region == "WIO" & Type == "Rast"), 
                 aes(x = Variable, y = Contribution, color = Type)) + 
    facet_grid(Resolution~Species) + theme_bw() + 
    geom_hline(yintercept = 0, color = "black") + 
    scale_color_colorblind() + labs(title = "B - WIO", x = NULL, y = "Contribution") + 
  coord_flip() +
  scale_y_continuous(labels = abs, limits = c(-1,1)) +
  theme(axis.title.y = element_text(size = 9), 
        axis.title.x = element_text(),
        strip.text = element_text(face = "bold", hjust = 0, size = 10.5),
        legend.position = "bottom") 
  
  
# change background
g5 <- ggplot_gtable(ggplot_build(p5))
g5$grobs[[26]]$grobs[[1]]$children[[1]]$gp$fill <- "#A181A3C5" # strip x 1
g5$grobs[[27]]$grobs[[1]]$children[[1]]$gp$fill <- "#ED8080C5" # strip x 2
g5$grobs[[28]]$grobs[[1]]$children[[1]]$gp$fill <- "white" # strip x 2
g5$grobs[[29]]$grobs[[1]]$children[[1]]$gp$fill <- "white" # strip x 2
g5$grobs[[30]]$grobs[[1]]$children[[1]]$gp$fill <- "white" # strip x 2
g5$grobs[[31]]$grobs[[1]]$children[[1]]$gp$fill <- "white" # strip x 2
g5$grobs[[32]]$grobs[[1]]$children[[1]]$gp$fill <- "white" # strip x 2
grid::grid.draw(g5)

g6 <- ggplot_gtable(ggplot_build(p6))
g6$grobs[[26]]$grobs[[1]]$children[[1]]$gp$fill <- "#A181A3C5" # strip x 1
g6$grobs[[27]]$grobs[[1]]$children[[1]]$gp$fill <- "#ED8080C5" # strip x 2
g6$grobs[[28]]$grobs[[1]]$children[[1]]$gp$fill <- "white" # strip x 2
g6$grobs[[29]]$grobs[[1]]$children[[1]]$gp$fill <- "white" # strip x 2
g6$grobs[[30]]$grobs[[1]]$children[[1]]$gp$fill <- "white" # strip x 2
g6$grobs[[31]]$grobs[[1]]$children[[1]]$gp$fill <- "white" # strip x 2
g6$grobs[[32]]$grobs[[1]]$children[[1]]$gp$fill <- "white" # strip x 2
grid::grid.draw(g6)


# arrange
ggarrange(
  as_ggplot(g5), as_ggplot(g6),
  ncol = 2, nrow = 1, align = "hv") + 
  bgcolor("white") + border("white")
ggsave(paste(outdir, "0_Contributions.png", sep = ""), height = 9, width = 10, dpi= 350)



### 5 - Deviance by simulations ------------------------------------------------
model_summary$simu <- as.numeric(sapply(stringr::str_split(model_summary$Simulation, pattern = " "), "[[", 2))

model_summary$Type <- ifelse(model_summary$Type == "Rast", "Raster", "Segment")

p6a <- ggplot(model_summary %>% subset(Region == "ENA")) + 
    geom_point(aes(y = ExpDev, x = simu, color = Resolution), 
               size = 0.8, shape = 19,
               position = position_jitterdodge(jitter.width = 0.1, dodge.width = 0.1)) + 
    facet_grid(Type~Species) + theme_bw() + scale_color_colorblind() +
    labs(title = "A - ENA", x = "Simulation", y = "Explained deviance (%)") +
    theme(axis.title.y = element_text(size = 9), 
        axis.title.x = element_text(size = 9),
        strip.text = element_text(face = "bold", hjust = 0, size = 10.5),
        legend.position = "bottom") 

p6b <- ggplot(model_summary %>% subset(Region == "WIO")) + 
    geom_point(aes(y = ExpDev, x = simu, color = Resolution), 
               size = 0.8, shape = 19,
               position = position_jitterdodge(jitter.width = 0.1, dodge.width = 0.1)) + 
    facet_grid(Type~Species) + theme_bw() + scale_color_colorblind() +
    labs(title = "B - WIO", x = "Simulation", y = "Explained deviance (%)") +
  theme(axis.title.y = element_text(size = 9), 
        axis.title.x = element_text(size = 9),
        strip.text = element_text(face = "bold", hjust = 0, size = 10.5),
        legend.position = "bottom") 


## change background
g6a <- ggplot_gtable(ggplot_build(p6a))
strip_both <- which(grepl('strip-', g6a$layout$name))
fills_both <- c("#A181A3C5", "#ED8080C5","white", "white")
k <- 1
for (i in strip_both) {
  j <- which(grepl('rect', g6a$grobs[[i]]$grobs[[1]]$childrenOrder))
  g6a$grobs[[i]]$grobs[[1]]$children[[j]]$gp$fill <- fills_both[k]
  k <- k+1
}
grid::grid.draw(g6a)


g6b <- ggplot_gtable(ggplot_build(p6b))
strip_both <- which(grepl('strip-', g6b$layout$name))
fills_both <- c("#A181A3C5", "#ED8080C5","white", "white")
k <- 1
for (i in strip_both) {
  j <- which(grepl('rect', g6b$grobs[[i]]$grobs[[1]]$childrenOrder))
  g6b$grobs[[i]]$grobs[[1]]$children[[j]]$gp$fill <- fills_both[k]
  k <- k+1
}
grid::grid.draw(g6b)

## arrange
ggarrange(
  as_ggplot(g6a), as_ggplot(g6b),
  ncol = 2, nrow = 1, align = "hv") + 
  bgcolor("white") + border("white")
ggsave(paste0(outdir, "0_Deviances_by_simu.png"), height = 6, width = 11, dpi = 350)  



