##--------------------------------------------------------------------------------------------------------
## PROJECT : The effect of spatial resolution on SDM efficiency
## SCRIPT : Compare predictions to simulated presences
##
## Author : Charlotte Lambert
## Last update : July 2022
## R version 4.0.4 (2021-02-15) -- "Lost Library Book"
##--------------------------------------------------------------------------------------------------------


library(dplyr)
library(tidyverse)
library(ggplot2)
library(ggthemes)
library(ggpubr)
library(RColorBrewer)
library(viridis)
library(sf)
library(mgcv)
library(raster)
library(terra)

#### ----------------------- ####
#### ---- SOURCES FILES ---- ####
#### ----------------------- ####

### 1 - Open presences ---------------------------------------------------------
load("~/3_Survey_simulation/Survey_ENA_sp0083.RData")
rm(g, g1, g2, g3, dd, seg_buffer, segdata, segments_WE, simultpp, WEsp1_occ, WEsp1_suit, strata_WE, strata_WE2, smry, land_WE, bbx_WE,
   countdata, pts_det, pts_in, transects_WE, detection_process, rpoispp_custom, simul_spat, pts_simu)

land_ENA <- st_read(dsn = mapdir, layer = "Land_ENA_EPSG2154") %>%
  st_transform(st_crs("epsg:4326"))


### 2 - Open directories and vectors -------------------------------------------
outdir <- "~/5_Modelling/03_Model_prediction/"
spdir <- "~/3_Survey_simulation/"
mapdir <- "~/1_source/GIS"

simu_nbr <- 1:100
reso <- c("Seg_0083", "Seg_017", "Seg_025", "Seg_050", "Seg_100", 
          "Rast_017", "Rast_025", "Rast_050", "Rast_100")


### 3 - Open predictions ------------------------------------------------------
ENA_sp0083_pred_Seg_0083 <- stack(paste(outdir, "ENA_Seg_0083_Sp0083.envi", sep = ""))
ENA_sp0083_pred_Seg_017 <- stack(paste(outdir, "ENA_Seg_017_Sp0083.envi", sep = ""))
ENA_sp0083_pred_Seg_025 <- stack(paste(outdir, "ENA_Seg_025_Sp0083.envi", sep = ""))
ENA_sp0083_pred_Seg_050 <- stack(paste(outdir, "ENA_Seg_050_Sp0083.envi", sep = ""))
ENA_sp0083_pred_Seg_100 <- stack(paste(outdir, "ENA_Seg_100_Sp0083.envi", sep = ""))

ENA_sp0083_pred_Rast_017 <- stack(paste(outdir, "ENA_Rast_017_Sp0083.envi", sep = ""))
ENA_sp0083_pred_Rast_025 <- stack(paste(outdir, "ENA_Rast_025_Sp0083.envi", sep = ""))
ENA_sp0083_pred_Rast_050 <- stack(paste(outdir, "ENA_Rast_050_Sp0083.envi", sep = ""))
ENA_sp0083_pred_Rast_100 <- stack(paste(outdir, "ENA_Rast_100_Sp0083.envi", sep = ""))




#### ----------------------- ####
#### ---- Pred vs Prsce ---- ####
#### ----------------------- ####

### 1 - Rasterize presence to prediction resolutions ---------------------------
## convert pts_sf as dataframe to lighten the file
pts <- pts_sf %>% st_drop_geometry()

## set up function and run it
rast_psce <- function(simu, presences, reso){
  toto <- pbapply::pblapply(seq_along(simu), function(i){
    ras <- terra::rasterize(x = presences %>% subset(Simulation == simu[i]) %>% dplyr::select(all_of(c("lon", "lat"))) %>% vect(), 
                            y = rast(get(paste0("ENA_sp0083_pred_", reso))),
                            fun = sum)
    ras <- raster::raster(ras)
    names(ras) <- paste0("ENA_Sp0083_Simulation.", simu_nbr[i], "_", reso)
    return(ras)
  }) 
  toto <- raster::stack(sapply(toto, "[[", 1))
  return(toto)
}

for(i in 1:length(reso)){
  out <- rast_psce(simu, 
                   presences = pts, 
                   reso = reso[i])
  assign(paste0("ENA_sp0083_psce_", reso[i]), out)
}

## alleviate memory
rm(pts, pts_sf)


## convert to ind per km
surf_0083 <- raster::area(ENA_sp0083_pred_Seg_0083)
surf_017 <- raster::area(ENA_sp0083_pred_Seg_017)
surf_025 <- raster::area(ENA_sp0083_pred_Seg_025)
surf_050 <- raster::area(ENA_sp0083_pred_Seg_050)
surf_100 <- raster::area(ENA_sp0083_pred_Seg_100)

ENA_sp0083_psce_Seg_0083_relative <- ENA_sp0083_psce_Seg_0083/surf_0083
ENA_sp0083_psce_Seg_017_relative <- ENA_sp0083_psce_Seg_017/surf_017
ENA_sp0083_psce_Seg_025_relative <- ENA_sp0083_psce_Seg_025/surf_025
ENA_sp0083_psce_Seg_050_relative <- ENA_sp0083_psce_Seg_050/surf_050
ENA_sp0083_psce_Seg_100_relative <- ENA_sp0083_psce_Seg_100/surf_100

ENA_sp0083_psce_Rast_017_relative <- ENA_sp0083_psce_Rast_017/surf_017
ENA_sp0083_psce_Rast_025_relative <- ENA_sp0083_psce_Rast_025/surf_025
ENA_sp0083_psce_Rast_050_relative <- ENA_sp0083_psce_Rast_050/surf_050
ENA_sp0083_psce_Rast_100_relative <- ENA_sp0083_psce_Rast_100/surf_100


## put NAs to zeros
ENA_sp0083_psce_Seg_0083_relative[is.na(ENA_sp0083_psce_Seg_0083_relative)] <- 0
ENA_sp0083_psce_Seg_017_relative[is.na(ENA_sp0083_psce_Seg_017_relative)] <- 0
ENA_sp0083_psce_Seg_025_relative[is.na(ENA_sp0083_psce_Seg_025_relative)] <- 0
ENA_sp0083_psce_Seg_050_relative[is.na(ENA_sp0083_psce_Seg_050_relative)] <- 0
ENA_sp0083_psce_Seg_100_relative[is.na(ENA_sp0083_psce_Seg_100_relative)] <- 0
ENA_sp0083_psce_Rast_017_relative[is.na(ENA_sp0083_psce_Rast_017_relative)] <- 0
ENA_sp0083_psce_Rast_025_relative[is.na(ENA_sp0083_psce_Rast_025_relative)] <- 0
ENA_sp0083_psce_Rast_050_relative[is.na(ENA_sp0083_psce_Rast_050_relative)] <- 0
ENA_sp0083_psce_Rast_100_relative[is.na(ENA_sp0083_psce_Rast_100_relative)] <- 0



save.image(paste0(outdir, "ENA_prediction_assessment_Sp0083.RData"))


### 2 - Compute difference between presences and predictions -------------------
writeENVI <- function(x, f, ...) {
  raster::writeRaster(x, f, overwrite=TRUE, ...)
  cat("band names = {", paste(names(x),collapse=","), "}", "\n", file=extension(f, "hdr"), append=TRUE)
}

for(i in 1:length(reso)){
  diff <- get(paste0("ENA_sp0083_psce_", reso[i], "_relative")) - get(paste0("ENA_sp0083_pred_", reso[i]))
  assign(paste0("ENA_sp0083_diff_", reso[i]), diff)
  writeENVI(x = diff, f = paste0(outdir, "ENA_", reso[i], "_sp0083_difference.envi"))
}


par(mfrow = c(1,3))
plot(ENA_sp0083_psce_Seg_0083_relative[[1]])
plot(ENA_sp0083_diff_Seg_0083[[1]])
plot(ENA_sp0083_pred_Seg_0083[[1]])


### 3 - Compute mean difference over simulations (maps) ------------------------
smry_diff_maps <- NULL
for(i in 1:length(reso)){
  diff <- get(paste0("ENA_sp0083_diff_", reso[i]))
  
  # summarize over the stack for plotting purpose
  mean_diff <- overlay(diff, fun = "mean") # compute mean by cell over all layers
  min_diff <- overlay(diff, fun = "min")
  max_diff <- overlay(diff, fun = "max")
  sd_diff <- overlay(diff, fun = "sd")
  st <- stack(mean_diff, min_diff, max_diff, sd_diff)
  names(st) <- c("Mean", "Min", "Max", "SD")
  assign(paste0("ENA_sp0083_diff_mean_", reso[i]), st)
  writeENVI(x = diff, f = paste0(outdir, "ENA_", reso[i], "_sp0083_difference_summary.envi"))
  
  # summarize by layer
  mean_vect <- cellStats(diff, stat = "mean", na.rm = T) # compute sum on each layer
  sd_vect <- cellStats(diff, stat = "sd", na.rm = T)
  min_vect <- cellStats(diff, stat = "min", na.rm = T)
  max_vect <- cellStats(diff, stat = "max", na.rm = T)
  
  df <- data.frame(Diff_mean = mean_vect,
                   DiffSD_mean = sd_vect,
                   Resolution = reso[i],
                   Simulation = 1:100)
  smry_diff_maps <- rbind(df, smry_diff_maps)
}
smry_diff_maps <- smry_diff_maps %>% mutate(Region = "ENA", Species = "Sp0083")
write.table(x = smry_diff_maps, file = paste0(outdir, "ENA_Sp0083_difference_summary.csv"), row.names = F, dec = ",", sep = ";")

ggplot(smry_diff_maps) + geom_pointrange(aes(y = Diff_mean, 
                                             ymin = Diff_mean - DiffSD_mean, 
                                             ymax = Diff_mean + DiffSD_mean, 
                                             x = Resolution))


png("ENA_Sp0083_difference_mean.png", height = 16, width = 15, units = "cm", res = 350) 
par(mfrow = c(3,3), mar = c(0.5,0.5,2.5,5), cex = 0.8)
for(i in 1:length(reso)){
  diff <- get(paste0("ENA_sp0083_diff_mean_", reso[i]))
  plot(diff[[1]], col = viridis(20, option = "H"), axes = F)
  mtext(text = reso[i], side = 3, line = 0.4)
}
dev.off()



### 4 - Compute difference over simulations in total number of predicted ind ---
diff_table <- NULL
for(i in 1:length(reso)){
  prsce <- get(paste0("ENA_sp0083_psce_", reso[i], "_relative"))
  pred <- get(paste0("ENA_sp0083_pred_", reso[i]))
  
  prsce_sum <- cellStats(prsce, stat = "sum", na.rm = T) # compute sum on each layer
  pred_sum <- cellStats(pred, stat = "sum", na.rm = T)
  
  df <- data.frame(N_prsce = prsce_sum,
                   N_pred = pred_sum,
                   Resolution = reso[i],
                   Simulation = 1:100)
  df$Diff <- df$N_prsce - df$N_pred
  df$DiffPercent <- (df$Diff/df$N_prsce)*100
  diff_table <- rbind(df, diff_table)
}
diff_table <- diff_table  %>% mutate(Region = "ENA", Species = "Sp0083")
write.table(x = diff_table, file = paste0(outdir, "ENA_Sp0083_difference_overall.csv"), row.names = F, dec = ",", sep = ";")

ggplot(diff_table) + geom_boxplot(aes(y = DiffPercent, fill = Resolution))


### 5 - Compute correlation ----------------------------------------------------
# save as table pred and presence
df_maps <- NULL
for(i in 1:length(reso)){
  psce <- get(paste0("ENA_sp0083_psce_", reso[i], "_relative")) 
  pred <- get(paste0("ENA_sp0083_pred_", reso[i]))
  
  prsce_df <-  as.data.frame(psce, xy = T) %>% pivot_longer(cols = all_of(contains("layer")), values_to = "Presence", names_to = "Simulation")
  pred_df <-  as.data.frame(pred) %>% pivot_longer(cols = all_of(names(pred)))
  
  df <- data.frame(Presence = prsce_df,
                   Prediction = pred_df$value,
                   Resolution = reso[i])
  df_maps <- rbind(df, df_maps)
}
df_maps <- df_maps %>% mutate(Region = "ENA", Species = "Sp0083")
write.table(x = df_maps, file = paste0(outdir, "ENA_Sp0083_prsce_vs_pred.csv"), row.names = F, dec = ",", sep = ";")

# correlation
corr_table <- NULL
for(i in 1:length(reso)){
  prsce <- get(paste0("ENA_sp0083_psce_", reso[i], "_relative"))
  pred <- get(paste0("ENA_sp0083_pred_", reso[i]))
  
  ras1 <- prsce / cellStats(prsce, sum, na.rm = T)
  ras2 <- pred/ cellStats(pred, sum, na.rm = T)
  
  listcor <- pbapply::pblapply(1:100, function(j){
    tt <- Hmisc::rcorr(as.matrix(as.data.frame(ras1[[j]])), as.matrix(as.data.frame(ras2[[j]])), type = "pearson")
    tt$r[1,2]
  }) %>% unlist()
  df <- data.frame(Correlation = listcor,
                   Simulation = 1:100,
                   Resolution = reso[i],
                   Species = "Sp0083",
                   Region = "ENA")
  corr_table <- rbind(corr_table, df)
}

write.table(x = corr_table, file = paste0(outdir, "ENA_Sp0083_correlation_pred_ipp.csv"), row.names = F, dec = ",", sep = ";")

ggplot(corr_table) + geom_boxplot(aes(y = Correlation, fill = Resolution))

save.image(paste0(outdir, "ENA_prediction_assessment_Sp0083.RData"))

