##--------------------------------------------------------------------------------------------------------
## PROJECT : The effect of spatial resolution on SDM efficiency
## SCRIPT : Predict from selected models
##
## Author : Charlotte Lambert
## Last update : Apr 2022
## R version 4.0.4 (2021-02-15) -- "Lost Library Book"
##--------------------------------------------------------------------------------------------------------


library(dplyr)
library(tidyverse)
library(ggplot2)
library(ggthemes)
library(ggpubr)
library(RColorBrewer)
library(viridis)
library(sf)
library(mgcv)
library(raster)

#### ----------------------- ####
#### ---- SOURCES FILES ---- ####
#### ----------------------- ####
datadir <- "~/4_Formatting/"
moddir <- "~/5_Modelling/02_Model_selection/"
outdir <- "~/5_Modelling/03_Model_prediction/"
spdir <- "~/2_Species_Simulations/"
sourcedir <- "~/1_source/Variables/"
mapdir <- "~/1_source/GIS"

### 1 - Open effort files ------------------------------------------------------
seg0083_WIO <- read.table(paste(datadir, "WIO_seg_cov_0083.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Seg_0083", region = "ENA") %>% rename(effort = SegLengthKm)
seg017_WIO <- read.table(paste(datadir, "WIO_seg_cov_017.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Seg_017", region = "ENA") %>% rename(effort = SegLengthKm)
seg025_WIO <- read.table(paste(datadir, "WIO_seg_cov_025.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Seg_025", region = "ENA") %>% rename(effort = SegLengthKm)
seg050_WIO <- read.table(paste(datadir, "WIO_seg_cov_050.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Seg_050", region = "ENA") %>% rename(effort = SegLengthKm)
seg100_WIO <- read.table(paste(datadir, "WIO_seg_cov_100.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Seg_100", region = "ENA") %>% rename(effort = SegLengthKm)

rast017_WIO <- read.table(paste(datadir, "WIO_rast_cov_017.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Rast_017", region = "ENA")
rast025_WIO <- read.table(paste(datadir, "WIO_rast_cov_025.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Rast_025", region = "ENA")
rast050_WIO <- read.table(paste(datadir, "WIO_rast_cov_050.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Rast_050", region = "ENA")
rast100_WIO <- read.table(paste(datadir, "WIO_rast_cov_100.csv", sep = ""), dec = ".", sep = ";",h = T) %>%
  mutate(sampling_resolution = "Rast_100", region = "ENA")

vars <- c("Bathy", "Slope", "Temp", "SSH", "NPP", "MLD", "ZEu", "CurrentSpeed", "EKE")

data_WIO <- rbind(seg0083_WIO[,c("Longitude", "Latitude", "effort", "n_sp0083", "n_sp025",
                                 "Simulation", "sampling_resolution", "region", vars)],
                  seg017_WIO[,c("Longitude", "Latitude", "effort", "n_sp0083", "n_sp025",
                                "Simulation", "sampling_resolution", "region", vars)],
                  seg025_WIO[,c("Longitude", "Latitude", "effort", "n_sp0083", "n_sp025",
                                "Simulation", "sampling_resolution", "region", vars)],
                  seg050_WIO[,c("Longitude", "Latitude", "effort", "n_sp0083", "n_sp025",
                                "Simulation", "sampling_resolution", "region", vars)],
                  seg100_WIO[,c("Longitude", "Latitude", "effort", "n_sp0083", "n_sp025",
                                "Simulation", "sampling_resolution", "region", vars)],
                  rast017_WIO[,c("Longitude", "Latitude", "effort", "n_sp0083", "n_sp025",
                                 "Simulation", "sampling_resolution", "region", vars)],
                  rast025_WIO[,c("Longitude", "Latitude", "effort", "n_sp0083", "n_sp025",
                                 "Simulation", "sampling_resolution", "region", vars)],
                  rast050_WIO[,c("Longitude", "Latitude", "effort", "n_sp0083", "n_sp025",
                                 "Simulation", "sampling_resolution", "region", vars)],
                  rast100_WIO[,c("Longitude", "Latitude", "effort", "n_sp0083", "n_sp025",
                                 "Simulation", "sampling_resolution", "region", vars)])
rm(seg0083_WIO, seg017_WIO, seg025_WIO, seg050_WIO, seg100_WIO, rast017_WIO, rast025_WIO, rast050_WIO, rast100_WIO)

data_WIO <- data_WIO %>%
  mutate(Type = purrr::map(str_split(sampling_resolution, "_"), `[[`, 1) %>% unlist(),
         Resolution = purrr::map(str_split(sampling_resolution, "_"), `[[`,2) %>% unlist())



### 2 - Open model summary file ------------------------------------------------
model_summary_WIO <- read.table(paste(moddir, "WIO_Model_summary.csv", sep = ""), h = T, dec = ".", sep = ";") %>% 
  mutate(Type = purrr::map(str_split(sampling_resolution, "_"), `[[`, 1) %>% unlist(),
         Resolution = purrr::map(str_split(sampling_resolution, "_"), `[[`,2) %>% unlist())



### 3 - Open map files ---------------------------------------------------------
bbx_WIO <- st_read(dsn = mapdir, layer = "BoundingBox_WIO")
land_WIO <- st_read(dsn = mapdir, layer = "Land_WIO_EPSG2737") %>%
  st_transform(st_crs("epsg:4326"))
strata_WIO <- st_read(dsn = mapdir, layer = "Strata_WIO_clipped_EPSG2737", type = 3) %>%
  st_transform(st_crs("epsg:4326"))
transects_WIO <- st_read(dsn = mapdir, layer = "Transects_WIO_EPSG2737", type = 3)  %>%
  st_transform(st_crs("epsg:4326"))
segments_WIO <- st_read(dsn = mapdir, layer = "Segments_WIO_EPSG2737", type = 3) %>%
  rename(transect = transct, TransectLengthKm = TrnsLnK, SegLengthKm = SgLngtK, Longitude = Longitd, Latitude = Latitud) %>%
  st_transform(st_crs("epsg:4326")) 



### 4 - Open variables rasters -------------------------------------------------
stack_WIO_0083 <- stack(paste(sourcedir, "EnvironmentalVariables_WIO_0083.envi", sep = ""))
stack_WIO_017 <- stack(paste(sourcedir, "EnvironmentalVariables_WIO_017.envi", sep = ""))
stack_WIO_025 <- stack(paste(sourcedir, "EnvironmentalVariables_WIO_025.envi", sep = ""))
stack_WIO_050 <- stack(paste(sourcedir, "EnvironmentalVariables_WIO_050.envi", sep = ""))
stack_WIO_100 <- stack(paste(sourcedir, "EnvironmentalVariables_WIO_100.envi", sep = ""))



### 5 - Open species reference maps --------------------------------------------
# open
WIOsp0083_suit <- raster(paste(spdir, "Species0083_WIO_Suitability_EPSG2737.grd", sep = ""))
WIOsp0083_occ <- raster(paste(spdir, "Species0083_WIO_ProbOcc_EPSG2737.grd", sep = ""))
WIOsp0083_occ[WIOsp0083_occ < 0] <- 0

WIOsp025_suit <- raster(paste(spdir, "Species025_WIO_Suitability_EPSG2737.grd", sep = ""))
WIOsp025_occ <- raster(paste(spdir, "Species025_WIO_ProbOcc_EPSG2737.grd", sep = ""))
WIOsp025_occ[WIOsp025_occ < 0] <- 0

# transform to WGS
WIOsp0083_occ <- projectRaster(from = WIOsp0083_occ, to = stack_WIO_0083)
WIOsp025_occ <- projectRaster(from = WIOsp025_occ, to = stack_WIO_025)



#### --------------------------- ####
#### ---- Model predictions ---- ####
#### --------------------------- ####
simu <- unique(data_WIO$Simulation)
reso <- unique(data_WIO$sampling_resolution)

### 1 - functions --------------------------------------------------------------
pred_function <- function(region, simulation, resolution, data, table_model, savedir){
  # recup tableau pour prediction
  tab <- get(paste0("stack_", region, "_", purrr::map(str_split(resolution, "_"), `[[`,2) %>% unlist()))

  res <- pbapply::pblapply(seq_along(simulation), function(j){
    # original dataframe for model fitting
    subdata <- data %>% subset(sampling_resolution == resolution & 
                               Simulation == simulation[j])
    
    # selected best model
    sp0083_selected_mod <- table_model %>% 
      subset(Species == "Sp_0083" &
             sampling_resolution == resolution &
             Simulation == simulation[j])
    sp025_selected_mod <- table_model %>% 
      subset(Species == "Sp_025" &
             sampling_resolution == resolution &
             Simulation == simulation[j])
    
    
    # fit model
    sp0083_mod <- mgcv::gam(as.formula(sp0083_selected_mod[1,1]), data = subdata, offset = log(2*0.2*subdata$effort), family = tw(), method = "REML")
    sp025_mod <- mgcv::gam(as.formula(sp025_selected_mod[1,1]), data = subdata, offset = log(2*0.2*subdata$effort), family = tw(), method = "REML")
    
    # predict
    pred_sp0083 <- cbind(as.data.frame(tab, xy = T), predict(sp0083_mod, newdata = as.data.frame(tab), se.fit = T, type = "response") )
    pred_sp025 <- cbind(as.data.frame(tab, xy = T), predict(sp025_mod, newdata = as.data.frame(tab), se.fit = T, type = "response") )
    
    names(pred_sp0083)[16:17] <- c("Sp_0083_fit", "Sp_0083_sefit")
    names(pred_sp025)[16:17] <- c("Sp_025_fit", "Sp_025_sefit")

    # rasterize
    rast_sp0083 <- mask(rasterFromXYZ(pred_sp0083[, c("x", "y", "Sp_0083_fit")]), strata_WIO)
    crs(rast_sp0083) <- crs(strata_WIO)
    names(rast_sp0083) <- paste0(region, "_Sp0083_", resolution, "_", simulation[j])
      
    rast_sp025 <- mask(rasterFromXYZ(pred_sp025[, c("x", "y", "Sp_025_fit")]), strata_WIO)
    crs(rast_sp025) <- crs(strata_WIO)
    names(rast_sp025) <- paste0(region, "_Sp025_", resolution, "_", simulation[j])
    
    # save plot
    png(paste0(savedir, "/", region, "_Sp0083_", resolution, "_", simulation[j], "_SCALED.png"), height = 15, width = 15, res = 250, units = "cm", bg= NA)
    par(mar = c(0,0,0,0))
    plot(rast_sp0083 / cellStats(rast_sp0083, "max"), col = viridis(50, option = "H"), 
         legend = F, axes = F, zlim = c(0,1))
    dev.off()
    
    png(paste0(savedir, "/", region, "_Sp025_", resolution, "_", simulation[j], "_SCALED.png"), height = 15, width = 15, res = 250, units = "cm", bg= NA)
    par(mar = c(0,0,0,0))
    plot(rast_sp025 / cellStats(rast_sp025, "max"), col = viridis(50, option = "H"), 
         legend = F, axes = F, zlim = c(0,1))
    dev.off()
    
    # join results
    prediction_table <- cbind(pred_sp0083, pred_sp025[,16:17])
    prediction_table <- prediction_table %>% mutate(Region = region, sampling_resolution = resolution, Simulation = simulation[j])
    return(list(prediction_table, rast_sp0083, rast_sp025))
  })
  
  # concatenate results from simulations
  prediction_table <- res %>% 
    purrr::map(`[[`, 1) %>% 
    data.table::rbindlist()
  rast_sp0083 <- stack(sapply(res, "[[", 2))
  rast_sp025 <- stack(sapply(res, "[[", 3))
    # res %>% 
    # purrr::map(`[[`, 2) %>% 
    # stack()
  
  return(list(prediction_table = prediction_table, rast_sp0083 = rast_sp0083, rast_sp025 = rast_sp025))
}
writeENVI <- function(x, f, ...) {
  writeRaster(x, f, overwrite=TRUE, ...)
  cat("band names = {", paste(names(x),collapse=","), "}", "\n", file=extension(f, "hdr"), append=TRUE)
}


### 2 - compute ----------------------------------------------------------------
for(i in seq_along(reso)){
  res <- pred_function(data = data_WIO, region = "WIO", 
                       resolution = reso[i], simulation = simu,
                       table_model = model_summary_WIO, savedir = paste0(outdir, "WIO_predictions"))
  assign(paste0(reso[i], ".prediction_table"), res$prediction_table)
  assign(paste0(reso[i], ".rast_sp0083"), res$rast_sp0083)
  assign(paste0(reso[i], ".rast_sp025"), res$rast_sp025)
  
  save.image(paste(outdir, "WIO_Model_Prediction.RData", sep ="")) 
}


### 3 - save -------------------------------------------------------------------
for(i in seq_along(reso)){
  table <- get(paste0(reso[i], ".prediction_table"))
  stack_sp0083 <- get(paste0(reso[i], ".rast_sp0083"))
  stack_sp025 <- get(paste0(reso[i], ".rast_sp025"))
  
  write.table(x = table, file = paste0(outdir, "/WIO_", reso[i], "_predictions", ".csv"), row.names = F)
  writeENVI(x = stack_sp0083, f = paste0(outdir, "/WIO_", reso[i], "_Sp0083", ".envi"))
  writeENVI(x = stack_sp025, f = paste0(outdir, "/WIO_", reso[i], "_Sp025", ".envi"))
}


### 4 - scale raster stacks (to get all between 0 and 1) -----------------------
f1 <- function(j, mn, mx) {
  k <- j < 0
  k[is.na(k)] <- TRUE
  j[k] <- j[k] / abs(mn)
  j[!k] <- j[!k] / mx
  j
}
for(i in seq_along(reso)){
  # sp 0083
  r1 <- get(paste0(reso[i], ".rast_sp0083"))
  minv <- abs(cellStats(r1,'min'))
  maxv <- cellStats(r1,'max')
  
  r1b <- list()
  for (j in 1:nlayers(r1)) {
    r1b[[j]] <- calc(r1[[j]], fun=function(x) f1(x, minv[j], maxv[j]))
  }
  
  r1c <- stack(r1b)
  assign(paste0(reso[i], ".rast_sp0083_scaled"), r1c)
  
  # sp 0025
  r2 <- get(paste0(reso[i], ".rast_sp025"))
  minv <- abs(cellStats(r2,'min'))
  maxv <- cellStats(r2,'max')
  
  r2b <- list()
  for (j in 1:nlayers(r2)) {
    r2b[[j]] <- calc(r2[[j]], fun=function(x) f1(x, minv[j], maxv[j]))
  }
  
  r2c <- stack(r2b)
  assign(paste0(reso[i], ".rast_sp025_scaled"), r2c)
}




#### --------------------- ####
#### ---- Exploration ---- ####
#### --------------------- ####

### 1 - Across simulations variability in predictions --------------------------
smry <- NULL
for(i in seq_along(reso)){
  df <- get(paste0(reso[i], ".prediction_table")) %>% 
    group_by(Simulation) %>% 
    summarize(Sp0083_mean = mean(Sp_0083_fit, na.rm = T),
              Sp0083_var = var(Sp_0083_fit, na.rm = T),
              Sp0083_max = max(Sp_0083_fit, na.rm = T),
              Sp025_mean = mean(Sp_025_fit, na.rm = T),
              Sp025_var = var(Sp_025_fit, na.rm = T),
              Sp025_max = max(Sp_025_fit, na.rm = T)) %>%
    mutate(sampling_resolution = reso[i], Region = "ENA")
  smry <- rbind(smry, df)
}
smry_long <- pivot_longer(smry, cols = all_of(c("Sp0083_mean", "Sp0083_var", "Sp0083_max", "Sp025_mean", "Sp025_var", "Sp025_max"))) %>%
  mutate(Type = purrr::map(str_split(sampling_resolution, "_"), `[[`, 1) %>% unlist(),
         Resolution = purrr::map(str_split(sampling_resolution, "_"), `[[`,2) %>% unlist(),
         Species = purrr::map(str_split(name, "_"), `[[`, 1) %>% unlist(),
         What = purrr::map(str_split(name, "_"), `[[`,2) %>% unlist())

ggarrange(
  ggplot(smry_long %>% subset(What == "mean")) + geom_boxplot(aes(x = Resolution, y = value, color = Type)) +
    facet_wrap("Species") + scale_color_colorblind() + ylim(c(0,0.075)) + ggtitle("Mean prediction") + theme_bw()
  ,
  ggplot(smry_long %>% subset(What == "var")) + geom_boxplot(aes(x = Resolution, y = value, color = Type)) +
    facet_wrap("Species") + scale_color_colorblind() + ylim(c(0,0.025)) + ggtitle("Prediction variance") + theme_bw()
  ,
  ggplot(smry_long %>% subset(What == "max")) + geom_boxplot(aes(x = Resolution, y = value, color = Type)) +
    facet_wrap("Species") + scale_color_colorblind() + ylim(c(0,10)) + ggtitle("Maximum prediction") + theme_bw()
  , nrow = 3, ncol = 1, align = "hv"
)
ggsave(paste0(outdir, "ENA_Predictions_summary_statistics.png"), height = 10, width = 8, dpi = 350)


boxplot(cellStats(Seg_0083.rast_sp0083, "max"), ylab = "Max pred")
boxplot(cellStats(Seg_0083.rast_sp0083, "var"), ylab = "var")


### 2 - Predictions spatial variance and mean --------------------------

Seg_0083_sp0083_var <- calc(Seg_0083.rast_sp0083_scaled, fun = var, na.rm = T)
Seg_0083_sp025_var <- calc(Seg_0083.rast_sp025_scaled, fun = var, na.rm = T)

Seg_0083_sp0083_mean <- calc(Seg_0083.rast_sp0083_scaled, fun = mean, na.rm = T)
Seg_0083_sp025_mean <- calc(Seg_0083.rast_sp025_scaled, fun = mean, na.rm = T)

par(mfrow = c(2,2))
plot(Seg_0083_sp0083_mean, col = viridis(20, option = "H"))
plot(Seg_0083_sp0083_var, col = viridis(20, option = "H"))

plot(Seg_0083_sp025_mean, col = viridis(20, option = "H"))
plot(Seg_0083_sp025_var, col = viridis(20, option = "H"))


save.image(paste(outdir, "WIO_Model_Prediction.RData", sep ="")) 

