##--------------------------------------------------------------------------------------------------------
## PROJECT : The effect of spatial resolution on SDM efficiency
## SCRIPT : Prediction - plot mean and sd maps
##
## Author : Charlotte Lambert
## Last update : July 2022
## R version 4.0.4 (2021-02-15) -- "Lost Library Book"
##--------------------------------------------------------------------------------------------------------


# library(dplyr)
library(RColorBrewer)
library(viridis)
library(sf)
library(raster)
library(stringr)

#### ----------------------- ####
#### ---- SOURCES FILES ---- ####
#### ----------------------- ####
outdir <- "~/5_Modelling/03_Model_prediction/"
mapdir <- "~/1_source/GIS"


### 1 - Open files -------------------------------------------------------------
simu_nbr <- 1:100
reso <- c("Seg_0083", "Seg_017", "Seg_025", "Seg_050", "Seg_100", 
          "Rast_017", "Rast_025", "Rast_050", "Rast_100")
sp <- c("sp025", "sp0083")
region <- c("ENA", "WIO")

for(i in 1:length(reso)){
  for(j in 1:2){
    for(r in 1:2){
      ras <- stack(paste(outdir, region[r], "_", reso[i], "_", sp[j], ".envi", sep = ""))
      ras <- ras / cellStats(ras, stat = "max") # standardise chaque couche
      
      ras_m <- raster::overlay(ras, fun = "mean")
      ras_s <- raster::overlay(ras, fun = "sd")      
      
      assign(paste(region[r], sp[j], "pred", reso[i], sep = "_"), ras)
      assign(paste(region[r], sp[j], "pred", reso[i], "mean", sep = "_"), ras_m)
      assign(paste(region[r], sp[j], "pred", reso[i], "sd", sep = "_"), ras_s)
    }
  }
}

# scale to 0:1
for(i in 1:length(reso)){
  for(j in 1:2){
    for(r in 1:2){
      ras <- get(paste(region[r], sp[j], "pred", reso[i], "mean", sep = "_")) 
      ras <- ras / cellStats(ras, stat = "max")
      assign(paste(region[r], sp[j], "pred_relative", reso[i], sep = "_"), ras)
    }
  }
}



### 2 - Open map files ---------------------------------------------------------
## ENA
bbx_ENA <- st_read(dsn = mapdir, layer = "BoundingBox_ENA")
land_ENA <- st_read(dsn = mapdir, layer = "Land_ENA_EPSG2154") %>%
  st_transform(st_crs("epsg:4326")) %>% as_Spatial()
strata_ENA <- st_read(dsn = mapdir, layer = "Strata_ENA_clipped_EPSG2154", type = 3) %>%
  st_transform(st_crs("epsg:4326")) %>% as_Spatial()


## WIO
bbx_WIO <- st_read(dsn = mapdir, layer = "BoundingBox_WIO")
land_WIO <- st_read(dsn = mapdir, layer = "Land_WIO_EPSG2737") %>%
  st_transform(st_crs("epsg:4326")) %>% as_Spatial()
strata_WIO <- st_read(dsn = mapdir, layer = "Strata_WIO_clipped_EPSG2737", type = 3) %>%
  st_transform(st_crs("epsg:4326")) %>% as_Spatial()


save.image("Mean_predictions.RData")


#### ------------------- ####
#### ---- PLOT MAPS ---- ####
#### ------------------- ####


## ENA 
for(j in 1:2){
  png(paste0(outdir, "0_ENA_Mean_predictions_", sp[j], ".png"), height = 17, width = 7.7, units = "cm", res = 350)  
  par(mfrow = c(5, 2), mar = c(0.5,0.5,1,1.5), oma = c(0.5,2.5,2.5,0.5))
  # seg 0083
  plot(get(paste("ENA", sp[j], "pred_relative_Seg_0083", sep = "_")), 
       col = viridis(20, option = "H"), axes = F, cex = 0.5, zlim = c(0,1),
       addfun = c(plot(land_ENA, add = T, col = "white"), box()))
  mtext("0.083", side = 2, line = 1)
  mtext("Segment-based", side = 3, line = 1, adj = 0)
  
  # empty plot
  plot.new()
  mtext("Raster-based", side = 3, line = 1, adj = 0)
  
  # seg 017
  plot(get(paste("ENA", sp[j], "pred_relative_Seg_017", sep = "_")), 
       col = viridis(20, option = "H"), axes = F, cex = 0.5,  zlim = c(0,1),
       addfun = c(plot(land_ENA, add = T, col = "white"), box()))
  mtext("0.17", side = 2, line = 1)
  
  # rast 017
  plot(get(paste("ENA", sp[j], "pred_relative_Rast_017", sep = "_")), 
       col = viridis(20, option = "H"), axes = F, cex = 0.5,  zlim = c(0,1),
       addfun = c(plot(land_ENA, add = T, col = "white"), box()))
  
  # seg 025
  plot(get(paste("ENA", sp[j], "pred_relative_Seg_025", sep = "_")), 
       col = viridis(20, option = "H"), axes = F, cex = 0.5, zlim = c(0,1), 
       addfun = c(plot(land_ENA, add = T, col = "white"), box()))
  mtext("0.25", side = 2, line = 1)
  
  # rast 025
  plot(get(paste("ENA", sp[j], "pred_relative_Rast_025", sep = "_")), 
       col = viridis(20, option = "H"), axes = F, cex = 0.5, zlim = c(0,1), 
       addfun = c(plot(land_ENA, add = T, col = "white"), box()))
  
  # seg 050
  plot(get(paste("ENA", sp[j], "pred_relative_Seg_050", sep = "_")), 
       col = viridis(20, option = "H"), axes = F, cex = 0.5, zlim = c(0,1), 
       addfun = c(plot(land_ENA, add = T, col = "white"), box()))
  mtext("0.50", side = 2, line = 1)
  
  # rast 050
  plot(get(paste("ENA", sp[j], "pred_relative_Rast_050", sep = "_")), 
       col = viridis(20, option = "H"), axes = F, cex = 0.5, zlim = c(0,1), 
       addfun = c(plot(land_ENA, add = T, col = "white"), box()))
  
  # seg 100
  plot(get(paste("ENA", sp[j], "pred_relative_Seg_100", sep = "_")), 
       col = viridis(20, option = "H"), axes = F, cex = 0.5, zlim = c(0,1), 
       addfun = c(plot(land_ENA, add = T, col = "white"), box()))
  mtext("1.00", side = 2, line = 1)
  
  # rast 100
  plot(get(paste("ENA", sp[j], "pred_relative_Rast_100", sep = "_")), 
       col = viridis(20, option = "H"), axes = F, cex = 0.5, zlim = c(0,1), 
       addfun = c(plot(land_ENA, add = T, col = "white"), box()))

  dev.off()
}


## WIO 
for(j in 1:2){
  png(paste0(outdir, "0_WIO_Mean_predictions_", sp[j], ".png"), height = 17, width = 7.7, units = "cm", res = 350)  
  par(mfrow = c(5, 2), mar = c(0.5,0.5,1,1.5), oma = c(0.5,2.5,2.5,0.5))
  # seg 0083
  plot(get(paste("WIO", sp[j], "pred_relative_Seg_0083", sep = "_")), 
       col = viridis(20, option = "H"), axes = F, cex = 0.5, zlim = c(0,1), 
       addfun = c(plot(land_WIO, add = T, col = "white"), box()))
  mtext("0.083", side = 2, line = 1)
  mtext("Segment-based", side = 3, line = 1, adj = 0)
  
  # empty plot
  plot.new()
  mtext("Raster-based", side = 3, line = 1, adj = 0)
  
  # seg 017
  plot(get(paste("WIO", sp[j], "pred_relative_Seg_017", sep = "_")), 
       col = viridis(20, option = "H"), axes = F, cex = 0.5,  zlim = c(0,1),
       addfun = c(plot(land_WIO, add = T, col = "white"), box()))
  mtext("0.17", side = 2, line = 1)
  
  # rast 017
  plot(get(paste("WIO", sp[j], "pred_relative_Rast_017", sep = "_")), 
       col = viridis(20, option = "H"), axes = F, cex = 0.5, zlim = c(0,1), 
       addfun = c(plot(land_WIO, add = T, col = "white"), box()))
  
  # seg 025
  plot(get(paste("WIO", sp[j], "pred_relative_Seg_025", sep = "_")), 
       col = viridis(20, option = "H"), axes = F, cex = 0.5, zlim = c(0,1), 
       addfun = c(plot(land_WIO, add = T, col = "white"), box()))
  mtext("0.25", side = 2, line = 1)
  
  # rast 025
  plot(get(paste("WIO", sp[j], "pred_relative_Rast_025", sep = "_")), 
       col = viridis(20, option = "H"), axes = F, cex = 0.5, zlim = c(0,1), 
       addfun = c(plot(land_WIO, add = T, col = "white"), box()))
  
  # seg 050
  plot(get(paste("WIO", sp[j], "pred_relative_Seg_050", sep = "_")), 
       col = viridis(20, option = "H"), axes = F, cex = 0.5, zlim = c(0,1), 
       addfun = c(plot(land_WIO, add = T, col = "white"), box()))
  mtext("0.50", side = 2, line = 1)
  
  # rast 050
  plot(get(paste("WIO", sp[j], "pred_relative_Rast_050", sep = "_")), 
       col = viridis(20, option = "H"), axes = F, cex = 0.5,  zlim = c(0,1),
       addfun = c(plot(land_WIO, add = T, col = "white"), box()))
  
  # seg 100
  plot(get(paste("WIO", sp[j], "pred_relative_Seg_100", sep = "_")), 
       col = viridis(20, option = "H"), axes = F, cex = 0.5, zlim = c(0,1), 
       addfun = c(plot(land_WIO, add = T, col = "white"), box()))
  mtext("1.00", side = 2, line = 1)
  
  # rast 100
  plot(get(paste("WIO", sp[j], "pred_relative_Rast_100", sep = "_")), 
       col = viridis(20, option = "H"), axes = F, cex = 0.5,  zlim = c(0,1),
       addfun = c(plot(land_WIO, add = T, col = "white"), box()))
  
  dev.off()
}
