##--------------------------------------------------------------------------------------------------------
## PROJECT : The effect of spatial resolution on SDM efficiency
## SCRIPT : Prediction assessment
##
## Author : Charlotte Lambert
## Last update : July 2022
## R version 4.0.4 (2021-02-15) -- "Lost Library Book"
##--------------------------------------------------------------------------------------------------------


library(dplyr)
library(tidyverse)
library(ggplot2)
library(ggthemes)
library(ggpubr)
library(RColorBrewer)
library(viridis)
library(sf)
library(mgcv)
library(ggmosaic)
library(ggalt)
library(grid)
library(gridExtra)


outdir <- "~/5_Modelling/03_Model_prediction/"

### 1 - Open files -------------------------------------------------------------
ENA_sp0083_corr <- read.table(file = paste0(outdir, "ENA_Sp0083_correlation_pred_ipp.csv"), h = T, dec = ",", sep = ";") %>% 
  mutate(Region = "ENA", Species = "Sp0083")

EENA_sp025_corr <- read.table(file = paste0(outdir, "ENA_Sp025_correlation_pred_ipp.csv"), h = T, dec = ",", sep = ";") %>% 
  mutate(Region = "ENA", Species = "Sp025")

WIO_sp0083_corr <- read.table(file = paste0(outdir, "WIO_Sp0083_correlation_pred_ipp.csv"), h = T, dec = ",", sep = ";") %>% 
  mutate(Region = "WIO", Species = "Sp0083")

WIO_sp025_corr <- read.table(file = paste0(outdir, "WIO_Sp025_correlation_pred_ipp.csv"), h = T, dec = ",", sep = ";") %>% 
  mutate(Region = "WIO", Species = "Sp025")



### 2 - Format files -----------------------------------------------------------
correlation <- rbind(ENA_sp0083_corr, ENA_sp025_corr, WIO_sp0083_corr, WIO_sp025_corr)%>%
  mutate(Type = purrr::map(str_split(Resolution, "_"), `[[`, 1) %>% unlist(),
         Resolution = purrr::map(str_split(Resolution, "_"), `[[`,2) %>% unlist(),
         Simulation = paste("Simulation", Simulation, sep = " "),
         Species = case_when(
           Species == "Sp0083" ~ "Species 0.083",
           Species == "Sp025" ~ "Species 0.25"),
		  Resolution = case_when(
			Resolution == "0083" ~ "0.083",
			Resolution == "017" ~ "0.17",
			Resolution == "025" ~ "0.25",
			Resolution == "050" ~ "0.50",
			Resolution == "100" ~ "1.00")
)
correlation$Resolution <- factor(correlation$Resolution, levels = c("0.083","0.17","0.25","0.50","1.00"))

### 3 - Plot --------------------------------------------------------------------
ggplot(correlation) + 
  geom_point(aes(y = Correlation*100, x = Resolution, color = Type), 
             size = 0.8, shape = 19,
             position = position_jitterdodge(jitter.width = 0.2, dodge.width = 0.5)) + 
  facet_grid(Species~Region) + theme_bw() + 
  scale_color_colorblind(labels = c("Raster", "Segment")) + 
  labs(y = "Pearson correlation",
       title = "Correlation of true and predicted distributions",
       caption = "") +
  theme(axis.title.y = element_text(size = 11), 
        axis.title.x = element_text(size = 11),
        strip.text = element_text(face = "bold", hjust = 0, size = 10.5),
        legend.position = "bottom") 

ggsave(paste0(outdir, "0_Summary_correlation_pred_ipp.png"), height = 7, width = 7, dpi = 350)  


